@@ -1,5 +1,7 @@
 import os
 import sys
+import time
+
 import numpy as np
 import pathlib
 import pytest
@@ -17,6 +19,7 @@ from deeplake.tests.storage_fixtures import enabled_remote_storages
 from deeplake.core.storage import GCSProvider
 from deeplake.util.exceptions import (
     GroupInfoNotSupportedError,
+    IncompatibleHtypeError,
     InvalidOperationError,
     SampleAppendError,
     TensorDoesNotExistError,
@@ -40,26 +43,28 @@ from deeplake.util.exceptions import (
     InvalidDatasetNameException,
     UnsupportedParameterException,
     DynamicTensorNumpyError,
+    SampleExtendError,
 )
 from deeplake.util.path import convert_string_to_pathlib_if_needed, verify_dataset_name
+from deeplake.tests.common import requires_libdeeplake, disabale_hidden_tensors_config
 from deeplake.util.testing import assert_array_equal
 from deeplake.util.pretty_print import summary_tensor, summary_dataset
 from deeplake.util.shape_interval import ShapeInterval
-from deeplake.constants import GDRIVE_OPT, MB
-from deeplake.client.config import REPORTING_CONFIG_FILE_PATH
+from deeplake.constants import GDRIVE_OPT, MB, KB
 
 from click.testing import CliRunner
-from deeplake.cli.auth import login, logout
-from deeplake.util.bugout_reporter import feature_report_path
 from rich import print as rich_print
 from io import BytesIO
 
+import pickle
+
 # need this for 32-bit and 64-bit systems to have correct tests
 MAX_INT_DTYPE = np.int_.__name__
 MAX_FLOAT_DTYPE = np.float_.__name__
 
 
 # not using the predefined parametrizes because `hub_cloud_ds_generator` is not enabled by default
+@pytest.mark.slow
 @pytest.mark.parametrize(
     "ds_generator",
     [
@@ -88,7 +93,6 @@ def test_persist(ds_generator):
 
     ds2 = ds_generator()
 
-    ds2.storage["dataset_meta.json"] == ds_new.storage["dataset_meta.json"]
     assert len(ds2) == 4
     assert_array_equal(ds2.label.numpy(), np.array([[1], [2], [3], [4]]))
 
@@ -103,16 +107,10 @@ def test_persist_keys(local_ds_generator):
         "dataset_meta.json",
         "image/commit_diff",
         "image/tensor_meta.json",
-        "image/chunks_index/unsharded",
-        "image/tiles_index/unsharded",
         "_image_id/tensor_meta.json",
         "_image_id/commit_diff",
-        "_image_id/chunks_index/unsharded",
-        "_image_id/tiles_index/unsharded",
         "_image_shape/tensor_meta.json",
         "_image_shape/commit_diff",
-        "_image_shape/chunks_index/unsharded",
-        "_image_shape/tiles_index/unsharded",
         "version_control_info.json",
     }
 
@@ -208,24 +206,24 @@ def test_stringify(memory_ds, capsys):
     ds.summary()
     assert (
         capsys.readouterr().out
-        == "Dataset(path='mem://hub_pytest/test_api/test_stringify', tensors=['image'])\n\n tensor    htype    shape    dtype  compression\n -------  -------  -------  -------  ------- \n  image   generic  (4, 4)    None     None   \n"
+        == "Dataset(path='mem://hub_pytest/test_api/test_stringify', tensors=['image'])\n\n tensor    htype    shape    dtype  compression\n -------  -------  -------  -------  ------- \n  image   generic  (4, 4)   float64   None   \n"
     )
 
     ds[1:2].summary()
     assert (
         capsys.readouterr().out
-        == "Dataset(path='mem://hub_pytest/test_api/test_stringify', index=Index([slice(1, 2, None)]), tensors=['image'])\n\n tensor    htype    shape    dtype  compression\n -------  -------  -------  -------  ------- \n  image   generic  (1, 4)    None     None   \n"
+        == "Dataset(path='mem://hub_pytest/test_api/test_stringify', index=Index([slice(1, 2, None)]), tensors=['image'])\n\n tensor    htype    shape    dtype  compression\n -------  -------  -------  -------  ------- \n  image   generic  (1, 4)   float64   None   \n"
     )
 
     ds.image.summary()
     assert (
         capsys.readouterr().out
-        == "Tensor(key='image')\n\n  htype    shape    dtype  compression\n -------  -------  -------  ------- \n generic  (4, 4)    None     None   \n"
+        == "Tensor(key='image')\n\n  htype    shape    dtype  compression\n -------  -------  -------  ------- \n generic  (4, 4)   float64   None   \n"
     )
     ds[1:2].image.summary()
     assert (
         capsys.readouterr().out
-        == "Tensor(key='image', index=Index([slice(1, 2, None)]))\n\n  htype    shape    dtype  compression\n -------  -------  -------  ------- \n generic  (1, 4)    None     None   \n"
+        == "Tensor(key='image', index=Index([slice(1, 2, None)]))\n\n  htype    shape    dtype  compression\n -------  -------  -------  ------- \n generic  (1, 4)   float64   None   \n"
     )
 
 
@@ -237,11 +235,11 @@ def test_summary(memory_ds):
 
     assert (
         summary_dataset(ds)
-        == "\n tensor    htype    shape    dtype  compression\n -------  -------  -------  -------  ------- \n   abc    generic  (4, 4)    None     None   \n images    image    (0,)     int32    jpeg   "
+        == "\n tensor    htype    shape    dtype  compression\n -------  -------  -------  -------  ------- \n   abc    generic  (4, 4)   float64   None   \n images    image    (0,)     int32    jpeg   "
     )
     assert (
         summary_tensor(ds.abc)
-        == "\n  htype    shape    dtype  compression\n -------  -------  -------  ------- \n generic  (4, 4)    None     None   "
+        == "\n  htype    shape    dtype  compression\n -------  -------  -------  ------- \n generic  (4, 4)   float64   None   "
     )
     assert (
         summary_tensor(ds.images)
@@ -263,12 +261,12 @@ def test_view_summary(memory_ds, capsys):
         captured = capsys.readouterr().out
         assert (
             captured
-            == f"{str(ds[2500:])}\n\n tensor    htype          shape          dtype  compression\n -------  -------        -------        -------  ------- \n   abc    generic  (10000, 5, 5:10, 3)   None     None   \n"
+            == f"{str(ds[2500:])}\n\n tensor    htype          shape          dtype  compression\n -------  -------        -------        -------  ------- \n   abc    generic  (10000, 5, 5:10, 3)  float64   None   \n"
         )
 
         assert (
             summary_dataset(ds[:7500])
-            == "\n tensor    htype         shape          dtype  compression\n -------  -------       -------        -------  ------- \n   abc    generic  (7500, 5:10, 5, 3)   None     None   "
+            == "\n tensor    htype         shape          dtype  compression\n -------  -------       -------        -------  ------- \n   abc    generic  (7500, 5:10, 5, 3)  float64   None   "
         )
 
 
@@ -680,7 +678,7 @@ def test_htype(memory_ds: Dataset):
     # Along the first direcection three matrices are concatenated, the first matrix is P,
     # the second one is Tr and the third one is R
     intrinsics.append(np.zeros((3, 4, 4), dtype=np.float32))
-    embedding.append(np.random.rand((100)))
+    embedding.append(np.random.rand((100)).astype(np.float32))
 
 
 def test_dtype(memory_ds: Dataset):
@@ -746,6 +744,7 @@ def test_array_interface(memory_ds: Dataset):
     assert_array_equal(tensor.numpy(), np.concatenate([arr1, arr2]))
 
 
+@pytest.mark.slow
 def test_hub_dataset_suffix_bug(hub_cloud_ds, hub_cloud_dev_token):
     # creating dataset with similar name but some suffix removed from end
     ds = deeplake.dataset(hub_cloud_ds.path[:-1], token=hub_cloud_dev_token)
@@ -831,6 +830,28 @@ def test_like(local_path, convert_to_pathlib):
     assert len(dest_ds) == 0
 
 
+def test_inplace_like(local_ds):
+    with local_ds as ds:
+        ds.create_tensor("abc", htype="class_label")
+        ds.abc.extend([1, 0, 1, 0, 0, 1])
+        ds.abc.info.update(class_names=["a", "b"])
+
+    # reload to ensure we are not using cached data
+    ds = pickle.loads(pickle.dumps(ds))
+
+    with deeplake.like(local_ds.path, local_ds.path, overwrite=True) as ds:
+        assert ds.abc.meta.htype == "class_label"
+        assert ds.abc.info.class_names == ["a", "b"]
+
+        ds.abc.extend([1, 0, 1, 0, 0, 1])
+
+    ds = pickle.loads(pickle.dumps(ds))
+
+    with deeplake.like(ds, ds, overwrite=True) as ds:
+        assert ds.abc.meta.htype == "class_label"
+        assert ds.abc.info.class_names == ["a", "b"]
+
+
 def test_tensor_creation_fail_recovery():
     with CliRunner().isolated_filesystem():
         ds = deeplake.dataset("test")
@@ -888,6 +909,7 @@ def test_dataset_delete():
         deeplake.constants.DELETE_SAFETY_SIZE = old_size
 
 
+@pytest.mark.slow
 def test_invalid_token():
     with pytest.raises(InvalidTokenException):
         ds = deeplake.load(
@@ -905,53 +927,35 @@ def test_invalid_token():
         )
 
 
-@pytest.mark.parametrize(
-    ("ds_generator", "path", "hub_token"),
-    [
-        ("local_ds_generator", "local_path", "hub_cloud_dev_token"),
-        ("s3_ds_generator", "s3_path", "hub_cloud_dev_token"),
-        ("gcs_ds_generator", "gcs_path", "hub_cloud_dev_token"),
-        ("azure_ds_generator", "azure_path", "hub_cloud_dev_token"),
-        ("hub_cloud_ds_generator", "hub_cloud_path", "hub_cloud_dev_token"),
-    ],
-    indirect=True,
-)
-@pytest.mark.parametrize("convert_to_pathlib", [True, False])
-def test_dataset_rename(ds_generator, path, hub_token, convert_to_pathlib):
-    ds = ds_generator()
+@pytest.mark.slow
+def test_dataset_rename(hub_cloud_ds_generator, hub_cloud_path, hub_cloud_dev_token):
+    ds = hub_cloud_ds_generator()
     ds.create_tensor("abc")
     ds.abc.append([1, 2, 3, 4])
+    original_path = ds.path
 
-    new_path = "_".join([path, "renamed"])
-
-    ds.path = convert_string_to_pathlib_if_needed(ds.path, convert_to_pathlib)
-    new_path = convert_string_to_pathlib_if_needed(new_path, convert_to_pathlib)
+    new_name = f"renamed-{time.time()}"
+    new_path = original_path.rsplit("/", 1)[0] + "/" + new_name
 
     with pytest.raises(RenameError):
-        ds.rename("wrongfolder/new_ds")
+        ds.rename("hub://wrong_org/test")
 
-    if str(ds.path).startswith("hub://"):
-        with pytest.raises(BadRequestException):
-            ds.rename(ds.path)
-    else:
-        with pytest.raises(PathNotEmptyException):
-            ds.rename(ds.path)
+    ds.rename(new_name)
 
-    ds = deeplake.rename(ds.path, new_path, token=hub_token)
-    assert ds.path == str(new_path)
+    assert ds.path.endswith("/" + new_name)
     assert_array_equal(ds.abc.numpy(), np.array([[1, 2, 3, 4]]))
 
-    ds = deeplake.load(new_path, token=hub_token)
+    ds = deeplake.load(new_path, token=hub_cloud_dev_token)
     assert_array_equal(ds.abc.numpy(), np.array([[1, 2, 3, 4]]))
 
-    deeplake.delete(new_path, token=hub_token)
+    deeplake.delete(new_path, token=hub_cloud_dev_token)
 
 
 @pytest.mark.parametrize(
     "path,hub_token",
     [
         ["local_path", "hub_cloud_dev_token"],
-        ["hub_cloud_path", "hub_cloud_dev_token"],
+        pytest.param("hub_cloud_path", "hub_cloud_dev_token", marks=pytest.mark.slow),
     ],
     indirect=True,
 )
@@ -962,7 +966,6 @@ def test_dataset_deepcopy(path, hub_token, num_workers, progressbar):
     dest_path = "_".join((path, "dest1"))
 
     src_ds = deeplake.empty(src_path, overwrite=True, token=hub_token)
-    # dest_ds = deeplake.empty(dest_path, overwrite=True, token=hub_token)
 
     with src_ds:
         src_ds.info.update(key=0)
@@ -1006,7 +1009,7 @@ def test_dataset_deepcopy(path, hub_token, num_workers, progressbar):
     "path,hub_token",
     [
         ["local_path", "hub_cloud_dev_token"],
-        ["hub_cloud_path", "hub_cloud_dev_token"],
+        pytest.param("hub_cloud_path", "hub_cloud_dev_token", marks=pytest.mark.slow),
     ],
     indirect=True,
 )
@@ -1077,7 +1080,7 @@ def test_deepcopy(path, hub_token):
     "path,hub_token",
     [
         ["local_path", "hub_cloud_dev_token"],
-        ["hub_cloud_path", "hub_cloud_dev_token"],
+        pytest.param("hub_cloud_path", "hub_cloud_dev_token", marks=pytest.mark.slow),
     ],
     indirect=True,
 )
@@ -1121,13 +1124,6 @@ def test_deepcopy_errors(path, hub_token):
         )
 
 
-def test_cloud_delete_doesnt_exist(hub_cloud_path, hub_cloud_dev_token):
-    username = hub_cloud_path.split("/")[2]
-    # this dataset doesn't exist
-    new_path = f"hub://{username}/doesntexist123"
-    deeplake.delete(new_path, token=hub_cloud_dev_token, force=True)
-
-
 def test_invalid_tensor_name(memory_ds):
     with pytest.raises(InvalidTensorNameError):
         memory_ds.create_tensor("group/version_state")
@@ -1164,6 +1160,7 @@ def test_compressions_list():
         "png",
         "ppm",
         "sgi",
+        "stl",
         "tga",
         "tiff",
         "wav",
@@ -1198,6 +1195,7 @@ def test_htypes_list():
         "point_cloud",
         "polygon",
         "segment_mask",
+        "tag",
         "text",
         "video",
     ]
@@ -1376,6 +1374,7 @@ def test_vc_bug(local_ds_generator):
 @pytest.mark.skipif(
     os.name == "nt" and sys.version_info < (3, 7), reason="requires python 3.7 or above"
 )
+@pytest.mark.slow
 def test_tobytes(memory_ds, compressed_image_paths, audio_paths):
     ds = memory_ds
     ds.create_tensor("image", sample_compression="jpeg")
@@ -1393,6 +1392,7 @@ def test_tobytes(memory_ds, compressed_image_paths, audio_paths):
         assert ds.audio[i].tobytes() == audio_bytes
 
 
+@pytest.mark.slow
 def test_tobytes_link(memory_ds):
     with memory_ds as ds:
         ds.create_tensor("images", htype="link[image]", sample_compression="jpg")
@@ -1497,7 +1497,9 @@ def test_no_view(memory_ds):
 @pytest.mark.parametrize(
     "y_args", [{}, {"sample_compression": "lz4"}, {"chunk_compression": "lz4"}]
 )
-@pytest.mark.parametrize("x_size", [5, (32 * 1000)])
+@pytest.mark.parametrize(
+    "x_size", [5, pytest.param((32 * 1000), marks=pytest.mark.slow)]
+)
 @pytest.mark.parametrize("htype", ["generic", "sequence"])
 def test_ds_append(memory_ds, x_args, y_args, x_size, htype):
     ds = memory_ds
@@ -1535,6 +1537,15 @@ def test_ds_append(memory_ds, x_args, y_args, x_size, htype):
     assert len(ds) == 4
 
 
+def test_getting_root_tensor():
+    ds = deeplake.dataset("mem://x")
+    ds.create_tensor("/t")
+    _ = ds["/t"]
+
+    with pytest.raises(TensorDoesNotExistError):
+        t = ds["/ttt"]
+
+
 def test_ds_append_with_ds_view():
     ds1 = deeplake.dataset("mem://x")
     ds2 = deeplake.dataset("mem://y")
@@ -1565,6 +1576,7 @@ def test_ds_extend():
     assert_array_equal(ds1.y, ds2.y)
 
 
+@pytest.mark.slow
 @pytest.mark.parametrize(
     "src_args", [{}, {"sample_compression": "png"}, {"chunk_compression": "png"}]
 )
@@ -1645,6 +1657,15 @@ def test_auto_htype(memory_ds):
     assert ds.f.htype == "json"
 
 
+def test_datatype_conversions(memory_ds):
+    ds = memory_ds
+    ds.create_tensor("uuid", dtype=np.int32)
+
+    with pytest.raises(SampleAppendError) as e:
+        ds.uuid.append("asdasd")
+
+
+@pytest.mark.slow
 @pytest.mark.parametrize(
     "args", [{}, {"sample_compression": "lz4"}, {"chunk_compression": "lz4"}]
 )
@@ -1700,7 +1721,7 @@ def test_hub_remote_read_images(storage, memory_ds, color_image_paths, gdrive_cr
 
 def test_hub_remote_read_gdrive_root(request, memory_ds, gdrive_creds):
     if not is_opt_true(request, GDRIVE_OPT):
-        pytest.skip()
+        pytest.skip(f"{GDRIVE_OPT} flag not set")
     memory_ds.create_tensor("images", htype="image", sample_compression="jpg")
     memory_ds.images.append(deeplake.read("gdrive://cat.jpeg", creds=gdrive_creds))
     assert memory_ds.images[0].shape == (900, 900, 3)
@@ -1727,6 +1748,7 @@ def test_hub_remote_read_videos(storage, memory_ds):
         assert memory_ds.videos[1].shape == (361, 720, 1280, 3)
 
 
+@pytest.mark.slow
 @pytest.mark.parametrize("aslist", (True, False))
 @pytest.mark.parametrize(
     "args", [{}, {"sample_compression": "png"}, {"chunk_compression": "png"}]
@@ -1760,6 +1782,7 @@ def test_sequence_htype(memory_ds, aslist, args, idx):
 
 
 @pytest.mark.parametrize("shape", [(13, 17, 3), (1007, 3001, 3)])
+@pytest.mark.slow
 def test_sequence_htype_with_hub_read(local_ds, shape, compressed_image_paths):
     ds = local_ds
     imgs = list(map(deeplake.read, compressed_image_paths["jpeg"][:3]))
@@ -1899,9 +1922,24 @@ def test_dataset_copy(
     ("ds_generator", "path", "hub_token"),
     [
         ("local_ds_generator", "local_path", "hub_cloud_dev_token"),
-        ("s3_ds_generator", "s3_path", "hub_cloud_dev_token"),
-        ("gcs_ds_generator", "gcs_path", "hub_cloud_dev_token"),
-        ("hub_cloud_ds_generator", "hub_cloud_path", "hub_cloud_dev_token"),
+        pytest.param(
+            "s3_ds_generator",
+            "s3_path",
+            "hub_cloud_dev_token",
+            marks=pytest.mark.slow,
+        ),
+        pytest.param(
+            "gcs_ds_generator",
+            "gcs_path",
+            "hub_cloud_dev_token",
+            marks=pytest.mark.slow,
+        ),
+        pytest.param(
+            "hub_cloud_ds_generator",
+            "hub_cloud_path",
+            "hub_cloud_dev_token",
+            marks=pytest.mark.slow,
+        ),
     ],
     indirect=True,
 )
@@ -1913,15 +1951,7 @@ def test_hub_exists(ds_generator, path, hub_token, convert_to_pathlib):
     assert deeplake.exists(f"{path}_does_not_exist", token=hub_token) == False
 
 
-def test_pyav_not_installed(local_ds, video_paths):
-    pyav_installed = deeplake.core.compression._PYAV_INSTALLED
-    deeplake.core.compression._PYAV_INSTALLED = False
-    local_ds.create_tensor("videos", htype="video", sample_compression="mp4")
-    with pytest.raises(SampleAppendError):
-        local_ds.videos.append(deeplake.read(video_paths["mp4"][0]))
-    deeplake.core.compression._PYAV_INSTALLED = pyav_installed
-
-
+@pytest.mark.slow
 def test_partial_read_then_write(s3_ds_generator):
     ds = s3_ds_generator()
     with ds:
@@ -1948,6 +1978,36 @@ def test_exist_ok(local_ds):
         ds.create_group("grp", exist_ok=True)
 
 
+def test_exist_ok_htype(local_ds):
+    with local_ds as ds:
+        ds.create_tensor("test1", htype="text")
+        with pytest.raises(TensorAlreadyExistsError):
+            ds.create_tensor("test1")
+        ds.create_tensor("test1", htype="text", exist_ok=True)
+
+        ds.create_tensor("test2", htype="bbox")
+        with pytest.raises(TensorAlreadyExistsError):
+            ds.create_tensor("test2")
+        ds.create_tensor("test2", htype="bbox", exist_ok=True)
+
+        ds.create_tensor("test3", htype="polygon")
+        with pytest.raises(TensorAlreadyExistsError):
+            ds.create_tensor("test3")
+        ds.create_tensor("test3", htype="polygon", exist_ok=True)
+
+        ds.create_tensor("test4", htype="image", sample_compression="jpg")
+        with pytest.raises(TensorAlreadyExistsError):
+            ds.create_tensor("test4")
+        ds.create_tensor(
+            "test4", htype="image", sample_compression="jpg", exist_ok=True
+        )
+
+        ds.create_tensor("test5", htype="class_label")
+        with pytest.raises(TensorAlreadyExistsError):
+            ds.create_tensor("test5")
+        ds.create_tensor("test5", htype="class_label", exist_ok=True)
+
+
 def verify_label_data(ds):
     text_labels = [
         ["airplane"],
@@ -2080,6 +2140,7 @@ def test_text_label(local_ds_generator):
 
 @pytest.mark.parametrize("scheduler", ["threaded", "processed"])
 @pytest.mark.parametrize("num_workers", [0, 2])
+@pytest.mark.flaky(retry_count=3)
 def test_text_labels_transform(local_ds_generator, scheduler, num_workers):
     with local_ds_generator() as ds:
         ds.create_tensor("labels", htype="class_label")
@@ -2128,6 +2189,7 @@ def test_text_labels_transform(local_ds_generator, scheduler, num_workers):
             assert_array_equal(a, e)
 
 
+@pytest.mark.slow
 @pytest.mark.parametrize("num_workers", [0, 2])
 def test_transform_upload_fail(local_ds_generator, num_workers):
     @deeplake.compute
@@ -2192,6 +2254,7 @@ def test_ignore_temp_tensors(local_path):
         assert ds.__temptensor[0].numpy() == 123
 
 
+@pytest.mark.slow
 def test_empty_sample_partial_read(s3_ds):
     with s3_ds as ds:
         ds.create_tensor("xyz")
@@ -2254,20 +2317,6 @@ def test_uneven_iteration(memory_ds):
             np.testing.assert_equal(y, target_y)
 
 
-def token_permission_error_check(
-    username,
-    password,
-    runner,
-):
-    result = runner.invoke(login, f"-u {username} -p {password}")
-    with pytest.raises(TokenPermissionError):
-        deeplake.empty("hub://activeloop-test/sohas-weapons-train")
-
-    with pytest.raises(TokenPermissionError):
-        ds = deeplake.load("hub://activeloop/fake-path")
-    runner.invoke(logout)
-
-
 def invalid_token_exception_check():
     with pytest.raises(InvalidTokenException):
         ds = deeplake.empty("hub://adilkhan/demo", token="invalid_token")
@@ -2285,29 +2334,6 @@ def user_not_logged_in_exception_check(runner):
         ds = deeplake.empty("hub://adilkhan/demo")
 
 
-def dataset_handler_error_check(runner, username, password):
-    result = runner.invoke(login, f"-u {username} -p {password}")
-    with pytest.raises(DatasetHandlerError):
-        ds = deeplake.load(f"hub://{username}/wrong-path")
-    runner.invoke(logout)
-
-
-def test_hub_related_permission_exceptions(
-    hub_cloud_dev_credentials, hub_cloud_dev_token, hub_dev_token
-):
-    username, password = hub_cloud_dev_credentials
-    runner = CliRunner()
-
-    token_permission_error_check(
-        username,
-        password,
-        runner,
-    )
-    invalid_token_exception_check()
-    user_not_logged_in_exception_check(runner)
-    dataset_handler_error_check(runner, username, password)
-
-
 def test_incompat_dtype_msg(local_ds, capsys):
     local_ds.create_tensor("abc", dtype="uint32")
     with pytest.raises(SampleAppendError):
@@ -2371,6 +2397,7 @@ def test_columnar_views(memory_ds):
     assert view.group_index == "a"
 
 
+@pytest.mark.slow
 @pytest.mark.parametrize("verify", [True, False])
 def test_bad_link(local_ds_generator, verify):
     with local_ds_generator() as ds:
@@ -2477,6 +2504,8 @@ def test_random_split_views(local_ds):
             assert len(test) == 2
 
 
+@pytest.mark.flaky
+@pytest.mark.slow
 def test_invalid_ds_name():
     with pytest.raises(InvalidDatasetNameException):
         deeplake.dataset("folder/datasets/dataset name *")
@@ -2489,9 +2518,6 @@ def test_invalid_ds_name():
     with pytest.raises(InvalidDatasetNameException):
         ds = deeplake.empty("hub://test/ Mnist123")
 
-    with pytest.raises(InvalidDatasetNameException):
-        ds = deeplake.like("hub://test/Mnist123 ", "hub://activeloop/mnist-train")
-
     with pytest.raises(InvalidDatasetNameException):
         ds = deeplake.deepcopy(
             "hub://activeloop/mnist-train", "hub://activeloop/mnist$train"
@@ -2519,6 +2545,31 @@ def test_pickle_bug(local_ds):
     )
 
 
+@requires_libdeeplake
+def test_max_view_with_query(local_auth_ds):
+    import random
+
+    with local_auth_ds as ds:
+        ds.create_tensor(
+            "label1", htype="generic", dtype=np.int32, **disabale_hidden_tensors_config
+        )
+        ds.create_tensor(
+            "label2", htype="generic", dtype=np.int32, **disabale_hidden_tensors_config
+        )
+
+        for _ in range(1000):
+            ds.label1.append(int(100 * random.uniform(0.0, 1.0)))
+
+        for _ in range(100):
+            ds.label2.append(int(100 * random.uniform(0.0, 1.0)))
+
+    view = local_auth_ds.query("select label1, label2 LIMIT 200")
+    view_max = view.max_view
+
+    labels2 = view_max.label2.numpy(aslist=True)
+    assert labels2[150] == None
+
+
 def test_max_view(memory_ds):
     with memory_ds as ds:
         ds.create_tensor("abc")
@@ -2608,12 +2659,12 @@ def test_tensor_dtype_bug(local_path):
         ds.abc.append(deeplake.link(f"{data_path}/standard.nii.gz"))
 
     assert ds.abc[0].numpy().shape == (4, 5, 7)
-    assert ds.abc.dtype == np.dtype("<U1")
+    assert ds.abc.dtype == np.dtype("uint8")
 
     ds2 = ds.copy(f"{local_path}_2", overwrite=True)
 
     assert ds2.abc[0].numpy().shape == (4, 5, 7)
-    assert ds2.abc.dtype == np.dtype("<U1")
+    assert ds2.abc.dtype == np.dtype("uint8")
 
 
 def test_iterate_with_groups(memory_ds):
@@ -2757,13 +2808,13 @@ def test_slice_shape_interval(memory_ds):
 
 
 def test_non_local_org_id():
-    with pytest.raises(ValueError):
+    with pytest.raises(InvalidTokenException):
         ds = deeplake.dataset("hub://test/test_dataset", org_id="test")
 
-    with pytest.raises(ValueError):
+    with pytest.raises(InvalidTokenException):
         ds = deeplake.empty("hub://test/test_dataset", org_id="test")
 
-    with pytest.raises(ValueError):
+    with pytest.raises(InvalidTokenException):
         ds = deeplake.load("hub://test/test_dataset", org_id="test")
 
     with pytest.raises(ValueError):
@@ -2773,3 +2824,250 @@ def test_non_local_org_id():
 def test_azure_bad_path():
     with pytest.raises(ValueError):
         ds = deeplake.empty("az://storage_account")
+
+
+def test_dataset_extend_error_suggestion(local_ds):
+    with local_ds as ds:
+        ds.create_tensor("abc")
+
+    with pytest.raises(SampleExtendError) as e:
+        ds.extend({"abc": [1, 2, 3, 4, "abcd", 5]})
+
+    assert (
+        "If you wish to skip the samples that cause errors,"
+        " please specify `ignore_errors=True`."
+    ) in str(e)
+
+
+@pytest.mark.flaky(retry_count=3)
+def test_extend_rollbacks(local_ds, lfpw_links):
+    with local_ds as ds:
+        ds.create_tensor("images", htype="image", sample_compression="jpg")
+        ds.extend(
+            {"images": [deeplake.read(link) for link in lfpw_links]},
+            ignore_errors=True,
+        )
+
+    # Commit should work
+    ds.commit()
+
+
+@pytest.mark.slow
+@pytest.mark.parametrize(
+    "compression_args",
+    [
+        {"sample_compression": None},
+        {"sample_compression": "jpg"},
+        {"chunk_compression": "jpg"},
+    ],
+)
+def test_tensor_extend_ignore(local_ds, lfpw_links, compression_args):
+    with local_ds as ds:
+        ds.create_tensor("images", htype="image", **compression_args)
+        ds.create_tensor(
+            "tiled_images",
+            htype="image",
+            tiling_threshold=1 * KB,
+            max_chunk_size=1 * KB,
+            **compression_args,
+        )
+        ds.create_tensor("seq_images", htype="sequence[image]", **compression_args)
+        ds.create_tensor("link_images", htype="link[image]", **compression_args)
+
+    images = [deeplake.read(link) for link in lfpw_links]
+    ds.images.extend(images, ignore_errors=True)
+    ds.tiled_images.extend(images, ignore_errors=True)
+
+    seqs = [
+        list(map(deeplake.read, lfpw_links[i : i + 2]))
+        for i in range(0, len(lfpw_links), 2)
+    ]
+    ds.seq_images.extend(seqs, ignore_errors=True)
+
+    links = [deeplake.link(link) for link in lfpw_links]
+    ds.link_images.extend(links, ignore_errors=True)
+
+    # Commit should work
+    ds.commit()
+
+
+def test_change_htype(local_ds_generator):
+    with local_ds_generator() as ds:
+        ds.create_tensor("images", sample_compression="jpg")
+        ds.images.extend(np.random.randint(0, 256, (10, 10, 3), dtype=np.uint8))
+
+        ds.create_tensor("labels")
+        ds.labels.extend([1, 2, 3, 4, 5, 6, 7, 8, 9, 10])
+
+        ds.create_tensor("boxes")
+        ds.boxes.extend(np.random.randn(10, 5, 4))
+
+        ds.create_tensor("boxes_3d")
+        ds.boxes_3d.extend(np.random.randn(10, 5, 8))
+
+        ds.create_tensor("embeddings")
+        ds.embeddings.extend(np.random.randn(10, 1536).astype(np.float32))
+
+        mask = np.zeros((10, 100, 100, 5), dtype=bool)
+        mask[:, :, :512, 1] = 1
+        ds.create_tensor("masks")
+        ds.masks.extend(mask)
+        ds.create_tensor("image_masks", htype="image", sample_compression=None)
+        ds.image_masks.extend(mask)
+
+        ds.create_tensor("keypoints")
+        ds.keypoints.extend(np.zeros((10, 9, 5)))
+
+        ds.create_tensor("points")
+        ds.points.extend(np.zeros((10, 5, 3)))
+
+    ds.images.htype = "image"
+    ds.labels.htype = "class_label"
+    ds.boxes.htype = "bbox"
+    ds.boxes_3d.htype = "bbox.3d"
+    ds.embeddings.htype = "embedding"
+    ds.masks.htype = "binary_mask"
+    ds.image_masks.htype = "binary_mask"
+    ds.keypoints.htype = "keypoints_coco"
+    ds.points.htype = "point"
+
+    with local_ds_generator() as ds:
+        assert ds.images.htype == "image"
+        assert ds.labels.htype == "class_label"
+        assert ds.boxes.htype == "bbox"
+        assert ds.boxes_3d.htype == "bbox.3d"
+        assert ds.embeddings.htype == "embedding"
+        assert ds.masks.htype == "binary_mask"
+        assert ds.image_masks.htype == "binary_mask"
+        assert ds.keypoints.htype == "keypoints_coco"
+        assert ds.points.htype == "point"
+
+
+def test_change_htype_fail(local_ds_generator):
+    with local_ds_generator() as ds:
+        ds.create_tensor("images")
+        ds.images.extend(np.zeros((10, 5, 5, 5, 5)))
+        with pytest.raises(IncompatibleHtypeError):
+            ds.images.htype = "image"
+
+        ds.create_tensor("images2")
+        ds.images2.extend(np.zeros((10, 5, 5, 6)))
+        with pytest.raises(IncompatibleHtypeError):
+            ds.images2.htype = "image"
+
+        ds.create_tensor("labels")
+        ds.labels.extend(np.ones((10, 5, 5)))
+        with pytest.raises(IncompatibleHtypeError):
+            ds.labels.htype = "class_label"
+
+        ds.create_tensor("boxes")
+        ds.boxes.extend(np.zeros((10, 5, 5, 2)))
+        with pytest.raises(IncompatibleHtypeError):
+            ds.boxes.htype = "bbox"
+        with pytest.raises(IncompatibleHtypeError):
+            ds.boxes.htype = "bbox.3d"
+
+        ds.create_tensor("boxes2")
+        ds.boxes2.extend(np.zeros((10, 5, 5)))
+        with pytest.raises(IncompatibleHtypeError):
+            ds.boxes2.htype = "bbox"
+        with pytest.raises(IncompatibleHtypeError):
+            ds.boxes2.htype = "bbox.3d"
+
+        # bad dtype
+        ds.create_tensor("embeddings")
+        ds.embeddings.extend(np.zeros((1, 1536)))
+        with pytest.raises(IncompatibleHtypeError):
+            ds.embeddings.htype = "embedding"
+
+        ds.create_tensor("masks")
+        ds.masks.extend(np.zeros((10, 5, 5, 5, 5)))
+        with pytest.raises(IncompatibleHtypeError):
+            ds.masks.htype = "binary_mask"
+
+        ds.create_tensor("keypoints")
+        ds.keypoints.extend(np.zeros((10, 5, 5, 5)))
+        with pytest.raises(IncompatibleHtypeError):
+            ds.keypoints.htype = "keypoints_coco"
+
+        ds.create_tensor("keypoints2")
+        ds.keypoints2.extend(np.zeros((10, 10, 5)))
+        with pytest.raises(IncompatibleHtypeError):
+            ds.keypoints2.htype = "keypoints_coco"
+
+        ds.create_tensor("points")
+        ds.points.extend(np.zeros((10, 5, 5, 5)))
+        with pytest.raises(IncompatibleHtypeError):
+            ds.points.htype = "point"
+
+        ds.create_tensor("points2")
+        ds.points2.extend(np.zeros((10, 5, 5)))
+        with pytest.raises(IncompatibleHtypeError):
+            ds.points2.htype = "point"
+
+        with pytest.raises(ValueError):
+            ds.images.htype = "link[image]"
+
+        with pytest.raises(ValueError):
+            ds.images.htype = "sequence[image]"
+
+        ds.create_tensor("boxes3", htype="bbox")
+        ds.boxes3.extend(np.zeros((10, 5, 4), dtype=np.float32))
+        with pytest.raises(NotImplementedError):
+            ds.boxes3.htype = "embedding"
+
+        with pytest.raises(NotImplementedError):
+            ds.images.htype = "text"
+
+        ds.create_tensor("images3", htype="image", sample_compression="jpg")
+        with pytest.raises(UnsupportedCompressionError):
+            ds.images3.htype = "embedding"
+
+
+def test_append_non_uint8_to_image(local_ds):
+    with local_ds as ds:
+        ds.create_tensor("images", htype="image", sample_compression="png")
+        ds.images.append(np.zeros((40, 40, 1), dtype=np.int16))
+        ds.images.append(np.zeros((40, 40, 1), dtype=np.uint8))
+
+    assert ds.images.dtype.name == "int16"
+
+
+@pytest.mark.slow
+def test_create_and_load_with_managed_credentials(
+    hub_cloud_path: str, hub_cloud_dev_token
+):
+    old_environ = dict(os.environ)
+    os.environ.pop("AWS_ACCESS_KEY_ID", None)
+    os.environ.pop("AWS_SECRET_ACCESS_KEY", None)
+    os.environ.pop("AWS_SESSION_TOKEN", None)
+
+    try:
+        dir_name = hub_cloud_path.rsplit("/", 1)[1]
+        ds = deeplake.empty(
+            f"s3://deeplake-tests/{dir_name}",
+            creds={"creds_key": "aws_creds"},
+            org_id="testingacc2",
+            token=hub_cloud_dev_token,
+        )
+        ds.create_tensor("id", htype="text")
+
+        assert ds.path == f"s3://deeplake-tests/{dir_name}"
+
+        ds = deeplake.load(
+            f"s3://deeplake-tests/{dir_name}",
+            creds={"creds_key": "aws_creds"},
+            org_id="testingacc2",
+            token=hub_cloud_dev_token,
+        )
+        assert "id" in ds.tensors
+
+        deeplake.delete(
+            f"s3://deeplake-tests/{dir_name}",
+            creds={"creds_key": "aws_creds"},
+            org_id="testingacc2",
+            token=hub_cloud_dev_token,
+        )
+    finally:
+        os.environ.clear()
+        os.environ.update(old_environ)
