@@ -1,6 +1,9 @@
+import pytest
 import deeplake
 
 
+@pytest.mark.flaky
+@pytest.mark.slow
 def test_mnist():
     ds = deeplake.load("hub://activeloop/mnist-test")
     sample = ds[0]
