@@ -1,5 +1,5 @@
 from deeplake.constants import KB, MB
-from deeplake.util.exceptions import SampleUpdateError
+from deeplake.util.exceptions import SampleUpdateError, TensorDoesNotExistError
 import pytest
 from typing import Callable
 from deeplake.tests.common import assert_array_lists_equal
@@ -51,9 +51,8 @@ def _make_update_assert_equal(
     # this is necessary because `expected` uses `aslist=True` to handle dynamic cases.
     # with `aslist=False`, this wouldn't be necessary.
     expected_value = value
-    if hasattr(value, "__len__"):
-        if len(value) == 1:
-            expected_value = value[0]
+    if hasattr(value, "__len__") and len(value) == 1:
+        expected_value = value[0]
 
     # make updates
     tensor[index] = value
@@ -356,6 +355,7 @@ def test_sequence_htype_with_broadcasting(memory_ds):
 
 
 @pytest.mark.parametrize("shape", [(13, 17, 3), (1007, 3001, 3)])
+@pytest.mark.slow
 def test_sequence_htype_with_deeplake_read(local_ds, shape, compressed_image_paths):
     ds = local_ds
     imgs = list(map(deeplake.read, compressed_image_paths["jpeg"][:3]))
@@ -426,6 +426,7 @@ def test_update_partial(memory_ds, htype, args):
     np.testing.assert_array_equal(arr.reshape(-1), exp.reshape(-1))
 
 
+@pytest.mark.slow
 def test_ds_update_image(local_ds, cat_path, dog_path):
     with local_ds as ds:
         ds.create_tensor("images_sc", htype="image", sample_compression="png")
@@ -569,6 +570,7 @@ def test_ds_update_text_like(local_ds, sc, cc):
     assert ds.json.data()["value"] == [j] * 6
 
 
+@pytest.mark.slow
 def test_ds_update_sequence(local_ds, cat_path, dog_path):
     with local_ds as ds:
         ds.create_tensor("seq", htype="sequence")
@@ -678,6 +680,7 @@ def test_ds_update_polygon(local_ds):
     assert ds.xyz.shape == (6, 3, 3, 2)
 
 
+@pytest.mark.slow
 def test_ds_update_tiles(local_ds, cat_path, dog_path):
     with local_ds as ds:
         ds.create_tensor(
@@ -750,5 +753,22 @@ def test_update_vc_bug(local_path):
 
     ds = deeplake.load(local_path)
 
-    print(ds.abc.numpy())
     np.testing.assert_array_equal(ds.abc.numpy(), [[1], [2], [3], [4], [1]])
+
+
+def test_setattr_update(local_path):
+    with deeplake.empty(local_path, overwrite=True) as ds:
+        ds.create_tensor("abc")
+        ds.create_tensor("xyz/t1")
+        ds.abc.extend([1, 2, 3, 4, 5])
+        ds.xyz.t1.extend([1, 2, 3, 4, 5])
+
+    ds[3].abc = 10
+
+    np.testing.assert_array_equal(ds.abc.numpy(), [[1], [2], [3], [10], [5]])
+
+    with pytest.raises(TypeError):
+        ds[3] = 10
+
+    with pytest.raises(TypeError):
+        ds[3].xyz = 10
