@@ -1,23 +1,18 @@
 import deeplake
-import requests
-from typing import Any, Optional, Dict
+import requests  # type: ignore
+from typing import Any, Optional, Dict, Tuple, List
 from deeplake.util.exceptions import (
     AgreementNotAcceptedError,
     AuthorizationException,
-    LoginException,
-    InvalidPasswordException,
     ManagedCredentialsNotFoundError,
     NotLoggedInAgreementError,
     ResourceNotFoundException,
     InvalidTokenException,
-    UserNotLoggedInException,
     TokenPermissionError,
 )
 from deeplake.client.utils import (
     check_response_status,
-    write_token,
-    read_token,
-    remove_token,
+    JobResponseStatusSchema,
 )
 from deeplake.client.config import (
     ACCEPT_AGREEMENTS_SUFFIX,
@@ -26,9 +21,8 @@ from deeplake.client.config import (
     HUB_REST_ENDPOINT,
     HUB_REST_ENDPOINT_LOCAL,
     HUB_REST_ENDPOINT_DEV,
-    GET_TOKEN_SUFFIX,
+    HUB_REST_ENDPOINT_TESTING,
     HUB_REST_ENDPOINT_STAGING,
-    REGISTER_USER_SUFFIX,
     DEFAULT_REQUEST_TIMEOUT,
     GET_DATASET_CREDENTIALS_SUFFIX,
     CREATE_DATASET_SUFFIX,
@@ -37,10 +31,13 @@ from deeplake.client.config import (
     SEND_EVENT_SUFFIX,
     UPDATE_SUFFIX,
     GET_PRESIGNED_URL_SUFFIX,
+    GET_BLOB_PRESIGNED_URL_SUFFIX,
     CONNECT_DATASET_SUFFIX,
     REMOTE_QUERY_SUFFIX,
+    ORG_PERMISSION_SUFFIX,
 )
 from deeplake.client.log import logger
+from deeplake.client.auth import initialize_auth_context
 import jwt  # should add it to requirements.txt
 
 # for these codes, we will retry requests upto 3 times
@@ -58,35 +55,28 @@ class DeepLakeBackendClient:
         )
 
         self.version = deeplake.__version__
-        self._token_from_env = False
-        self.auth_header = None
-        self.token = token or self.get_token()
-        self.auth_header = f"Bearer {self.token}"
+        self.auth_context = initialize_auth_context(token=token)
 
         # remove public token, otherwise env var will be ignored
         # we can remove this after a while
-        orgs = self.get_user_organizations()
-        if orgs == ["public"]:
-            remove_token()
+        self._username, self._organizations = self._get_username_and_organizations()
+        if self._username == "public":
             self.token = token or self.get_token()
-            self.auth_header = f"Bearer {self.token}"
-        if self._token_from_env:
-            username = self.get_user_profile()["name"]
-            if get_reporting_config().get("username") != username:
-                save_reporting_config(True, username=username)
-                set_username(username)
+        else:
+            if get_reporting_config().get("username") != self._username:
+                save_reporting_config(True, username=self._username)
+                set_username(self._username)
 
     def get_token(self):
-        """Returns a token"""
-        self._token_from_env = False
-        token = read_token(from_env=False)
-        if token is None:
-            token = read_token(from_env=True)
-            if token is None:
-                token = self.request_auth_token(username="public", password="")
-            else:
-                self._token_from_env = True
-        return token
+        return self.auth_context.get_token()
+
+    @property
+    def username(self) -> str:
+        return self._username
+
+    @property
+    def organizations(self) -> List[str]:
+        return self._organizations
 
     def request(
         self,
@@ -119,11 +109,12 @@ class DeepLakeBackendClient:
             headers (dict, optional): Dictionary of HTTP Headers to send with the request.
             timeout (float,optional): How many seconds to wait for the server to send data before giving up.
 
-        Raises:
-            InvalidPasswordException: `password` cannot be `None` inside `json`.
-
         Returns:
             requests.Response: The response received from the server.
+
+        Raises:
+            # noqa: DAR401
+            requests.exceptions.ConnectionError: If any exceptions are thrown during the request
         """
         params = params or {}
         data = data or None
@@ -135,28 +126,34 @@ class DeepLakeBackendClient:
         request_url = f"{endpoint}/{relative_url}"
         headers = headers or {}
         headers["hub-cli-version"] = self.version
-        headers["Authorization"] = self.auth_header
-
-        # clearer error than `ServerUnderMaintenence`
-        if json is not None and "password" in json and json["password"] is None:
-            # do NOT pass in the password here. `None` is explicitly typed.
-            raise InvalidPasswordException("Password cannot be `None`.")
+        headers = {**headers, **self.auth_context.get_auth_headers()}
 
         status_code = None
         tries = 0
+        last_exception: requests.exceptions.ConnectionError | None = None
         while status_code is None or (status_code in retry_status_codes and tries < 3):
-            response = requests.request(
-                method,
-                request_url,
-                params=params,
-                data=data,
-                json=json,
-                headers=headers,
-                files=files,
-                timeout=timeout,
-            )
+            last_exception = None
+            try:
+                response = requests.request(
+                    method,
+                    request_url,
+                    params=params,
+                    data=data,
+                    json=json,
+                    headers=headers,
+                    files=files,
+                    timeout=timeout,
+                )
+            except requests.exceptions.ConnectionError as e:
+                print(f"Connection error on {request_url}. Retrying... Error:", e)
+                tries += 1
+                last_exception = e
+                continue
             status_code = response.status_code
             tries += 1
+        if last_exception:
+            raise last_exception
+
         check_response_status(response)
         return response
 
@@ -165,48 +162,13 @@ class DeepLakeBackendClient:
             return HUB_REST_ENDPOINT_LOCAL
         if deeplake.client.config.USE_DEV_ENVIRONMENT:
             return HUB_REST_ENDPOINT_DEV
+        if deeplake.client.config.USE_TESTING_ENVIRONMENT:
+            return HUB_REST_ENDPOINT_TESTING
         if deeplake.client.config.USE_STAGING_ENVIRONMENT:
             return HUB_REST_ENDPOINT_STAGING
 
         return HUB_REST_ENDPOINT
 
-    def request_auth_token(self, username: str, password: str):
-        """Sends a request to backend to retrieve auth token.
-
-        Args:
-            username (str): The Activeloop username to request token for.
-            password (str): The password of the account.
-
-        Returns:
-            string: The auth token corresponding to the accound.
-
-        Raises:
-            UserNotLoggedInException: if user is not authorised
-            LoginException: If there is an issue retrieving the auth token.
-
-        """
-        json = {"username": username, "password": password}
-        response = self.request("POST", GET_TOKEN_SUFFIX, json=json)
-
-        try:
-            token_dict = response.json()
-            token = token_dict["token"]
-        except Exception:
-            raise LoginException()
-        return token
-
-    def send_register_request(self, username: str, email: str, password: str):
-        """Sends a request to backend to register a new user.
-
-        Args:
-            username (str): The Activeloop username to create account for.
-            email (str): The email id to link with the Activeloop account.
-            password (str): The new password of the account. Should be atleast 6 characters long.
-        """
-
-        json = {"username": username, "email": email, "password": password}
-        self.request("POST", REGISTER_USER_SUFFIX, json=json)
-
     def get_dataset_credentials(
         self,
         org_id: str,
@@ -229,11 +191,11 @@ class DeepLakeBackendClient:
             tuple: containing full url to dataset, credentials, mode and expiration time respectively.
 
         Raises:
-            UserNotLoggedInException: When user is not logged in
+            UserNotLoggedInException: When user is not authenticated
             InvalidTokenException: If the specified token is invalid
             TokenPermissionError: when there are permission or other errors related to token
             AgreementNotAcceptedError: when user has not accepted the agreement
-            NotLoggedInAgreementError: when user is not logged in and dataset has agreement which needs to be signed
+            NotLoggedInAgreementError: when user is not authenticated and dataset has agreement which needs to be signed
         """
         import json
 
@@ -252,9 +214,11 @@ class DeepLakeBackendClient:
             ).json()
         except Exception as e:
             if isinstance(e, AuthorizationException):
-                authorization_exception_prompt = "You don't have permission"
-                response_data = e.response.json()
-                code = response_data.get("code")
+                code = -1
+                if e.response is not None:
+                    response_data = e.response.json()
+                    code = response_data.get("code")
+
                 if code == 1:
                     agreements = response_data["agreements"]
                     agreements = [agreement["text"] for agreement in agreements]
@@ -263,19 +227,13 @@ class DeepLakeBackendClient:
                     raise NotLoggedInAgreementError from e
                 else:
                     try:
-                        decoded_token = jwt.decode(
-                            self.token, options={"verify_signature": False}
+                        jwt.decode(
+                            self.get_token(), options={"verify_signature": False}
                         )
                     except Exception:
                         raise InvalidTokenException
 
-                    if (
-                        authorization_exception_prompt.lower()
-                        in response_data["description"].lower()
-                        and decoded_token["id"] == "public"
-                    ):
-                        raise UserNotLoggedInException()
-                    raise TokenPermissionError()
+                    raise TokenPermissionError(e.original_message)
             raise
         full_url = response.get("path")
         repository = response.get("repository")
@@ -399,22 +357,25 @@ class DeepLakeBackendClient:
             "PUT", suffix, endpoint=self.endpoint(), json={"basename": new_name}
         )
 
-    def get_user_organizations(self):
-        """Get list of user organizations from the backend. If user is not logged in, returns ['public'].
+    def _get_username_and_organizations(self) -> Tuple[str, List[str]]:
+        """Get the username plus a list of user organizations from the backend. If user is not authenticated, returns ('public', ['public']).
 
         Returns:
-            list: user/organization names
+            (str, List[str]): user + organization names
         """
+
+        if self.auth_context.is_public_user():
+            return "public", ["public"]
+
         response = self.request(
             "GET", GET_USER_PROFILE, endpoint=self.endpoint()
         ).json()
-        return response["organizations"]
+        return response["_id"], response["organizations"]
 
     def get_workspace_datasets(
         self, workspace: str, suffix_public: str, suffix_user: str
     ):
-        organizations = self.get_user_organizations()
-        if workspace in organizations:
+        if workspace in self.organizations:
             response = self.request(
                 "GET",
                 suffix_user,
@@ -448,6 +409,21 @@ class DeepLakeBackendClient:
         presigned_url = response["data"]
         return presigned_url
 
+    def get_blob_presigned_url(self, org_id, blob_path, creds_key, expiration=3600):
+        relative_url = GET_BLOB_PRESIGNED_URL_SUFFIX.format(org_id)
+        response = self.request(
+            "GET",
+            relative_url,
+            endpoint=self.endpoint(),
+            params={
+                "creds_key": creds_key,
+                "blob_path": blob_path,
+                "expiration": expiration,
+            },
+        ).json()
+        presigned_url = response["data"]
+        return presigned_url
+
     def get_user_profile(self):
         response = self.request(
             "GET",
@@ -509,3 +485,132 @@ class DeepLakeBackendClient:
         ).json()
 
         return response
+
+    def has_indra_org_permission(self, org_id: str) -> Dict[str, Any]:
+        """Queries a remote dataset.
+
+        Args:
+            org_id (str): The organization to which the dataset belongs.
+
+        Returns:
+            Dict[str, Any]: The json response containing org permissions.
+        """
+        response = self.request(
+            "GET",
+            ORG_PERMISSION_SUFFIX.format(org_id),
+            endpoint=self.endpoint(),
+        ).json()
+
+        return response
+
+
+class DeepMemoryBackendClient(DeepLakeBackendClient):
+    def __init__(self, token: Optional[str] = None):
+        super().__init__(token=token)
+
+    def deepmemory_is_available(self, org_id: str):
+        """Checks if DeepMemory is available for the user.
+        Args:
+            org_id (str): The name of the user/organization to which the dataset belongs.
+        Returns:
+            bool: True if DeepMemory is available, False otherwise.
+        """
+        try:
+            response = self.request(
+                "GET",
+                f"/api/organizations/{org_id}/features/deepmemory",
+                endpoint=self.endpoint(),
+            )
+            return response.json()["available"]
+        except Exception:
+            return False
+
+    def start_taining(
+        self,
+        corpus_path: str,
+        queries_path: str,
+    ) -> Dict[str, Any]:
+        """Starts training of DeepMemory model.
+        Args:
+            corpus_path (str): The path to the corpus dataset.
+            queries_path (str): The path to the queries dataset.
+        Returns:
+            Dict[str, Any]: The json response containing job_id.
+        """
+        response = self.request(
+            method="POST",
+            relative_url="/api/deepmemory/v1/train",
+            json={"corpus_dataset": corpus_path, "query_dataset": queries_path},
+        )
+        check_response_status(response)
+        return response.json()
+
+    def cancel_job(self, job_id: str):
+        """Cancels a job with job_id.
+        Args:
+            job_id (str): The job_id of the job to be cancelled.
+        Returns:
+            bool: True if job was cancelled successfully, False otherwise.
+        """
+        try:
+            response = self.request(
+                method="POST",
+                relative_url=f"/api/deepmemory/v1/jobs/{job_id}/cancel",
+            )
+            check_response_status(response)
+        except Exception as e:
+            print(f"Job with job_id='{job_id}' was not cancelled!\n Error: {e}")
+            return False
+        print("Job cancelled successfully")
+        return True
+
+    def check_status(self, job_id: str, recall: str, improvement: str):
+        """Checks status of a job with job_id.
+        Args:
+            job_id (str): The job_id of the job to be checked.
+            recall (str): Current best top 10 recall
+            importvement (str): Current best improvement over baseline
+        Returns:
+            Dict[str, Any]: The json response containing job status.
+        """
+        response = self.request(
+            method="GET",
+            relative_url=f"/api/deepmemory/v1/jobs/{job_id}/status",
+        )
+        check_response_status(response)
+        response_status_schema = JobResponseStatusSchema(response=response.json())
+        response_status_schema.print_status(job_id, recall, improvement)
+        return response.json()
+
+    def list_jobs(self, dataset_path: str):
+        """Lists all jobs for a dataset.
+        Args:
+            dataset_path (str): The path to the dataset.
+        Returns:
+            Dict[str, Any]: The json response containing list of jobs.
+        """
+        dataset_id = dataset_path[6:]
+        response = self.request(
+            method="GET",
+            relative_url=f"/api/deepmemory/v1/{dataset_id}/jobs",
+        )
+        check_response_status(response)
+        return response.json()
+
+    def delete_job(self, job_id: str):
+        """Deletes a job with job_id.
+        Args:
+            job_id (str): The job_id of the job to be deleted.
+        Returns:
+            bool: True if job was deleted successfully, False otherwise.
+        """
+        try:
+            response = self.request(
+                method="DELETE",
+                relative_url=f"/api/deepmemory/v1/jobs/{job_id}",
+            )
+            check_response_status(response)
+            return True
+        except Exception as e:
+            print(f"Job with job_id='{job_id}' was not deleted!\n Error: {e}")
+            return False
