@@ -7,15 +7,16 @@ REPORTING_CONFIG_FILE_PATH = os.path.expanduser("~/.activeloop/reporting_config.
 HUB_REST_ENDPOINT = "https://app.activeloop.ai"
 HUB_REST_ENDPOINT_STAGING = "https://app-staging.activeloop.dev"
 HUB_REST_ENDPOINT_DEV = "https://app-dev.activeloop.dev"
+HUB_REST_ENDPOINT_TESTING = "https://testing.activeloop.dev"
 HUB_REST_ENDPOINT_LOCAL = "http://localhost:7777"
 USE_LOCAL_HOST = False
 USE_DEV_ENVIRONMENT = False
+USE_TESTING_ENVIRONMENT = False
 USE_STAGING_ENVIRONMENT = False
 
-GET_TOKEN_SUFFIX = "/api/user/token"
-REGISTER_USER_SUFFIX = "/api/user/register"
 GET_DATASET_CREDENTIALS_SUFFIX = "/api/org/{}/ds/{}/creds"
 GET_PRESIGNED_URL_SUFFIX = "/api/org/{}/ds/{}/chunks/url/presigned"
+GET_BLOB_PRESIGNED_URL_SUFFIX = "/api/org/{}/managed-credentials/blob/url/presigned"
 CREATE_DATASET_SUFFIX = "/api/dataset/create"
 SEND_EVENT_SUFFIX = "/api/event"
 DATASET_SUFFIX = "/api/dataset"
@@ -30,3 +31,5 @@ REMOTE_QUERY_SUFFIX = "/api/query/dataset/{}/{}"
 DEFAULT_REQUEST_TIMEOUT = 170
 
 DEEPLAKE_AUTH_TOKEN = "ACTIVELOOP_TOKEN"
+DEEPLAKE_AUTH_PROVIDER = "ACTIVELOOP_AUTH_PROVIDER"
+ORG_PERMISSION_SUFFIX = "/api/organizations/{}/features/dataset_query"
