@@ -1,3 +1,4 @@
+import numbers
 from abc import abstractmethod
 import struct
 import numpy as np
@@ -91,7 +92,7 @@ class BaseChunk(DeepLakeMemoryObject):
             else (len(tensor_meta.max_shape) if tensor_meta.max_shape else None)
         )
         self.is_text_like = (
-            self.htype in {"json", "list", "text"} or self.tensor_meta.is_link
+            self.htype in {"json", "list", "text", "tag"} or self.tensor_meta.is_link
         )
 
         self.compression = compression
@@ -251,7 +252,12 @@ class BaseChunk(DeepLakeMemoryObject):
 
     @abstractmethod
     def extend_if_has_space(
-        self, incoming_samples, update_meta: bool = True, end: bool = True, **kwargs
+        self,
+        incoming_samples,
+        update_meta: bool = True,
+        end: bool = True,
+        ignore_errors: bool = False,
+        **kwargs,
     ) -> float:
         """Extends the chunk with the incoming samples."""
 
@@ -323,7 +329,7 @@ class BaseChunk(DeepLakeMemoryObject):
                 incoming_sample = incoming_sample.path
             if incoming_sample is None:
                 htype = "text" if self.tensor_meta.is_link else self.htype
-                empty_mapping = {"text": "", "list": [], "json": {}}
+                empty_mapping = {"text": "", "list": [], "json": {}, "tag": []}
                 incoming_sample = empty_mapping[htype]
 
             if isinstance(incoming_sample, Sample):
@@ -600,6 +606,9 @@ class BaseChunk(DeepLakeMemoryObject):
         )
 
     def _text_sample_to_byte_string(self, sample):
+        if isinstance(sample, numbers.Number):
+            sample = str(sample)
+
         try:
             return str(sample.numpy().reshape(())).encode("utf-8")
         except AttributeError:
@@ -619,6 +628,12 @@ class BaseChunk(DeepLakeMemoryObject):
                 "Need to add at least one non-empty sample before retrieving data."
             )
 
+    def __getstate__(self):
+        state = self.__dict__.copy()
+        if isinstance(state["_data_bytes"], memoryview):
+            state["_data_bytes"] = state["_data_bytes"].tobytes()
+        return state
+
 
 def catch_chunk_read_error(fn):
     @wraps(fn)
