@@ -25,38 +25,17 @@ from pathlib import Path
 from PIL import Image  # type: ignore
 from io import BytesIO
 
+import lz4.frame  # type: ignore
+import lz4.block  # type: ignore
 import mmap
 import struct
 import sys
 import re
-import numcodecs.lz4  # type: ignore
 from numpy.core.fromnumeric import compress  # type: ignore
 import math
 from pathlib import Path
 from gzip import GzipFile
 
-try:
-    import av  # type: ignore
-
-    _PYAV_INSTALLED = True
-except ImportError:
-    _PYAV_INSTALLED = False
-
-try:
-    import lz4.frame  # type: ignore
-
-    _LZ4_INSTALLED = True
-except ImportError:
-    _LZ4_INSTALLED = False
-
-try:
-    import nibabel as nib  # type: ignore
-    from nibabel import FileHolder, Nifti1Image, Nifti2Image  # type: ignore
-
-    _NIBABEL_INSTALLED = True
-except ImportError:
-    _NIBABEL_INSTALLED = False
-
 if sys.byteorder == "little":
     _NATIVE_INT32 = "<i4"
     _NATIVE_FLOAT32 = "<f4"
@@ -64,6 +43,8 @@ else:
     _NATIVE_INT32 = ">i4"
     _NATIVE_FLOAT32 = ">f4"
 
+HEADER_MAX_BYTES = 132
+
 DIMS_RE = re.compile(rb" ([0-9]+)x([0-9]+)")
 FPS_RE = re.compile(rb" ([0-9]+) fps,")
 DURATION_RE = re.compile(rb"Duration: ([0-9:.]+),")
@@ -105,7 +86,7 @@ _STRUCT_HHB = struct.Struct(">HHB")
 _STRUCT_II = struct.Struct(">ii")
 
 
-def to_image(array: np.ndarray) -> Image:
+def to_image(array: np.ndarray) -> Image.Image:
     shape = array.shape
     if len(shape) == 3 and shape[0] != 1 and shape[2] == 1:
         # convert (X,Y,1) grayscale to (X,Y) for pillow compatibility
@@ -135,17 +116,22 @@ def _compress_apng(array: np.ndarray) -> bytes:
 
 def _decompress_apng(buffer: Union[bytes, memoryview]) -> np.ndarray:
     img = Image.open(BytesIO(buffer))
+
+    n_frames = getattr(img, "n_frames", 1)
+
+    if n_frames == 1:
+        raise ValueError("Image does not support multiple frames.")
     frame0 = np.array(img)
     if frame0.ndim == 2:
-        ret = np.zeros(frame0.shape + (img.n_frames,), dtype=frame0.dtype)
+        ret = np.zeros(frame0.shape + (n_frames,), dtype=frame0.dtype)
         ret[:, :, 0] = frame0
-        for i in range(1, img.n_frames):
+        for i in range(1, n_frames):
             img.seek(i)
             ret[:, :, i] = np.array(img)
     else:
-        ret = np.zeros((img.n_frames,) + frame0.shape, dtype=frame0.dtype)
+        ret = np.zeros((n_frames,) + frame0.shape, dtype=frame0.dtype)
         ret[0] = frame0
-        for i in range(1, img.n_frames):
+        for i in range(1, n_frames):
             img.seek(i)
             ret[i] = np.array(img)
     return ret
@@ -157,9 +143,7 @@ def compress_bytes(
     if not buffer:
         return b""
     if compression == "lz4":
-        if not buffer:
-            return b""
-        return numcodecs.lz4.compress(buffer)
+        return lz4.block.compress(buffer)
     else:
         raise SampleCompressionError(
             (len(buffer),), compression, f"Not a byte compression: {compression}"
@@ -175,15 +159,9 @@ def decompress_bytes(
         # weird edge case of lz4 + empty string
         if buffer == b"\x00\x00\x00\x00\x00":
             return b""
-        if (
-            buffer[:4] == b'\x04"M\x18'
-        ):  # python-lz4 magic number (backward compatiblity)
-            if not _LZ4_INSTALLED:
-                raise ModuleNotFoundError(
-                    "Module lz4 not found. Install using `pip install lz4`."
-                )
+        if buffer[:4] == b'\x04"M\x18':  # python-lz4 magic number
             return lz4.frame.decompress(buffer)
-        return numcodecs.lz4.decompress(buffer)
+        return lz4.block.decompress(buffer)
     else:
         raise SampleDecompressionError()
 
@@ -320,7 +298,7 @@ def decompress_array(
             return _decompress_audio(buffer)
         elif compr_type == VIDEO_COMPRESSION:
             return _decompress_video(buffer, start_idx, end_idx, step, reverse)  # type: ignore
-        elif compr_type in [POINT_CLOUD_COMPRESSION, MESH_COMPRESSION]:
+        elif compr_type in [POINT_CLOUD_COMPRESSION] or compression == "ply":
             return _decompress_3d_data(buffer)
 
         if compression == "apng":
@@ -331,6 +309,8 @@ def decompress_array(
             return _decompress_nifti(buffer)
         if compression == "nii.gz":
             return _decompress_nifti(buffer, gz=True)
+        if compression == "stl":
+            return _decompress_stl(buffer)
         if compression is None and isinstance(buffer, memoryview) and shape is not None:
             assert buffer is not None
             assert shape is not None
@@ -445,9 +425,9 @@ def decompress_multiple(
     next_x = 0
     for shape in shapes:
         if shape == (0, 0, 0):
-            arrays.append(np.zeros(shape, dtype=canvas.dtype))
+            arrays.append(np.zeros(shape, dtype=canvas.dtype))  # type: ignore
         else:
-            arrays.append(canvas[: shape[0], next_x : next_x + shape[1]])
+            arrays.append(canvas[: shape[0], next_x : next_x + shape[1]])  # type: ignore
             next_x += shape[1]
     return arrays
 
@@ -491,6 +471,8 @@ def verify_compressed_file(
             return _read_nifti_shape_and_dtype(file, gz=compression == "nii.gz")
         elif compression in ("las", "ply"):
             return _read_3d_data_shape_and_dtype(file)
+        elif compression == "stl":
+            return _read_stl_shape_and_dtype(file)
         else:
             return _fast_decompress(file)
     except Exception as e:
@@ -517,19 +499,47 @@ def get_compression(header=None, path=None):
             ".ply",
             ".nii",
             ".nii.gz",
+            ".stl",
         ]
-        path = str(path).lower()
+        path = str(path).lower().partition("?")[0].partition("#")[0].partition(";")[0]
         for fmt in file_formats:
             if path.endswith(fmt):
                 return fmt[1:]
     if header:
+        if (
+            header[4:12] == b"\x66\x74\x79\x70\x4d\x53\x4e\x56"
+            or header[4:12] == b"\x66\x74\x79\x70\x69\x73\x6f\x6d"
+        ):
+            return "mp4"
+        if header[0:4] == b"\x1a\x45\xdf\xa3":
+            return "mkv"
+        if (
+            header[0:2] == b"\xff\xfb"
+            or header[0:2] == b"\xff\xf3"
+            or header[0:2] == b"\xff\xf2"
+        ):
+            return "mp3"
+        if header[0:2] == b"\x66\x4c\x61\x43":
+            return "flac"
+        if header[0:4] == b"\x52\x49\x46\x46" and header[8:12] == b"\x57\x41\x56\x45":
+            return "wav"
+        if header[0:4] == b"\x52\x49\x46\x46" and header[8:12] == b"\x41\x56\x49\x20":
+            return "avi"
+        if header[128:132] == b"\x44\x49\x43\x4d":
+            return "dcm"
+        if header[0:4] == b"\x6e\x2b\x31\x00":
+            return "nii"
+        if any(
+            header[: len(x)] == x for x in [b"\x73\x6f\x6c\x69", b"numpy-stl", b"solid"]
+        ):
+            return "stl"
         if not Image.OPEN:
             Image.init()
         for fmt in Image.OPEN:
             accept = Image.OPEN[fmt][1]
             if accept and accept(header):
                 return fmt.lower()
-        raise SampleDecompressionError()
+        raise SampleDecompressionError(path)
 
 
 def _verify_png(buf):
@@ -680,10 +690,10 @@ def read_meta_from_compressed_file(
     try:
         if compression is None:
             if hasattr(f, "read"):
-                compression = get_compression(f.read(32), path)
+                compression = get_compression(f.read(HEADER_MAX_BYTES), path)
                 f.seek(0)
             else:
-                compression = get_compression(f[:32], path)  # type: ignore
+                compression = get_compression(f[:HEADER_MAX_BYTES], path)  # type: ignore
         if compression == "jpeg":
             try:
                 shape, typestr = _read_jpeg_shape(f), "|u1"
@@ -715,10 +725,15 @@ def read_meta_from_compressed_file(
                 shape, typestr = _read_3d_data_shape_and_dtype(file)
             except Exception as e:
                 raise CorruptedSampleError(compression, path) from e
+        elif compression == "stl":
+            try:
+                shape, typestr = _read_stl_shape_and_dtype(file)
+            except Exception as e:
+                raise CorruptedSampleError(compression, path) from e
         else:
             img = Image.open(f) if isfile else Image.open(BytesIO(f))  # type: ignore
             shape, typestr = Image._conv_type_shape(img)
-            compression = img.format.lower()
+            compression = None if img.format is None else img.format.lower()
         return compression, shape, typestr  # type: ignore
     finally:
         if close:
@@ -796,7 +811,7 @@ def _read_jpeg_shape_from_buffer(buf: bytes) -> Tuple[int, ...]:
 
 
 def _read_dicom_shape_and_dtype(
-    f: Union[bytes, BinaryIO]
+    f: Union[bytes, BinaryIO],
 ) -> Tuple[Tuple[int, ...], str]:
     try:
         from pydicom import dcmread
@@ -809,10 +824,15 @@ def _read_dicom_shape_and_dtype(
         f = BytesIO(f)  # type: ignore
     dcm = dcmread(f)
     nchannels = dcm[0x0028, 0x0002].value
-    shape = (dcm.Rows, dcm.Columns, nchannels)
+    numOfFrames = dcm.get("NumberOfFrames", -1)
+
     isfloat = "FloatPixelData" in dcm or "DoubleFloatPixelData" in dcm
     dtype = pixel_dtype(dcm, isfloat).str
-    return shape, dtype
+
+    if numOfFrames != -1:
+        return (int(numOfFrames), dcm.Rows, dcm.Columns, nchannels), dtype
+    else:
+        return (dcm.Rows, dcm.Columns, nchannels), dtype
 
 
 def _decompress_dicom(f: Union[str, bytes, BinaryIO]):
@@ -877,13 +897,19 @@ def _frame_to_stamp(nframe, stream):
 
 
 def _open_video(file: Union[str, bytes, memoryview]):
-    if not _PYAV_INSTALLED:
+    try:
+        import av  # type: ignore
+        import av.container
+    except ImportError:
         raise ModuleNotFoundError(
             "PyAV is not installed. Run `pip install deeplake[video]`."
         )
+
     if isinstance(file, str):
-        container = av.open(
-            file, options={"protocol_whitelist": "file,http,https,tcp,tls,subfile"}
+        container: Union[av.container.InputContainer, av.container.OutputContainer] = (
+            av.open(
+                file, options={"protocol_whitelist": "file,http,https,tcp,tls,subfile"}
+            )
         )
     else:
         container = av.open(BytesIO(file))
@@ -899,6 +925,8 @@ def _open_video(file: Union[str, bytes, memoryview]):
 
 
 def _read_metadata_from_vstream(container, vstream):
+    import av
+
     duration = vstream.duration
     if duration is None:
         duration = container.duration
@@ -933,17 +961,14 @@ def _decompress_video(
     step: int,
     reverse: bool,
 ):
+    import av
+
     container, vstream = _open_video(file)
     nframes, height, width, _ = _read_metadata_from_vstream(container, vstream)[0]
 
-    if start is None:
-        start = 0
-
-    if stop is None:
-        stop = nframes
-
-    if step is None:
-        step = 1
+    start = start or 0
+    stop = stop or nframes
+    step = step or 1
 
     nframes = math.ceil((stop - start) / step)
 
@@ -952,22 +977,20 @@ def _decompress_video(
     seek_target = _frame_to_stamp(start, vstream)
     step_time = _frame_to_stamp(step, vstream)
 
-    gop_size = (
-        vstream.codec_context.gop_size
-    )  # gop size is distance (in frames) between 2 I-frames
-    if step > gop_size:
-        step_seeking = True
-    else:
-        step_seeking = False
+    gop_size = 1
+    if vstream.codec_context.is_encoder:
+        gop_size = vstream.codec_context.gop_size
+
+    step_seeking = step > gop_size
 
     seekable = True
     try:
         container.seek(seek_target, stream=vstream)
-    except av.error.PermissionError:
+    except av.error.FFmpegError:
         seekable = False
-        container, vstream = _open_video(file)  # try again but this time don't seek
+        container, vstream = _open_video(file)  # Retry without seeking
         warning(
-            "Cannot seek. Possibly a corrupted video file. Retrying with seeking disabled..."
+            "Cannot seek. Possibly a corrupted video file. Retrying without seeking."
         )
 
     i = 0
@@ -980,7 +1003,6 @@ def _decompress_video(
                 seek_target += step_time
                 if step_seeking and seekable:
                     container.seek(seek_target, stream=vstream)
-
         if i == nframes:
             break
 
@@ -996,6 +1018,8 @@ def _read_timestamps(
     step: int,
     reverse: bool,
 ) -> np.ndarray:
+    import av
+
     container, vstream = _open_video(file)
 
     nframes = math.ceil((stop - start) / step)
@@ -1004,25 +1028,20 @@ def _read_timestamps(
     step_time = _frame_to_stamp(step, vstream)
 
     stamps = []
-    if vstream.duration is None:
-        time_base = 1 / av.time_base
-    else:
-        time_base = vstream.time_base.numerator / vstream.time_base.denominator
+    time_base = vstream.time_base.numerator / vstream.time_base.denominator
 
-    gop_size = (
-        vstream.codec_context.gop_size
-    )  # gop size is distance (in frames) between 2 I-frames
-    if step > gop_size:
-        step_seeking = True
-    else:
-        step_seeking = False
+    gop_size = 1
+    if vstream.codec_context.is_encoder:
+        gop_size = vstream.codec_context.gop_size
+
+    step_seeking = step > gop_size
 
     seekable = True
     try:
         container.seek(seek_target, stream=vstream)
-    except av.error.PermissionError:
+    except av.error.FFmpegError:
         seekable = False
-        container, vstream = _open_video(file)  # try again but this time don't seek
+        container, vstream = _open_video(file)  # Retry without seeking
         warning(
             "Cannot seek. Possibly a corrupted video file. Retrying with seeking disabled..."
         )
@@ -1043,6 +1062,7 @@ def _read_timestamps(
     # need to sort because when demuxing, frames are in order of dts (decoder timestamp)
     # we need it in order of pts (presentation timestamp)
     stamps.sort()
+
     stamps_arr = np.zeros((nframes,), dtype=np.float32)
     stamps_arr[: len(stamps)] = stamps
 
@@ -1052,13 +1072,19 @@ def _read_timestamps(
 
 
 def _open_audio(file: Union[str, bytes, memoryview]):
-    if not _PYAV_INSTALLED:
+    try:
+        import av
+        import av.container
+    except ImportError:
         raise ModuleNotFoundError(
             "PyAV is not installed. Please run `pip install deeplake[audio]`"
         )
+
     if isinstance(file, str):
-        container = av.open(
-            file, options={"protocol_whitelist": "file,http,https,tcp,tls,subfile"}
+        container: Union[av.container.InputContainer, av.container.OutputContainer] = (
+            av.open(
+                file, options={"protocol_whitelist": "file,http,https,tcp,tls,subfile"}
+            )
         )
     else:
         container = av.open(BytesIO(file))
@@ -1074,6 +1100,8 @@ def _open_audio(file: Union[str, bytes, memoryview]):
 
 
 def _read_shape_from_astream(container, astream):
+    import av
+
     nchannels = astream.channels
     duration = astream.duration
     if duration is None:
@@ -1103,6 +1131,8 @@ def _read_audio_shape(
 def _read_audio_meta(
     file: Union[bytes, memoryview, str],
 ) -> dict:
+    import av
+
     container, astream = _open_audio(file)
     meta = {}
     if astream.duration:
@@ -1110,7 +1140,7 @@ def _read_audio_meta(
         meta["time_base"] = astream.time_base.numerator / astream.time_base.denominator
     else:
         meta["duration"] = container.duration
-        meta["time_base"] = 1 / av.time_base
+        meta["time_base"] = 1 / av.time_base  # type: ignore
     meta["sample_rate"] = astream.sample_rate
     meta["duration"] = astream.duration or container.duration
     meta["frame_size"] = astream.frame_size
@@ -1160,6 +1190,15 @@ def _open_3d_data(file):
     return point_cloud
 
 
+def _open_mesh_data(file: Union[bytes, memoryview, str]):
+    from stl import mesh
+
+    if isinstance(file, str):
+        return mesh.Mesh.from_file(file)
+
+    return mesh.Mesh.from_file("", fh=BytesIO(file))
+
+
 def _decompress_3d_data(file: Union[bytes, memoryview, str]):
     point_cloud = _open_3d_data(file)
     return point_cloud.decompressed_3d_data
@@ -1170,13 +1209,39 @@ def _read_3d_data_shape_and_dtype(file: Union[bytes, BinaryIO]):
     return point_cloud.shape, point_cloud.dtype
 
 
+def _read_stl_shape_and_dtype(file):
+    mesh_data = _open_mesh_data(file)
+    return mesh_data.vectors.shape, mesh_data.vectors.dtype
+
+
+def _decompress_stl(file: Union[bytes, str]):
+    mesh_data = _open_mesh_data(file)
+    return mesh_data.vectors
+
+
 def _read_3d_data_meta(file: Union[bytes, memoryview, str]):
     point_cloud = _open_3d_data(file)
     return point_cloud.meta_data
 
 
+def _read_stl_data_meta(file: Union[bytes, memoryview, str]):
+    mesh_data = _open_mesh_data(file)
+    return {
+        "name": mesh_data.name,
+        "min_": mesh_data.min_,
+        "max_": mesh_data.max_,
+        "speedups": mesh_data.speedups,
+        "centroids": mesh_data.centroids,
+        "normals": mesh_data.normals,
+        "extension": "stl",
+    }
+
+
 def _open_nifti(file: Union[bytes, memoryview, str], gz: bool = False):
-    if not _NIBABEL_INSTALLED:
+    try:
+        import nibabel as nib  # type: ignore
+        from nibabel import FileHolder, Nifti1Image, Nifti2Image  # type: ignore
+    except ImportError:
         raise ModuleNotFoundError(
             "nibabel is not installed. Please run `pip install deeplake[medical]`"
         )
