@@ -1,4 +1,4 @@
-import requests
+import requests  # type: ignore
 from deeplake.core.compression import (
     compress_array,
     decompress_array,
@@ -9,7 +9,9 @@ from deeplake.core.compression import (
     _read_metadata_from_vstream,
     _read_audio_meta,
     _read_3d_data_meta,
+    _read_stl_data_meta,
     _open_nifti,
+    HEADER_MAX_BYTES,
 )
 from deeplake.compression import (
     get_compression_type,
@@ -19,6 +21,7 @@ from deeplake.compression import (
     POINT_CLOUD_COMPRESSION,
     MESH_COMPRESSION,
     NIFTI_COMPRESSION,
+    BYTE_COMPRESSION,
 )
 from deeplake.util.exceptions import SampleReadError, UnableToReadFromUrlError
 from deeplake.util.exif import getexif
@@ -59,6 +62,7 @@ class Sample:
         dtype: Optional[str] = None,
         creds: Optional[Dict] = None,
         storage: Optional[StorageProvider] = None,
+        timeout: Optional[float] = None,
     ):
         """Represents a single sample for a tensor. Provides all important meta information in one place.
 
@@ -76,6 +80,7 @@ class Sample:
             dtype (optional, str): Data type of the sample.
             creds (optional, Dict): Credentials for s3, gcp and http urls.
             storage (optional, StorageProvider): Storage provider.
+            timeout (optional, float): Timeout in seconds for reading the file. Applicable only for http(s) urls.
         """
         self._compressed_bytes = {}
         self._uncompressed_bytes = None
@@ -91,6 +96,7 @@ class Sample:
         self._buffer = None
         self._creds = creds or {}
         self._verify = verify
+        self._timeout = timeout
 
         if path is not None:
             self.path = path
@@ -100,7 +106,9 @@ class Sample:
                     self._compression = get_compression(path=self.path)
                 compressed_bytes = self._read_from_path()
                 if self._compression is None:
-                    self._compression = get_compression(header=compressed_bytes[:32])
+                    self._compression = get_compression(
+                        header=compressed_bytes[:HEADER_MAX_BYTES]
+                    )
                 self._shape, self._typestr = verify_compressed_file(compressed_bytes, self._compression)  # type: ignore
 
         if array is not None:
@@ -135,6 +143,7 @@ class Sample:
         sample._creds = self._creds
         sample._verify = self._verify
         sample._compression = self._compression
+        sample._timeout = self._timeout
         return sample
 
     @property
@@ -147,7 +156,7 @@ class Sample:
 
     @property
     def is_text_like(self):
-        return self.htype in {"text", "list", "json"}
+        return self.htype in {"text", "list", "json", "tag"}
 
     @property
     def dtype(self):
@@ -223,9 +232,11 @@ class Sample:
             x.keyword: {
                 "name": x.name,
                 "tag": str(x.tag),
-                "value": x.value
-                if isinstance(x.value, (str, int, float))
-                else x.to_json_dict(None, None).get("Value", ""),  # type: ignore
+                "value": (
+                    x.value
+                    if isinstance(x.value, (str, int, float))
+                    else x.to_json_dict(None, None).get("Value", "")  # type: ignore
+                ),
                 "vr": x.VR,
             }
             for x in dcm
@@ -265,6 +276,13 @@ class Sample:
             info = _read_3d_data_meta(self.buffer)
         return info
 
+    def _get_stl_meta(self) -> dict:
+        if self.path and get_path_type(self.path) == "local":
+            info = _read_stl_data_meta(self.path)
+        else:
+            info = _read_stl_data_meta(self.buffer)
+        return info
+
     @property
     def is_lazy(self) -> bool:
         return self._array is None
@@ -311,7 +329,9 @@ class Sample:
                     self._compression = get_compression(path=self.path)
                 compressed_bytes = self._read_from_path()
                 if self._compression is None:
-                    self._compression = get_compression(header=compressed_bytes[:32])
+                    self._compression = get_compression(
+                        header=compressed_bytes[:HEADER_MAX_BYTES]
+                    )
                 if self._compression == compression:
                     if self._shape is None:
                         _, self._shape, self._typestr = read_meta_from_compressed_file(
@@ -321,7 +341,9 @@ class Sample:
                     compressed_bytes = self._recompress(compressed_bytes, compression)
             elif self._buffer is not None:
                 if self._compression is None:
-                    self._compression = get_compression(header=self._buffer[:32])
+                    self._compression = get_compression(
+                        header=self._buffer[:HEADER_MAX_BYTES]
+                    )
                 if self._compression == compression:
                     compressed_bytes = self._buffer
                 else:
@@ -344,15 +366,17 @@ class Sample:
                 buffer = bytes(self._buffer)
                 self._array = bytes_to_text(buffer, self.htype)
             else:
-                self._array = np.frombuffer(self._buffer, dtype=self.dtype).reshape(
-                    self.shape
-                )
+                try:
+                    self._array = np.frombuffer(self._buffer, dtype=self.dtype).reshape(
+                        self.shape
+                    )
+                except Exception as e:
+                    raise e.__class__(
+                        f"Failed to decompress buffer of length {len(self._buffer)} into {self.shape} {self.dtype} array"
+                    ) from e
 
         else:
-            if self.path and get_path_type(self.path) == "local":
-                compressed = self.path
-            else:
-                compressed = self.buffer
+            compressed = self.buffer
 
             if to_pil:
                 self._pil = decompress_array(
@@ -368,10 +392,10 @@ class Sample:
                     compression=compression,
                     shape=self.shape,
                     dtype=self.dtype,
-                )
-                self._uncompressed_bytes = self._array.tobytes()
-                self._typestr = self._array.__array_interface__["typestr"]
-                self._dtype = np.dtype(self._typestr).name
+                )  # type: ignore
+                self._uncompressed_bytes = self._array.tobytes()  # type: ignore
+                self._typestr = self._array.__array_interface__["typestr"]  # type: ignore
+                self._dtype = np.dtype(self._typestr).name  # type: ignore
 
     def uncompressed_bytes(self) -> Optional[bytes]:
         """Returns uncompressed bytes."""
@@ -396,7 +420,7 @@ class Sample:
         return self._array  # type: ignore
 
     @property
-    def pil(self) -> Image.Image:  # type: ignore
+    def pil(self) -> Optional[Image.Image]:  # type: ignore
         """Return PIL image corresponding to the sample. Decompresses the sample if necessary.
 
         Example:
@@ -447,7 +471,7 @@ class Sample:
                 elif path_type == "gdrive":
                     self._buffer = self._read_from_gdrive()
                 elif path_type == "http":
-                    self._buffer = self._read_from_http()
+                    self._buffer = self._read_from_http(timeout=self._timeout)
             except Exception as e:
                 raise SampleReadError(self.path) from e  # type: ignore
         return self._buffer  # type: ignore
@@ -505,13 +529,13 @@ class Sample:
         )
         return gdrive.get_object_from_full_url(self.path)  # type: ignore
 
-    def _read_from_http(self) -> bytes:
+    def _read_from_http(self, timeout=None) -> bytes:
         assert self.path is not None
         if "Authorization" in self._creds:
             headers = {"Authorization": self._creds["Authorization"]}
         else:
             headers = {}
-        result = requests.get(self.path, headers=headers)
+        result = requests.get(self.path, headers=headers, timeout=timeout)
         if result.status_code != 200:
             raise UnableToReadFromUrlError(self.path, result.status_code)
         return result.content
@@ -536,6 +560,8 @@ class Sample:
         compression_type = get_compression_type(compression)
         if compression == "dcm":
             meta.update(self._get_dicom_meta())
+        elif compression == "stl":
+            meta.update(self._get_stl_meta())
         elif compression_type == NIFTI_COMPRESSION:
             meta.update(self._get_nifti_meta())
         elif compression_type == IMAGE_COMPRESSION:
