@@ -148,7 +148,7 @@ def get_header_from_url(url: str):
     enc_dtype = np.dtype(deeplake.constants.ENCODING_DTYPE)
     itemsize = enc_dtype.itemsize
 
-    headers = {"Range": "bytes=0-100"}
+    headers = {"Range": "bytes=0-1000"}
 
     request = Request(url, None, headers)
     byts = urlopen(request).read()
@@ -164,8 +164,16 @@ def get_header_from_url(url: str):
     if shape_info_nbytes == 0:
         shape_info = np.array([], dtype=enc_dtype)
     else:
+        end_bytes = offset + shape_info_nbytes
+        if len(byts) < end_bytes:
+            # need to fetch more data than the initial guess of 100 bytes. Doubling to hopefully have enough for reading the byte positions
+            headers = {"Range": f"bytes=0-{end_bytes * 2}"}
+
+            request = Request(url, None, headers)
+            byts = urlopen(request).read()
+
         shape_info = (
-            np.frombuffer(byts[offset : offset + shape_info_nbytes], dtype=enc_dtype)
+            np.frombuffer(byts[offset:end_bytes], dtype=enc_dtype)
             .reshape(shape_info_nrows, shape_info_ncols)
             .copy()
         )
@@ -178,10 +186,15 @@ def get_header_from_url(url: str):
     if byte_positions_nbytes == 0:
         byte_positions = np.array([], dtype=enc_dtype)
     else:
+        end_bytes = offset + byte_positions_nbytes
+        if len(byts) < end_bytes:
+            headers = {"Range": f"bytes=0-{end_bytes}"}
+
+            request = Request(url, None, headers)
+            byts = urlopen(request).read()
+
         byte_positions = (
-            np.frombuffer(
-                byts[offset : offset + byte_positions_nbytes], dtype=enc_dtype
-            )
+            np.frombuffer(byts[offset:end_bytes], dtype=enc_dtype)
             .reshape(byte_positions_rows, 3)
             .copy()
         )
@@ -298,7 +311,7 @@ def serialize_linked_tiled_sample(sample: LinkedTiledSample):
 
 
 def deserialize_linked_tiled_sample(
-    byts: Union[bytes, memoryview]
+    byts: Union[bytes, memoryview],
 ) -> LinkedTiledSample:
     byts = memoryview(byts)
     # Read version
@@ -361,7 +374,7 @@ def serialize_chunkids(version: str, arr: np.ndarray) -> memoryview:
 
 
 def deserialize_chunkids(
-    byts: Union[bytes, memoryview]
+    byts: Union[bytes, memoryview],
 ) -> Tuple[str, np.ndarray, type]:
     """Deserializes a chunk ID encoder from the serialized byte stream. This is how the encoder can be accessed/modified after it is read from storage.
 
@@ -439,7 +452,7 @@ def check_sample_shape(shape, num_dims):
 
 
 def text_to_bytes(sample, dtype, htype):
-    if htype in ("json", "list"):
+    if htype in ("json", "list", "tag"):
         if isinstance(sample, np.ndarray):
             if htype == "list":
                 sample = list(sample) if sample.dtype == object else sample.tolist()
@@ -450,7 +463,7 @@ def text_to_bytes(sample, dtype, htype):
                     sample = list(sample)
         validate_json_object(sample, dtype)
         byts = json.dumps(sample, cls=HubJsonEncoder).encode()
-        shape = (len(sample),) if htype == "list" else (1,)
+        shape = (len(sample),) if htype in ("list", "tag") else (1,)
     else:  # htype == "text":
         if isinstance(sample, np.ndarray) and sample.size == 1:
             sample = str(sample.reshape(()))
@@ -467,7 +480,7 @@ def bytes_to_text(buffer, htype):
         arr = np.empty(1, dtype=object)
         arr[0] = json.loads(bytes.decode(buffer), cls=HubJsonDecoder)
         return arr
-    elif htype == "list":
+    elif htype in ("list", "tag"):
         lst = json.loads(bytes.decode(buffer), cls=HubJsonDecoder)
         arr = np.empty(len(lst), dtype=object)
         arr[:] = lst
