@@ -5,31 +5,25 @@ from typing import Dict, Optional, Tuple
 from datetime import datetime, timedelta, timezone
 
 from deeplake.core.storage.provider import StorageProvider
-from deeplake.client.client import DeepLakeBackendClient
 from deeplake.util.exceptions import PathNotEmptyException
-
-try:
-    from azure.identity import DefaultAzureCredential
-    from azure.storage.blob import (
-        BlobServiceClient,
-        BlobSasPermissions,
-        ContainerSasPermissions,
-        generate_blob_sas,
-        generate_container_sas,
-    )
-    from azure.core.credentials import AzureNamedKeyCredential, AzureSasCredential
-
-    logger = logging.getLogger("azure.identity")
-    logger.setLevel(logging.ERROR)
-
-    _AZURE_PACKAGES_INSTALLED = True
-except ImportError:
-    _AZURE_PACKAGES_INSTALLED = False
+from deeplake.client.client import DeepLakeBackendClient
+from concurrent.futures import ThreadPoolExecutor
+from deeplake.util.path import relpath
+from concurrent import futures
+from deeplake.util.warnings import always_warn
+import time
 
 
 class AzureProvider(StorageProvider):
     def __init__(self, root: str, creds: Dict = {}, token: Optional[str] = None):
-        if not _AZURE_PACKAGES_INSTALLED:
+        try:
+            import azure.identity
+            import azure.storage.blob
+            import azure.core
+
+            logger = logging.getLogger("azure.identity")
+            logger.setLevel(logging.ERROR)
+        except ImportError:
             raise ImportError(
                 "Azure packages not installed. Run `pip install deeplake[azure]`."
             )
@@ -42,6 +36,8 @@ class AzureProvider(StorageProvider):
         self.expiration: Optional[str] = None
         self.db_engine: bool = False
         self.repository: Optional[str] = None
+        self.hub_path = ""
+        self.tag = ""
 
     def _set_attrs(self):
         self.account_name, self.container_name, self.root_folder = self._get_attrs(
@@ -66,6 +62,9 @@ class AzureProvider(StorageProvider):
         return account_name, container_name, root_folder
 
     def _set_credential(self, creds: Dict[str, str]):
+        from azure.identity import DefaultAzureCredential  # type: ignore
+        from azure.core.credentials import AzureNamedKeyCredential, AzureSasCredential
+
         self.account_name = (
             creds.get("account_name") or self.account_name
         )  # account name in creds can override account name in path
@@ -83,6 +82,8 @@ class AzureProvider(StorageProvider):
             self.credential = DefaultAzureCredential()
 
     def _set_clients(self):
+        from azure.storage.blob import BlobServiceClient  # type: ignore
+
         self.blob_service_client = BlobServiceClient(
             self.account_url, credential=self.credential
         )
@@ -90,7 +91,7 @@ class AzureProvider(StorageProvider):
             self.container_name
         )
 
-    def __setitem__(self, path, content):
+    def _set(self, path, content):
         self.check_readonly()
         self._check_update_creds()
         if isinstance(content, memoryview):
@@ -102,11 +103,53 @@ class AzureProvider(StorageProvider):
         )
         blob_client.upload_blob(content, overwrite=True)
 
-    def __getitem__(self, path):
-        return self.get_bytes(path)
+    def __setitem__(self, path, content):
+        from azure.core.exceptions import ClientAuthenticationError  # type: ignore
 
-    def __delitem__(self, path):
+        try:
+            self._set(path, content)
+        except ClientAuthenticationError as ex:
+            tries = 5
+            retry_wait = 0
+            for i in range(1, tries + 1):
+                always_warn(f"Encountered connection error, retry {i} out of {tries}")
+                self._check_update_creds(force=True)
+                retry_wait = self._retry_wait_and_extend(retry_wait, ex)
+                try:
+                    self._set(path, content)
+                    always_warn(
+                        f"Connection re-established after {i} {['retries', 'retry'][i==1]}."
+                    )
+                    return
+                except Exception:
+                    pass
+            raise ex
+
+    def __getitem__(self, path):
+        from azure.core.exceptions import ClientAuthenticationError  # type: ignore
+
+        try:
+            return self.get_bytes(path)
+        except ClientAuthenticationError as ex:
+            tries = 5
+            retry_wait = 0
+            for i in range(1, tries + 1):
+                always_warn(f"Encountered connection error, retry {i} out of {tries}")
+                self._check_update_creds(force=True)
+                retry_wait = self._retry_wait_and_extend(retry_wait, ex)
+                try:
+                    res = self.get_bytes(path)
+                    always_warn(
+                        f"Connection re-established after {i} {['retries', 'retry'][i==1]}."
+                    )
+                    return res
+                except Exception:
+                    pass
+            raise ex
+
+    def _del(self, path):
         self.check_readonly()
+        self._check_update_creds()
         blob_client = self.container_client.get_blob_client(
             f"{self.root_folder}/{path}"
         )
@@ -114,6 +157,28 @@ class AzureProvider(StorageProvider):
             raise KeyError(path)
         blob_client.delete_blob()
 
+    def __delitem__(self, path):
+        from azure.core.exceptions import ClientAuthenticationError  # type: ignore
+
+        try:
+            return self._del(path)
+        except ClientAuthenticationError as ex:
+            tries = 5
+            retry_wait = 0
+            for i in range(1, tries + 1):
+                always_warn(f"Encountered connection error, retry {i} out of {tries}")
+                self._check_update_creds(force=True)
+                retry_wait = self._retry_wait_and_extend(retry_wait, ex)
+                try:
+                    self._del(path)
+                    always_warn(
+                        f"Connection re-established after {i} {['retries', 'retry'][i==1]}."
+                    )
+                    return
+                except Exception:
+                    pass
+            raise ex
+
     def get_bytes(
         self,
         path: str,
@@ -141,18 +206,76 @@ class AzureProvider(StorageProvider):
         )
         if not blob_client.exists():
             raise KeyError(path)
+        # Azure raises an error when trying to download an empty blob
+        if blob_client.get_blob_properties().size == 0:
+            return b""
         byts = blob_client.download_blob(offset=offset, length=length).readall()
         return byts
 
-    def clear(self, prefix=""):
+    def _clear(self, prefix=""):
         self.check_readonly()
         self._check_update_creds()
-        blobs = [
-            posixpath.join(self.root_folder, key) for key in self._all_keys(prefix)
-        ]
-        self.container_client.delete_blobs(*blobs)
+
+        blobs_to_delete = []
+        directories_to_delete = []
+
+        try:
+            blob_list = self.container_client.list_blobs(name_starts_with=prefix)
+            for blob in blob_list:
+                if blob.size == 0:
+                    directories_to_delete.append(blob.name)
+                else:
+                    blobs_to_delete.append(blob.name)
+        except Exception as e:
+            return
+
+        if blobs_to_delete:
+            batches = [
+                blobs_to_delete[i : i + 256]
+                for i in range(0, len(blobs_to_delete), 256)
+            ]
+            for batch in batches:
+                try:
+                    self.container_client.delete_blobs(*batch)
+                except Exception as e:
+                    for blob_name in batch:
+                        try:
+                            self.container_client.delete_blob(blob_name)
+                        except Exception:
+                            pass
+
+        directories_to_delete = sorted(directories_to_delete, key=len, reverse=True)
+        for directory in directories_to_delete:
+            try:
+                self.container_client.delete_blob(directory)
+            except Exception:
+                pass
+
+    def clear(self, prefix=""):
+        from azure.core.exceptions import ClientAuthenticationError  # type: ignore
+
+        try:
+            return self._clear(prefix)
+        except ClientAuthenticationError as ex:
+            tries = 5
+            retry_wait = 0
+            for i in range(1, tries + 1):
+                always_warn(f"Encountered connection error, retry {i} out of {tries}")
+                self._check_update_creds(force=True)
+                retry_wait = self._retry_wait_and_extend(retry_wait, ex)
+                try:
+                    self._clear(prefix)
+                    always_warn(
+                        f"Connection re-established after {i} {['retries', 'retry'][i==1]}."
+                    )
+                    return
+                except Exception as ex:
+                    pass
+            raise ex
 
     def get_sas_token(self):
+        from azure.storage.blob import generate_container_sas, ContainerSasPermissions  # type: ignore
+
         self._check_update_creds()
         if self.sas_token:
             return self.sas_token
@@ -201,6 +324,8 @@ class AzureProvider(StorageProvider):
             "db_engine": self.db_engine,
             "repository": self.repository,
             "expiration": self.expiration,
+            "tag": getattr(self, "tag", ""),
+            "hub_path": getattr(self, "hub_path", ""),
         }
 
     def __setstate__(self, state):
@@ -269,7 +394,7 @@ class AzureProvider(StorageProvider):
             source_blob.delete_blob()
         self.root_folder = root_folder
 
-    def get_object_size(self, path: str) -> int:
+    def _get_object_size(self, path: str) -> int:
         self._check_update_creds()
         blob_client = self.container_client.get_blob_client(
             f"{self.root_folder}/{path}"
@@ -278,7 +403,31 @@ class AzureProvider(StorageProvider):
             raise KeyError(path)
         return blob_client.get_blob_properties().size
 
+    def get_object_size(self, path: str) -> int:
+        from azure.core.exceptions import ClientAuthenticationError  # type: ignore
+
+        try:
+            return self._get_object_size(path)
+        except ClientAuthenticationError as ex:
+            tries = 5
+            retry_wait = 0
+            for i in range(1, tries + 1):
+                always_warn(f"Encountered connection error, retry {i} out of {tries}")
+                self._check_update_creds(force=True)
+                retry_wait = self._retry_wait_and_extend(retry_wait, ex)
+                try:
+                    res = self._get_object_size(path)
+                    always_warn(
+                        f"Connection re-established after {i} {['retries', 'retry'][i==1]}."
+                    )
+                    return res
+                except Exception:
+                    pass
+            raise ex
+
     def get_clients_from_full_path(self, url: str):
+        from azure.storage.blob import BlobServiceClient  # type: ignore
+
         self._check_update_creds()
         account_name, container_name, blob_path = self._get_attrs(url)
         account_url = f"https://{account_name}.blob.core.windows.net"
@@ -289,6 +438,9 @@ class AzureProvider(StorageProvider):
         return blob_client, blob_service_client
 
     def get_presigned_url(self, path: str, full: bool = False) -> str:
+        from azure.core.credentials import AzureSasCredential  # type: ignore
+        from azure.storage.blob import BlobSasPermissions, generate_blob_sas  # type: ignore
+
         self._check_update_creds()
         if full:
             blob_client, blob_service_client = self.get_clients_from_full_path(path)
@@ -319,33 +471,66 @@ class AzureProvider(StorageProvider):
                 org_id, ds_name = self.tag.split("/")  # type: ignore
                 url = client.get_presigned_url(org_id, ds_name, path)
             else:
-                user_delegation_key = blob_service_client.get_user_delegation_key(
-                    datetime.now(timezone.utc),
-                    datetime.now(timezone.utc) + timedelta(hours=1),
-                )
-                sas_token = generate_blob_sas(
-                    account_name,
-                    container_name,
-                    blob_path,
-                    user_delegation_key=user_delegation_key,
-                    permission=BlobSasPermissions(read=True),
-                    expiry=datetime.now(timezone.utc) + timedelta(hours=1),
-                )
+                if not isinstance(self.credential, AzureSasCredential):
+                    user_delegation_key = blob_service_client.get_user_delegation_key(
+                        datetime.now(timezone.utc),
+                        datetime.now(timezone.utc) + timedelta(hours=1),
+                    )
+                    sas_token = generate_blob_sas(
+                        account_name,
+                        container_name,
+                        blob_path,
+                        user_delegation_key=user_delegation_key,
+                        permission=BlobSasPermissions(read=True),
+                        expiry=datetime.now(timezone.utc) + timedelta(hours=1),
+                    )
+                else:
+                    sas_token = self.credential.signature
+
                 url = f"{account_url}/{container_name}/{blob_path}?{sas_token}"
             self._presigned_urls[path] = (url, time.time())
 
         return url
 
-    def get_object_from_full_url(self, url: str) -> bytes:
+    def _get_object_from_full_url(self, url: str) -> bytes:
+        self._check_update_creds()
         blob_client, _ = self.get_clients_from_full_path(url)
+        # Azure raises an error when trying to download an empty blob
+        if blob_client.get_blob_properties().size == 0:
+            return b""
         return blob_client.download_blob().readall()
 
+    def get_object_from_full_url(self, url: str) -> bytes:
+        from azure.core.exceptions import ClientAuthenticationError  # type: ignore
+
+        try:
+            return self._get_object_from_full_url(url)
+        except ClientAuthenticationError as ex:
+            tries = 5
+            retry_wait = 0
+            for i in range(1, tries + 1):
+                always_warn(f"Encountered connection error, retry {i} out of {tries}")
+                self._check_update_creds(force=True)
+                retry_wait = self._retry_wait_and_extend(retry_wait, ex)
+                try:
+                    res = self._get_object_from_full_url(url)
+                    always_warn(
+                        f"Connection re-established after {i} {['retries', 'retry'][i==1]}."
+                    )
+                    return res
+                except Exception:
+                    pass
+            raise ex
+
     def _check_update_creds(self, force=False):
         """If the client has an expiration time, check if creds are expired and fetch new ones.
         This would only happen for datasets stored on Deep Lake storage for which temporary 12 hour credentials are generated.
         """
+        from azure.core.credentials import AzureSasCredential  # type: ignore
+
         if self.expiration and (
-            force or float(self.expiration) < datetime.now(timezone.utc).timestamp()
+            force
+            or float(self.expiration) - datetime.now(timezone.utc).timestamp() < 300
         ):
             client = DeepLakeBackendClient(self.token)
             org_id, ds_name = self.tag.split("/")
@@ -364,3 +549,27 @@ class AzureProvider(StorageProvider):
             self.sas_token = creds.get("sas_token")
             if self.sas_token:
                 self.credential = AzureSasCredential(self.sas_token)
+
+            self._set_clients()
+
+    def get_items(self, keys):
+        with ThreadPoolExecutor() as executor:
+            future_to_key = {
+                executor.submit(self.__getitem__, key): key for key in keys
+            }
+
+            for future in futures.as_completed(future_to_key):
+                key = future_to_key[future]
+                exception = future.exception()
+
+                if not exception:
+                    yield key, future.result()
+                else:
+                    yield key, exception
+
+    def _retry_wait_and_extend(self, retry_wait: int, err: Exception):
+        time.sleep(retry_wait)
+
+        if retry_wait == 0:
+            return 1
+        return retry_wait * 2
