@@ -1,4 +1,3 @@
-import datetime
 import posixpath
 import pickle
 import json
@@ -6,21 +5,9 @@ import os
 import tempfile
 import time
 from typing import Dict, Optional, Tuple, Union
+from datetime import timezone, datetime, timedelta
 
-try:
-    from google.cloud import storage  # type: ignore
-    from google.api_core import retry  # type: ignore
-    from google.oauth2 import service_account  # type: ignore
-    import google.auth as gauth  # type: ignore
-    import google.auth.compute_engine  # type: ignore
-    import google.auth.credentials  # type: ignore
-    import google.auth.exceptions  # type: ignore
-    from google_auth_oauthlib.flow import InstalledAppFlow  # type: ignore
-    from google.api_core.exceptions import NotFound  # type: ignore
-
-    _GOOGLE_PACKAGES_INSTALLED = True
-except ImportError:
-    _GOOGLE_PACKAGES_INSTALLED = False
+from deeplake.util.path import relpath
 
 
 from deeplake.core.storage.provider import StorageProvider
@@ -68,6 +55,8 @@ class GCloudCredentials:
             ValueError: If the name of the default project doesn't match the GCSProvider project name.
             DefaultCredentialsError: If no credentials are found.
         """
+        import google.auth as gauth  # type: ignore
+
         credentials, project = gauth.default(scopes=[self.scope])
         if self.project and self.project != project:
             raise ValueError(
@@ -98,6 +87,24 @@ class GCloudCredentials:
             json.dump(token, f)
         return token_file.name
 
+    def _connect_from_dict(self, token: Dict):
+        """
+        Connect using a dictionary type token.
+
+        Distinguishes between service account and user account tokens.
+        Args:
+            token (Dict): dictionary with token to be stored in either .json Service file, or credentials combination.
+        """
+        from google.oauth2.credentials import Credentials  # type: ignore
+
+        if "gcs_oauth_token" in token:
+            self.credentials = Credentials(token["gcs_oauth_token"])  # type: ignore
+            self.project = token.get("project_id", None)
+            return
+
+        service_file = self._dict_to_credentials(token)
+        self._connect_service(service_file)
+
     def _connect_token(self, token: Optional[Union[str, Dict]] = None):
         """
         Connect using a concrete token.
@@ -111,6 +118,8 @@ class GCloudCredentials:
             FileNotFoundError: If token file doesn't exist.
             ValueError: If token format isn't supported by gauth.
         """
+        import google.auth.credentials  # type: ignore
+
         if isinstance(token, str):
             if not os.path.exists(token):
                 raise FileNotFoundError(token)
@@ -120,16 +129,18 @@ class GCloudCredentials:
             except:
                 token = json.load(open(token))
         if isinstance(token, dict):
-            token = self._dict_to_credentials(token)
-            self._connect_service(token)
+            self._connect_from_dict(token)
             return
         elif isinstance(token, google.auth.credentials.Credentials):
             credentials = token
         else:
             raise ValueError("Token format not understood")
+
         self.credentials = credentials
 
     def _connect_service(self, fn):
+        from google.oauth2 import service_account  # type: ignore
+
         credentials = service_account.Credentials.from_service_account_file(
             fn, scopes=[self.scope]
         )
@@ -143,6 +154,8 @@ class GCloudCredentials:
         Raises:
             GCSDefaultCredsNotFoundError: if application deafault credentials can't be found.
         """
+        from google_auth_oauthlib.flow import InstalledAppFlow  # type: ignore
+
         try:
             if os.name == "nt":
                 path = os.path.join(
@@ -225,6 +238,9 @@ class GCloudCredentials:
             return {"json_credentials": json.dumps(token)}
         return {}
 
+    def are_credentials_downscoped(self):
+        return self.token is not None and "gcs_oauth_token" in self.token
+
 
 class GCSProvider(StorageProvider):
     """Provider class for using GC storage."""
@@ -232,7 +248,8 @@ class GCSProvider(StorageProvider):
     def __init__(
         self,
         root: str,
-        token: Optional[Union[str, Dict]] = None,
+        creds: Optional[Union[str, Dict]] = None,
+        token: Optional[str] = None,
         project: Optional[str] = None,
     ):
         """Initializes the GCSProvider.
@@ -242,23 +259,35 @@ class GCSProvider(StorageProvider):
 
         Args:
             root (str): The root of the provider. All read/write request keys will be appended to root.
-            token (str/Dict): GCP token, used for fetching credentials for storage).
+            creds (str/Dict): GCP creds, used for fetching credentials for storage).
                 Can be a path to the credentials file, actual credential dictionary or one of the folowing:
                 - ``google_default``: Tries to load default credentials for the specified project.
                 - ``cache``: Retrieves the previously used credentials from cache if exist.
                 - ``anon``: Sets ``credentials=None``.
-                - ``browser``: Generates and stores new token file using cli.
+                - ``browser``: Generates and stores new creds file using cli.
+            token (str, optional): Activeloop token, used for fetching credentials for Deep Lake datasets (if this is underlying storage for Deep Lake dataset).
+                This is optional, tokens are normally autogenerated.
             project (str): Name of the project from GCloud.
 
         Raises:
             ModuleNotFoundError: If google cloud packages aren't installed.
         """
-        if not _GOOGLE_PACKAGES_INSTALLED:
+
+        try:
+            import google.cloud.storage  # type: ignore
+            import google.api_core  # type: ignore
+            import google.oauth2  # type: ignore
+            import google.auth  # type: ignore
+            import google_auth_oauthlib  # type: ignore
+            from google.api_core.exceptions import NotFound  # type: ignore
+        except ImportError:
             raise ModuleNotFoundError(
                 "Google cloud packages are not installed. Run `pip install deeplake[gcp]`."
             )
+
         self.root = root
-        self.token: Union[str, Dict, None] = token
+        self.token: Union[str, Dict, None] = creds
+        self.activeloop_token = token
         self.tag: Optional[str] = None
         self.project = project
         self.missing_exceptions = (
@@ -268,6 +297,8 @@ class GCSProvider(StorageProvider):
             AttributeError,
             NotFound,
         )
+        self.org_id: Optional[str] = None
+        self.creds_key: Optional[str] = None
         self._initialize_provider()
         self._presigned_urls: Dict[str, Tuple[str, float]] = {}
         self.expiration: Optional[str] = None
@@ -277,25 +308,77 @@ class GCSProvider(StorageProvider):
     def subdir(self, path: str, read_only: bool = False):
         sd = self.__class__(
             root=posixpath.join(self.root, path),
-            token=self.token,
+            creds=self.token,
+            token=self.activeloop_token,
             project=self.project,
         )
-        if self.expiration:
+        if hasattr(self, "expiration") and self.expiration:
             sd._set_hub_creds_info(
                 self.hub_path, self.expiration, self.db_engine, self.repository
             )
         sd.read_only = read_only
         return sd
 
+    def _check_update_creds(self, force=False):
+        """If the client has an expiration time, check if creds are expired and fetch new ones.
+        This would only happen for datasets stored on Deep Lake storage for which temporary credentials are generated.
+        """
+
+        if (
+            hasattr(self, "expiration")
+            and self.expiration
+            and (
+                force or float(self.expiration) < datetime.now(timezone.utc).timestamp()
+            )
+        ):
+            client = DeepLakeBackendClient(self.activeloop_token)
+            org_id, ds_name = self.tag.split("/")
+
+            mode = "r" if self.read_only else "a"
+
+            url, creds, mode, expiration, repo = client.get_dataset_credentials(
+                org_id,
+                ds_name,
+                mode,
+                {"enabled": self.db_engine},
+                True,
+            )
+            self.expiration = expiration
+            self.repository = repo
+            self.token = creds
+            self._initialize_provider()
+
     def _initialize_provider(self):
+        from google.cloud import storage  # type: ignore
+        from google.api_core import retry  # type: ignore
+
+        # In case the storage provider is used with Managed Credentials
+        if isinstance(self.token, dict):
+            self.org_id = self.token.pop("org_id", None)
+            self.creds_key = self.token.pop("creds_key", None)
+
         self._set_bucket_and_path()
         if not self.token:
             self.token = None
         self.scoped_credentials = GCloudCredentials(self.token, project=self.project)
         self.retry = retry.Retry(deadline=60)
-        self.client = storage.Client(credentials=self.scoped_credentials.credentials)
+        kwargs = {}
+        if self.scoped_credentials.project:
+            kwargs["project"] = self.scoped_credentials.project
+
+        self.client = storage.Client(
+            credentials=self.scoped_credentials.credentials, **kwargs
+        )
         self._client_bucket = None
 
+    @property
+    def creds_key(self) -> Optional[str]:
+        return self._creds_key
+
+    @creds_key.setter
+    def creds_key(self, creds_key: Optional[str]):
+        self._creds_key = creds_key
+
     @property
     def client_bucket(self):
         if self._client_bucket is None:
@@ -317,6 +400,7 @@ class GCSProvider(StorageProvider):
         return posixpath.join(self.path, key)
 
     def _all_keys(self):
+        self._check_update_creds()
         self._blob_objects = self.client_bucket.list_blobs(prefix=self.path)
         return {posixpath.relpath(obj.name, self.path) for obj in self._blob_objects}
 
@@ -349,6 +433,7 @@ class GCSProvider(StorageProvider):
             Exercise caution!
         """
         self.check_readonly()
+        self._check_update_creds()
         path = posixpath.join(self.path, prefix) if prefix else self.path
         blob_objects = self.client_bucket.list_blobs(prefix=path)
         for blob in blob_objects:
@@ -360,6 +445,7 @@ class GCSProvider(StorageProvider):
     def rename(self, root):
         """Rename root folder."""
         self.check_readonly()
+        self._check_update_creds()
         path = _remove_protocol_from_path(root)
         new_bucket, new_path = path.split("/", 1)
         if new_bucket != self.client_bucket.name:
@@ -369,7 +455,7 @@ class GCSProvider(StorageProvider):
         for blob in dest_objects:
             raise PathNotEmptyException(use_hub=False)
         for blob in blob_objects:
-            new_key = "/".join([new_path, posixpath.relpath(blob.name, self.path)])
+            new_key = "/".join([new_path, relpath(blob.name, self.path)])
             self.client_bucket.rename_blob(blob, new_key)
 
         self.root = root
@@ -401,6 +487,7 @@ class GCSProvider(StorageProvider):
             InvalidBytesRequestedError: If ``start_byte`` > ``end_byte`` or ``start_byte`` < 0 or ``end_byte`` < 0.
             KeyError: If an object is not found at the path.
         """
+        self._check_update_creds()
         try:
             blob = self.client_bucket.get_blob(self._get_path_from_key(path))
             if end_byte is not None:
@@ -414,6 +501,7 @@ class GCSProvider(StorageProvider):
     def __setitem__(self, key, value):
         """Store value in key."""
         self.check_readonly()
+        self._check_update_creds()
         blob = self.client_bucket.blob(self._get_path_from_key(key))
         if isinstance(value, memoryview):
             value = value.tobytes()
@@ -427,11 +515,13 @@ class GCSProvider(StorageProvider):
 
     def __len__(self):
         """Returns length of the structure."""
+        self._check_update_creds()
         return len(self._all_keys())
 
     def __delitem__(self, key):
         """Remove key."""
         self.check_readonly()
+        self._check_update_creds()
         blob = self.client_bucket.blob(self._get_path_from_key(key))
         try:
             blob.delete()
@@ -440,6 +530,8 @@ class GCSProvider(StorageProvider):
 
     def __contains__(self, key):
         """Checks if key exists in mapping."""
+        from google.cloud import storage  # type: ignore
+
         stats = storage.Blob(
             bucket=self.client_bucket, name=self._get_path_from_key(key)
         ).exists(self.client_bucket.client)
@@ -467,6 +559,17 @@ class GCSProvider(StorageProvider):
         self._initialize_provider()
 
     def get_presigned_url(self, key, full=False):
+        """
+        Generate a presigned URL for accessing an object in GCS.
+
+        Args:
+            key (str): The key for the object.
+            full (bool): Whether the key is a full path or relative to the root.
+
+        Returns:
+            str: The presigned URL.
+        """
+        self._check_update_creds()
         if full:
             root = _remove_protocol_from_path(key)
             split_root = root.split("/", 1)
@@ -488,22 +591,34 @@ class GCSProvider(StorageProvider):
 
         if url is None:
             if self._is_hub_path:
-                client = DeepLakeBackendClient(self.token)  # type: ignore
+                client = DeepLakeBackendClient(self.activeloop_token)  # type: ignore
                 org_id, ds_name = self.tag.split("/")  # type: ignore
                 url = client.get_presigned_url(org_id, ds_name, key)
             else:
-                blob = client_bucket.get_blob(
-                    self._get_path_from_key(key) if not full else key
-                )
-                url = blob.generate_signed_url(datetime.timedelta(seconds=3600))
+                if self.scoped_credentials.are_credentials_downscoped():
+                    client = DeepLakeBackendClient(self.activeloop_token)
+                    url = client.get_blob_presigned_url(
+                        org_id=self.org_id,
+                        creds_key=self.creds_key,
+                        blob_path=f"gcs://{root}",
+                    )
+                else:
+                    blob = client_bucket.blob(
+                        self._get_path_from_key(key) if not full else key
+                    )
+                    url = blob.generate_signed_url(expiration=timedelta(hours=1))
+
             self._presigned_urls[key] = (url, time.time())
+
         return url
 
     def get_object_size(self, key: str) -> int:
+        self._check_update_creds()
         blob = self.client_bucket.get_blob(self._get_path_from_key(key))
         return blob.size
 
     def get_object_from_full_url(self, url: str):
+        self._check_update_creds()
         root = _remove_protocol_from_path(url)
         split_root = root.split("/", 1)
         bucket = split_root[0]
@@ -518,6 +633,12 @@ class GCSProvider(StorageProvider):
             raise KeyError(path)
 
     def get_creds(self):
-        d = self.scoped_credentials.get_token_info()
-        d["expiration"] = self.expiration or ""
+        if self.scoped_credentials.are_credentials_downscoped:
+            d = self.scoped_credentials.token
+        else:
+            d = self.scoped_credentials.get_token_info()
+
+        d["expiration"] = (
+            self.expiration if hasattr(self, "expiration") and self.expiration else ""
+        )
         return d
