@@ -6,6 +6,7 @@ from deeplake.core.chunk.base_chunk import BaseChunk
 from typing import Any, Dict, Optional, Union
 
 from deeplake.core.storage.provider import StorageProvider
+import time
 
 
 def _get_nbytes(obj: Union[bytes, memoryview, DeepLakeMemoryObject]):
@@ -185,6 +186,15 @@ class LRUCache(StorageProvider):
 
         raise ValueError(f"Item at '{path}' got an invalid type: '{type(item)}'.")
 
+    def _get_item_from_cache(self, path: str):
+        if path in self.deeplake_objects:
+            if path in self.lru_sizes:
+                self.lru_sizes.move_to_end(path)  # refresh position for LRU
+            return self.deeplake_objects[path]
+        elif path in self.lru_sizes:
+            self.lru_sizes.move_to_end(path)  # refresh position for LRU
+            return self.cache_storage[path]
+
     def __getitem__(self, path: str):
         """If item is in cache_storage, retrieves from there and returns.
         If item isn't in cache_storage, retrieves from next storage, stores in cache_storage (if possible) and returns.
@@ -198,13 +208,9 @@ class LRUCache(StorageProvider):
         Returns:
             bytes: The bytes of the object present at the path.
         """
-        if path in self.deeplake_objects:
-            if path in self.lru_sizes:
-                self.lru_sizes.move_to_end(path)  # refresh position for LRU
-            return self.deeplake_objects[path]
-        elif path in self.lru_sizes:
-            self.lru_sizes.move_to_end(path)  # refresh position for LRU
-            return self.cache_storage[path]
+        result = self._get_item_from_cache(path)
+        if result is not None:
+            return result
         else:
             if self.next_storage is not None:
                 # fetch from storage, may throw KeyError
@@ -215,6 +221,16 @@ class LRUCache(StorageProvider):
                 return result
             raise KeyError(path)
 
+    def get_items(self, paths):
+        """Pre-load items from next storage into cache"""
+        if self.next_storage is not None:
+            for key, result in self.next_storage.get_items(paths):
+                if isinstance(result, KeyError):
+                    continue
+                if _get_nbytes(result) <= self.cache_size:
+                    self._insert_in_cache(key, result)
+                yield key, result
+
     def get_bytes(
         self,
         path: str,
@@ -277,17 +293,7 @@ class LRUCache(StorageProvider):
 
         self.maybe_flush()
 
-    def __delitem__(self, path: str):
-        """Deletes the object present at the path from the cache and the underlying storage.
-
-        Args:
-            path (str): the path to the object relative to the root of the provider.
-
-        Raises:
-            KeyError: If an object is not found at the path.
-            ReadOnlyError: If the provider is in read-only mode.
-        """
-        self.check_readonly()
+    def _del_item_from_cache(self, path: str):
         deleted_from_cache = False
 
         if path in self.deeplake_objects:
@@ -301,6 +307,21 @@ class LRUCache(StorageProvider):
             self.dirty_keys.pop(path, None)
             deleted_from_cache = True
 
+        return deleted_from_cache
+
+    def __delitem__(self, path: str):
+        """Deletes the object present at the path from the cache and the underlying storage.
+
+        Args:
+            path (str): the path to the object relative to the root of the provider.
+
+        Raises:
+            KeyError: If an object is not found at the path.
+            ReadOnlyError: If the provider is in read-only mode.
+        """
+        self.check_readonly()
+        deleted_from_cache = self._del_item_from_cache(path)
+
         try:
             if self.next_storage is not None:
                 del self.next_storage[path]
@@ -451,6 +472,7 @@ class LRUCache(StorageProvider):
             "next_storage": self.next_storage,
             "cache_storage": self.cache_storage,
             "cache_size": self.cache_size,
+            "use_async": self.use_async,
         }
 
     def __setstate__(self, state: Dict[str, Any]):
@@ -469,6 +491,7 @@ class LRUCache(StorageProvider):
         self.next_storage = state["next_storage"]
         self.cache_storage = state["cache_storage"]
         self.cache_size = state["cache_size"]
+        self.use_async = state["use_async"]
         self.lru_sizes = OrderedDict()
         self.dirty_keys = OrderedDict()
         self.cache_used = 0
