@@ -1,13 +1,17 @@
 import deeplake
 from math import ceil
+from io import BytesIO
 import time
 import boto3
 import botocore  # type: ignore
 import posixpath
-from typing import Dict, Optional, Tuple, Type
+import ssl
+import urllib3
+from typing import Dict, Optional, Tuple, Type, Any
 from datetime import datetime, timezone
 from botocore.session import ComponentLocator
 from deeplake.client.client import DeepLakeBackendClient
+from deeplake.constants import GB
 from deeplake.core.storage.provider import StorageProvider
 from deeplake.util.exceptions import (
     S3GetAccessError,
@@ -17,6 +21,7 @@ from deeplake.util.exceptions import (
     S3Error,
     PathNotEmptyException,
 )
+from deeplake.util.path import relpath
 from deeplake.util.warnings import always_warn
 from botocore.exceptions import (
     ReadTimeoutError,
@@ -25,7 +30,10 @@ from botocore.exceptions import (
     ConnectionClosedError,
     EndpointConnectionError,
     IncompleteReadError,
+    SSLError,
 )
+from concurrent import futures
+from concurrent.futures import ThreadPoolExecutor
 
 CONNECTION_ERRORS = (
     ReadTimeoutError,
@@ -34,6 +42,9 @@ CONNECTION_ERRORS = (
     ConnectionClosedError,
     EndpointConnectionError,
     IncompleteReadError,
+    SSLError,
+    ssl.SSLError,
+    urllib3.exceptions.SSLError,
 )
 
 try:
@@ -87,6 +98,7 @@ class S3Provider(StorageProvider):
         aws_region: Optional[str] = None,
         profile_name: Optional[str] = None,
         token: Optional[str] = None,
+        config: Optional[Any] = None,
         **kwargs,
     ):
         """Initializes the S3Provider
@@ -109,6 +121,7 @@ class S3Provider(StorageProvider):
             profile_name (str, optional): Specifies the AWS profile name to use.
             token (str, optional): Activeloop token, used for fetching credentials for Deep Lake datasets (if this is underlying storage for Deep Lake dataset).
                 This is optional, tokens are normally autogenerated.
+            config  (Any, Optional): s3 client configuration provided by the user. Defaults to None.
             **kwargs: Additional arguments to pass to the S3 client. Includes: ``expiration``.
         """
         self.root = root
@@ -123,7 +136,7 @@ class S3Provider(StorageProvider):
         self.tag: Optional[str] = None
         self.token: Optional[str] = token
         self.loaded_creds_from_environment = False
-        self.client_config = deeplake.config["s3"]
+        self.client_config = config
         self.start_time = time.time()
         self.profile_name = profile_name
         self._initialize_s3_parameters()
@@ -150,12 +163,16 @@ class S3Provider(StorageProvider):
         return sd
 
     def _set(self, path, content):
-        self.client.put_object(
-            Bucket=self.bucket,
-            Body=content,
-            Key=path,
-            ContentType="application/octet-stream",  # signifies binary data
-        )
+        if len(content) >= 1 * GB:
+            stream = BytesIO(content)
+            self.client.upload_fileobj(stream, self.bucket, path)
+        else:
+            self.client.put_object(
+                Bucket=self.bucket,
+                Body=content,
+                Key=path,
+                ContentType="application/octet-stream",  # signifies binary data
+            )
 
     def __setitem__(self, path, content):
         """Sets the object present at the path with the value
@@ -171,7 +188,7 @@ class S3Provider(StorageProvider):
         self.check_readonly()
         self._check_update_creds()
         path = "".join((self.path, path))
-        content = bytearray(memoryview(content))
+        content = bytes(memoryview(content))
         try:
             self._set(path, content)
         except botocore.exceptions.ClientError as err:
@@ -179,8 +196,10 @@ class S3Provider(StorageProvider):
                 self._set(path, content)
         except CONNECTION_ERRORS as err:
             tries = self.num_tries
+            retry_wait = 0
             for i in range(1, tries + 1):
                 always_warn(f"Encountered connection error, retry {i} out of {tries}")
+                retry_wait = self._retry_wait_and_extend(retry_wait, err)
                 try:
                     self._set(path, content)
                     always_warn(
@@ -219,17 +238,16 @@ class S3Provider(StorageProvider):
     def _get_bytes(
         self, path, start_byte: Optional[int] = None, end_byte: Optional[int] = None
     ):
+        range_kwarg = {}
         if start_byte is not None and end_byte is not None:
             if start_byte == end_byte:
                 return b""
-            range = f"bytes={start_byte}-{end_byte - 1}"
+            range_kwarg["Range"] = f"bytes={start_byte}-{end_byte - 1}"
         elif start_byte is not None:
-            range = f"bytes={start_byte}-"
+            range_kwarg["Range"] = f"bytes={start_byte}-"
         elif end_byte is not None:
-            range = f"bytes=0-{end_byte - 1}"
-        else:
-            range = ""
-        resp = self.client.get_object(Bucket=self.bucket, Key=path, Range=range)
+            range_kwarg["Range"] = f"bytes=0-{end_byte - 1}"
+        resp = self.client.get_object(Bucket=self.bucket, Key=path, **range_kwarg)
         return resp["Body"].read()
 
     def get_bytes(
@@ -269,8 +287,11 @@ class S3Provider(StorageProvider):
                 return self._get_bytes(path, start_byte, end_byte)
         except CONNECTION_ERRORS as err:
             tries = self.num_tries
+            retry_wait = 0
             for i in range(1, tries + 1):
                 always_warn(f"Encountered connection error, retry {i} out of {tries}")
+                retry_wait = self._retry_wait_and_extend(retry_wait, err)
+
                 try:
                     ret = self._get_bytes(path, start_byte, end_byte)
                     always_warn(
@@ -311,8 +332,11 @@ class S3Provider(StorageProvider):
                 self._del(path)
         except CONNECTION_ERRORS as err:
             tries = self.num_tries
+            retry_wait = 0
             for i in range(1, tries + 1):
                 always_warn(f"Encountered connection error, retry {i} out of {tries}")
+                retry_wait = self._retry_wait_and_extend(retry_wait, err)
+
                 try:
                     self._del(path)
                     always_warn(
@@ -327,7 +351,7 @@ class S3Provider(StorageProvider):
 
     @property
     def num_tries(self):
-        return min(ceil((time.time() - self.start_time) / 300), 5)
+        return max(3, min(ceil((time.time() - self.start_time) / 300), 5))
 
     def _keys_iterator(self):
         self._check_update_creds()
@@ -377,6 +401,12 @@ class S3Provider(StorageProvider):
         self._check_update_creds()
         yield from self._all_keys()
 
+    def _clear(self, prefix):
+        bucket = self.resource.Bucket(self.bucket)
+        for response in bucket.objects.filter(Prefix=prefix).delete():
+            if response.get("Errors"):
+                raise S3DeletionError(response["Errors"][0]["Message"])
+
     def clear(self, prefix=""):
         """Deletes ALL data with keys having given prefix on the s3 bucket (under self.root).
 
@@ -388,12 +418,10 @@ class S3Provider(StorageProvider):
         path = posixpath.join(self.path, prefix) if prefix else self.path
         if self.resource is not None:
             try:
-                bucket = self.resource.Bucket(self.bucket)
-                bucket.objects.filter(Prefix=path).delete()
+                self._clear(path)
             except Exception as err:
                 with S3ResetReloadCredentialsManager(self, S3DeletionError):
-                    bucket = self.resource.Bucket(self.bucket)
-                    bucket.objects.filter(Prefix=self.path).delete()
+                    self._clear(path)
 
         else:
             super().clear()
@@ -420,7 +448,7 @@ class S3Provider(StorageProvider):
         for item in items:
             old_key = item["Key"]
             copy_source = {"Bucket": self.bucket, "Key": old_key}
-            new_key = "/".join([new_path, posixpath.relpath(old_key, self.path)])
+            new_key = "/".join([new_path, relpath(old_key, self.path)])
             self.client.copy_object(
                 CopySource=copy_source, Bucket=self.bucket, Key=new_key
             )
@@ -610,11 +638,46 @@ class S3Provider(StorageProvider):
             self._presigned_urls[path] = (url, time.time())
         return url
 
-    def get_object_size(self, path: str) -> int:
-        path = "".join((self.path, path))
+    def _get_object_size(self, path: str) -> int:
         obj = self.resource.Object(self.bucket, path)
         return obj.content_length
 
+    def get_object_size(self, path: str) -> int:
+        self._check_update_creds()
+        path = "".join((self.path, path))
+
+        try:
+            return self._get_object_size(path)
+        except botocore.exceptions.ClientError as err:
+            if err.response["Error"]["Code"] == "NoSuchKey":
+                raise KeyError(err) from err
+            if err.response["Error"]["Code"] == "InvalidAccessKeyId":
+                new_error_cls: Type[S3GetError] = S3GetAccessError
+            else:
+                new_error_cls = S3GetError
+            with S3ResetReloadCredentialsManager(self, new_error_cls):
+                return self._get_object_size(path)
+        except CONNECTION_ERRORS as err:
+            tries = self.num_tries
+            retry_wait = 0
+            for i in range(1, tries + 1):
+                always_warn(f"Encountered connection error, retry {i} out of {tries}")
+                retry_wait = self._retry_wait_and_extend(retry_wait, err)
+
+                try:
+                    ret = self._get_object_size(path)
+                    always_warn(
+                        f"Connection re-established after {i} {['retries', 'retry'][i==1]}."
+                    )
+                    return ret
+                except Exception:
+                    pass
+            raise S3GetError(err) from err
+        except botocore.exceptions.NoCredentialsError as err:
+            raise S3GetAccessError from err
+        except Exception as err:
+            raise S3GetError(err) from err
+
     def get_object_from_full_url(self, url: str):
         root = url.replace("s3://", "")
         split_root = root.split("/", 1)
@@ -629,8 +692,11 @@ class S3Provider(StorageProvider):
                 return self._get(path, bucket)
         except CONNECTION_ERRORS as err:
             tries = self.num_tries
+            retry_wait = 0
             for i in range(1, tries + 1):
                 always_warn(f"Encountered connection error, retry {i} out of {tries}")
+                retry_wait = self._retry_wait_and_extend(retry_wait, err)
+
                 try:
                     ret = self._get(path, bucket)
                     always_warn(
@@ -669,8 +735,11 @@ class S3Provider(StorageProvider):
                 self._set_items(items)
         except CONNECTION_ERRORS as err:
             tries = self.num_tries
+            retry_wait = 0
             for i in range(1, tries + 1):
                 always_warn(f"Encountered connection error, retry {i} out of {tries}")
+                retry_wait = self._retry_wait_and_extend(retry_wait, err)
+
                 try:
                     self._set_items(items)
                     always_warn(
@@ -682,3 +751,28 @@ class S3Provider(StorageProvider):
             raise S3SetError(err) from err
         except Exception as err:
             raise S3SetError(err) from err
+
+    def get_items(self, keys):
+        with ThreadPoolExecutor() as executor:
+            future_to_key = {
+                executor.submit(self.__getitem__, key): key for key in keys
+            }
+
+            for future in futures.as_completed(future_to_key):
+                key = future_to_key[future]
+                exception = future.exception()
+
+                if not exception:
+                    yield key, future.result()
+                else:
+                    yield key, exception
+
+    def _retry_wait_and_extend(self, retry_wait: int, err: Exception):
+        if not (isinstance(err, ssl.SSLError) or isinstance(err, SSLError)):
+            return 0
+
+        time.sleep(retry_wait)
+
+        if retry_wait == 0:
+            return 1
+        return retry_wait * 2
