@@ -1,12 +1,12 @@
 import pytest
-from deeplake.util.check_installation import ray_installed
 from deeplake.util.compute import get_compute_provider
 
 schedulers = ["threaded", "processed", "serial"]
-schedulers = schedulers + ["ray"] if ray_installed() else schedulers
 all_schedulers = pytest.mark.parametrize("scheduler", schedulers)
 
 
+@pytest.mark.slow
+@pytest.mark.flaky
 @all_schedulers
 def test_compute_with_progress_bar(scheduler):
     def f(pg_callback, x):
