@@ -10,11 +10,11 @@ from deeplake.core.version_control.test_version_control import (
     get_default_dataset_diff,
 )
 from deeplake.util.remove_cache import remove_memory_cache
-from deeplake.util.check_installation import ray_installed
 from deeplake.util.exceptions import (
     AllSamplesSkippedError,
     EmptyTensorError,
     InvalidOutputDatasetError,
+    SampleExtendingError,
     TransformError,
 )
 from deeplake.tests.common import parametrize_num_workers
@@ -31,7 +31,6 @@ TRANSFORM_TEST_NUM_WORKERS = 2
 all_compressions = pytest.mark.parametrize("sample_compression", [None, "png", "jpeg"])
 
 schedulers = ["threaded", "processed"]
-schedulers = schedulers + ["ray"] if ray_installed() else schedulers
 all_schedulers = pytest.mark.parametrize("scheduler", schedulers)
 commit_or_not = pytest.mark.parametrize("do_commit", [True, False])
 
@@ -138,7 +137,7 @@ def add_image(sample_in, samples_out):
 
 @deeplake.compute
 def add_images(i, sample_out):
-    for i in range(5):
+    for _ in range(5):
         image = deeplake.read(get_dummy_data_path("images/flower.png"))
         sample_out.append({"image": image})
 
@@ -164,11 +163,15 @@ def retrieve_objects_from_memory(object_type=deeplake.core.sample.Sample):
     total_n_of_occurences = 0
     gc_objects = gc.get_objects()
     for item in gc_objects:
-        if isinstance(item, object_type):
-            total_n_of_occurences += 1
+        try:
+            if isinstance(item, object_type):
+                total_n_of_occurences += 1
+        except ReferenceError:
+            pass  # weakly-referenced object which no longer exists
     return total_n_of_occurences
 
 
+@pytest.mark.slow
 @all_schedulers
 @pytest.mark.parametrize(
     "ds",
@@ -290,6 +293,7 @@ def test_groups_2(local_ds):
         assert ds_out.x.y.z.image.shape_interval.upper == (99, 99, 99)
 
 
+@pytest.mark.slow
 @parametrize_num_workers
 @all_schedulers
 def test_single_transform_deeplake_dataset_htypes(local_ds, num_workers, scheduler):
@@ -322,6 +326,7 @@ def test_single_transform_deeplake_dataset_htypes(local_ds, num_workers, schedul
     data_in.delete()
 
 
+@pytest.mark.slow
 @all_schedulers
 def test_chain_transform_list_small(local_ds, scheduler):
     ls = list(range(100))
@@ -348,6 +353,7 @@ def test_chain_transform_list_small(local_ds, scheduler):
             )
 
 
+@pytest.mark.slow
 @all_schedulers
 def test_chain_transform_list_big(local_ds, scheduler):
     ls = [i for i in range(2)]
@@ -373,6 +379,7 @@ def test_chain_transform_list_big(local_ds, scheduler):
             )
 
 
+@pytest.mark.slow
 @all_schedulers
 @commit_or_not
 def test_add_to_non_empty_dataset(local_ds, scheduler, do_commit):
@@ -433,6 +440,7 @@ def test_add_to_non_empty_dataset(local_ds, scheduler, do_commit):
     compare_dataset_diff([expected_dataset_diff], dataset_diff)
 
 
+@pytest.mark.slow
 @all_schedulers
 @all_compressions
 def test_transform_deeplake_read(local_ds, cat_path, sample_compression, scheduler):
@@ -453,6 +461,7 @@ def test_transform_deeplake_read(local_ds, cat_path, sample_compression, schedul
         np.testing.assert_array_equal(ds_out.image[i].numpy(), ds_out.image[0].numpy())
 
 
+@pytest.mark.slow
 @all_schedulers
 @all_compressions
 def test_transform_deeplake_read_pipeline(
@@ -633,6 +642,7 @@ def test_ds_append_in_transform(memory_ds):
     data_in.delete()
 
 
+@pytest.mark.slow
 def test_transform_pass_through():
     data_in = deeplake.dataset("mem://ds1")
     data_in.create_tensor("image", htype="image", sample_compression="png")
@@ -811,6 +821,7 @@ def test_inplace_transform_non_head(local_ds_generator):
         check_target_array(ds, i, target)
 
 
+@pytest.mark.slow
 def test_inplace_transform_bug(local_ds_generator):
     @deeplake.compute
     def construct(sample_in, sample_out):
@@ -899,6 +910,7 @@ def test_inplace_transform_clear_chunks(local_ds_generator):
             pass
 
 
+@pytest.mark.slow
 def test_transform_skip_ok(local_ds_generator):
     ds = local_ds_generator()
     ls = list(range(100))
@@ -1076,6 +1088,8 @@ def test_transform_bug_link(local_ds, cat_path):
             assert ds[i].abc.shape == (900, 900, 3)
 
 
+@pytest.mark.slow
+@pytest.mark.flaky(retry_count=3)
 def test_tensor_dataset_memory_leak(local_ds):
     local_ds.create_tensor("image", htype="image", sample_compression="png")
     add_images().eval(list(range(100)), local_ds, scheduler="threaded")
@@ -1095,6 +1109,7 @@ def test_transform_info(local_ds_generator):
     assert ds.info["test"] == 123
 
 
+@pytest.mark.slow
 @parametrize_num_workers
 @all_compressions
 @pytest.mark.parametrize(
@@ -1127,7 +1142,7 @@ def test_read_only_dataset_aggregation_image(ds, sample_compression, num_workers
 @parametrize_num_workers
 @pytest.mark.parametrize(
     "ds",
-    ["memory_ds", "local_ds", "s3_ds"],
+    ["memory_ds", "local_ds", pytest.param("s3_ds", marks=pytest.mark.slow)],
     indirect=True,
 )
 def test_read_only_dataset_aggregation_label(ds, num_workers):
@@ -1157,7 +1172,7 @@ def test_read_only_dataset_aggregation_label(ds, num_workers):
 @all_schedulers
 @pytest.mark.parametrize(
     "ds",
-    ["local_ds", "s3_ds"],
+    ["local_ds", pytest.param("s3_ds", marks=pytest.mark.slow)],
     indirect=True,
 )
 def test_read_only_dataset_raise(ds, scheduler, num_workers):
@@ -1228,6 +1243,7 @@ def test_classlabel_transform_bug(local_ds):
         np.testing.assert_array_equal(ds.x[0], -1)
 
 
+@pytest.mark.slow
 def test_downsample_transform(local_ds):
     with local_ds as ds:
         ds.create_tensor(
@@ -1284,6 +1300,7 @@ def test_none_rechunk_post_transform(local_ds):
     assert num_chunks == 2
 
 
+@pytest.mark.slow
 @pytest.mark.parametrize("scheduler", ["serial", "threaded", "processed"])
 def test_transform_checkpointing(local_ds, scheduler):
     @deeplake.compute
@@ -1375,6 +1392,7 @@ class BadSample:
     shape = (250, 250, 3)
 
 
+@pytest.mark.slow
 @all_schedulers
 @pytest.mark.parametrize("method", ["ds", "multiple", "checkpointed"])
 @pytest.mark.parametrize("error_at", ["transform", "chunk_engine"])
@@ -1428,7 +1446,10 @@ def test_ds_append_errors(
         err_msg += re.escape(
             " Last checkpoint: 10 samples processed. You can slice the input to resume from this point."
         )
-    err_msg += re.escape(" See traceback for more details.")
+    err_msg += re.escape(
+        " See traceback for more details."
+        " If you wish to skip the samples that cause errors, please specify `ignore_errors=True`."
+    )
 
     samples.insert(17, bad_sample)
 
@@ -1553,6 +1574,7 @@ def mul_by_2(sample_in, samples_out):
     samples_out.images.append(sample_in.images.numpy() - 1)
 
 
+@pytest.mark.slow
 def test_pipeline(local_ds, flower_path):
     pipeline = deeplake.compose([add_samples(flower_path), mul_by_2()])
 
@@ -1735,3 +1757,43 @@ def test_transform_summary(local_ds, capsys):
 
     captured = capsys.readouterr()
     assert captured.out == ""
+
+
+def test_transform_extend(local_ds):
+    # skip_ok is not supported with extend
+    @deeplake.compute
+    def bad_upload(batch, ds):
+        ds.extend(batch, skip_ok=True)
+
+    @deeplake.compute
+    def upload(batch, ds):
+        ds.extend(batch, append_empty=True)
+
+    with local_ds as ds:
+        ds.create_tensor("abc")
+        ds.create_tensor("xyz")
+        ds.abc.append(np.ones((10, 10)))
+        ds.xyz.append(1)
+
+    batches = [{"abc": np.ones((5, 10, 10))} for _ in range(10)]
+
+    with pytest.raises(TransformError):
+        bad_upload().eval(batches, ds, num_workers=TRANSFORM_TEST_NUM_WORKERS)
+
+    ds.xyz.append(2)
+    # unequal tensor lengths
+    with pytest.raises(TransformError):
+        upload().eval(batches, ds, num_workers=TRANSFORM_TEST_NUM_WORKERS)
+
+    ds.abc.append(np.ones((10, 10)))
+    # items should be dicts
+    with pytest.raises(TransformError):
+        upload().eval([1, 2, 3, 4, 5], ds, num_workers=TRANSFORM_TEST_NUM_WORKERS)
+
+    upload().eval(batches, ds, num_workers=TRANSFORM_TEST_NUM_WORKERS)
+
+    assert len(ds) == 52
+    assert ds.abc.numpy().shape == (52, 10, 10)
+    assert ds.xyz.shape == (52, None)
+    assert ds.xyz[:2].numpy().shape == (2, 1)
+    assert ds.xyz[2:].numpy().shape == (50, 0)
