@@ -12,4 +12,5 @@ from deeplake.core.vectorstore.vector_search.indra.search_algorithm import (
     search as indra_search_algorithm,
 )
 from deeplake.core.vectorstore.deeplake_vectorstore import VectorStore
-from deeplake.core.vectorstore.deeplake_vectorstore import DeepLakeVectorStore
+
+DeepLakeVectorStore = VectorStore
