@@ -1,29 +1,22 @@
 import logging
 import pathlib
-from typing import Optional, Any, Iterable, List, Dict, Union, Callable
+from typing import Optional, Any, List, Dict, Union, Callable
 
 import numpy as np
 
-try:
-    from indra import api  # type: ignore
-
-    _INDRA_INSTALLED = True
-except Exception:  # pragma: no cover
-    _INDRA_INSTALLED = False  # pragma: no cover
-
 import deeplake
+from deeplake.core.dataset import Dataset
+from deeplake.core.vectorstore.dataset_handlers import get_dataset_handler
+from deeplake.core.vectorstore.deep_memory import DeepMemory
 from deeplake.constants import (
     DEFAULT_VECTORSTORE_TENSORS,
+    MAX_BYTES_PER_MINUTE,
+    TARGET_BYTE_SIZE,
+    USE_INDRA,
 )
-from deeplake.core.vectorstore import utils
-from deeplake.core.vectorstore.vector_search import vector_search
-from deeplake.core.vectorstore.vector_search import dataset as dataset_utils
-from deeplake.core.vectorstore.vector_search import filter as filter_utils
+from deeplake.util.bugout_reporter import feature_report_path
+from deeplake.util.exceptions import DeepMemoryAccessError
 
-from deeplake.util.bugout_reporter import (
-    feature_report_path,
-    deeplake_reporter,
-)
 
 logger = logging.getLogger(__name__)
 
@@ -33,18 +26,23 @@ class VectorStore:
 
     def __init__(
         self,
-        path: Union[str, pathlib.Path],
+        path: Optional[Union[str, pathlib.Path]] = None,
+        dataset: Optional[Dataset] = None,
         tensor_params: List[Dict[str, object]] = DEFAULT_VECTORSTORE_TENSORS,
-        embedding_function: Optional[Callable] = None,
+        embedding_function: Optional[Any] = None,
         read_only: Optional[bool] = None,
         ingestion_batch_size: int = 1000,
-        num_workers: int = 0,
+        index_params: Optional[Dict[str, Union[int, str]]] = None,
         exec_option: str = "auto",
         token: Optional[str] = None,
         overwrite: bool = False,
         verbose: bool = True,
         runtime: Optional[Dict] = None,
         creds: Optional[Union[Dict, str]] = None,
+        org_id: Optional[str] = None,
+        logger: logging.Logger = logger,
+        branch: str = "main",
+        indra: bool = USE_INDRA,
         **kwargs: Any,
     ) -> None:
         """Creates an empty VectorStore or loads an existing one if it exists at the specified ``path``.
@@ -79,10 +77,20 @@ class VectorStore:
                 - a local file system path of the form ``./path/to/dataset`` or ``~/path/to/dataset`` or ``path/to/dataset``.
                 - a memory path of the form ``mem://path/to/dataset`` which doesn't save the dataset but keeps it in memory instead. Should be used only for testing as it does not persist.
             tensor_params (List[Dict[str, dict]], optional): List of dictionaries that contains information about tensors that user wants to create. See ``create_tensor`` in Deep Lake API docs for more information. Defaults to ``DEFAULT_VECTORSTORE_TENSORS``.
-            embedding_function (Optional[callable], optional): Function that converts the embeddable data into embeddings. Defaults to None.
+            embedding_function (Optional[Any], optional): Function or class that converts the embeddable data into embeddings. Input to `embedding_function` is a list of data and output is a list of embeddings. Defaults to None.
             read_only (bool, optional):  Opens dataset in read-only mode if True. Defaults to False.
-            num_workers (int): Number of workers to use for parallel ingestion.
             ingestion_batch_size (int): Batch size to use for parallel ingestion.
+            index_params (Dict[str, Union[int, str]]): Dictionary containing information about vector index that will be created. Defaults to ``None``, which will utilize ``DEFAULT_VECTORSTORE_INDEX_PARAMS`` from ``deeplake.constants``. The specified key-values override the default ones:
+
+                - 'threshold': The threshold for the dataset size above which an index will be created for the embedding tensor. When the threshold value is set to -1, index creation is turned off.
+                  Defaults to -1, which turns off the index.
+                - 'distance_metric': This key specifies the method of calculating the distance between vectors when creating the vector database (VDB) index. It can either be a string that corresponds to a member of the DistanceType enumeration, or the string value itself.
+
+                    - If no value is provided, it defaults to "L2".
+                    - "L2" corresponds to DistanceType.L2_NORM.
+                    - "COS" corresponds to DistanceType.COSINE_SIMILARITY.
+
+                - 'additional_params': Additional parameters for fine-tuning the index.
             exec_option (str): Default method for search execution. It could be either ``"auto"``, ``"python"``, ``"compute_engine"`` or ``"tensor_db"``. Defaults to ``"auto"``. If None, it's set to "auto".
                 - ``auto``- Selects the best execution method based on the storage location of the Vector Store. It is the default option.
                 - ``python`` - Pure-python implementation that runs on the client and can be used for data stored anywhere. WARNING: using this option with big datasets is discouraged because it can lead to memory issues.
@@ -96,8 +104,9 @@ class VectorStore:
                 - It supports 'aws_access_key_id', 'aws_secret_access_key', 'aws_session_token', 'endpoint_url', 'aws_region', 'profile_name' as keys.
                 - If 'ENV' is passed, credentials are fetched from the environment variables. This is also the case when creds is not passed for cloud datasets. For datasets connected to hub cloud, specifying 'ENV' will override the credentials fetched from Activeloop and use local ones.
             runtime (Dict, optional): Parameters for creating the Vector Store in Deep Lake's Managed Tensor Database. Not applicable when loading an existing Vector Store. To create a Vector Store in the Managed Tensor Database, set `runtime = {"tensor_db": True}`.
-
-            **kwargs (Any): Additional keyword arguments.
+            branch (str): Branch name to use for the Vector Store. Defaults to "main".
+            indra (bool): Flag indicating whether indra api should be used to create the underlying dataset. Defaults to false
+            **kwargs (dict): Additional keyword arguments.
 
         ..
             # noqa: DAR101
@@ -105,61 +114,50 @@ class VectorStore:
         Danger:
             Setting ``overwrite`` to ``True`` will delete all of your data if the Vector Store exists! Be very careful when setting this parameter.
         """
-        feature_report_path(
-            path,
-            "vs.initialize",
-            {
-                "tensor_params": "default"
-                if tensor_params is not None
-                else tensor_params,
-                "embedding_function": True if embedding_function is not None else False,
-                "num_workers": num_workers,
-                "overwrite": overwrite,
-                "read_only": read_only,
-                "ingestion_batch_size": ingestion_batch_size,
-                "exec_option": exec_option,
-                "token": token,
-                "verbose": verbose,
-                "runtime": runtime,
-            },
+
+        kwargs.pop("num_workers", None)
+
+        self.dataset_handler = get_dataset_handler(
+            path=path,
+            dataset=dataset,
+            tensor_params=tensor_params,
+            embedding_function=embedding_function,
+            read_only=read_only,
+            ingestion_batch_size=ingestion_batch_size,
+            index_params=index_params,
+            num_workers=0,
+            exec_option=exec_option,
             token=token,
-        )
-
-        self.ingestion_batch_size = ingestion_batch_size
-        self.num_workers = num_workers
-
-        if creds is None:
-            creds = {}
-
-        self.dataset = dataset_utils.create_or_load_dataset(
-            tensor_params,
-            path,
-            token,
-            creds,
-            logger,
-            read_only,
-            exec_option,
-            embedding_function,
-            overwrite,
-            runtime,
+            overwrite=overwrite,
+            verbose=verbose,
+            runtime=runtime,
+            creds=creds,
+            org_id=org_id,
+            logger=logger,
+            branch=branch,
             **kwargs,
         )
-        self.embedding_function = embedding_function
-        self.exec_option = utils.parse_exec_option(
-            self.dataset, exec_option, _INDRA_INSTALLED
+
+        self.deep_memory = DeepMemory(
+            dataset=self.dataset_handler.dataset,
+            path=self.dataset_handler.path,
+            token=self.dataset_handler.token,
+            logger=logger,
+            embedding_function=embedding_function,
+            creds=self.dataset_handler.creds,
         )
-        self.verbose = verbose
-        self.tensor_params = tensor_params
 
     def add(
         self,
         embedding_function: Optional[Union[Callable, List[Callable]]] = None,
         embedding_data: Optional[Union[List, List[List]]] = None,
         embedding_tensor: Optional[Union[str, List[str]]] = None,
-        total_samples_processed: int = 0,
         return_ids: bool = False,
-        num_workers: Optional[int] = None,
-        ingestion_batch_size: Optional[int] = None,
+        rate_limiter: Dict = {
+            "enabled": False,
+            "bytes_per_minute": MAX_BYTES_PER_MINUTE,
+            "batch_byte_size": TARGET_BYTE_SIZE,
+        },
         **tensors,
     ) -> Optional[List[str]]:
         """Adding elements to deeplake vector store.
@@ -173,14 +171,12 @@ class VectorStore:
             >>> metadatas = [{"timestamp": "01:20"}, {"timestamp": "01:22"}]
             >>> emebdding_fn = lambda x: [[1, 2, 3]] * len(x)
             >>> embedding_fn_2 = lambda x: [[4, 5]] * len(x)
-
             >>> # Directly upload embeddings
             >>> deeplake_vector_store.add(
             ...     text = texts,
             ...     embedding = embeddings,
             ...     metadata = metadatas,
             ... )
-
             >>> # Upload embedding via embedding function
             >>> deeplake_vector_store.add(
             ...     text = texts,
@@ -188,7 +184,6 @@ class VectorStore:
             ...     embedding_function = embedding_fn,
             ...     embedding_data = texts,
             ... )
-
             >>> # Upload embedding via embedding function to a user-defined embedding tensor
             >>> deeplake_vector_store.add(
             ...     text = texts,
@@ -197,14 +192,12 @@ class VectorStore:
             ...     embedding_data = texts,
             ...     embedding_tensor = "embedding_1",
             ... )
-
             >>> # Multiple embedding functions (user defined embedding tensors must be specified)
             >>> deeplake_vector_store.add(
             ...     embedding_tensor = ["embedding_1", "embedding_2"]
             ...     embedding_function = [embedding_fn, embedding_fn_2],
             ...     embedding_data = [texts, texts],
             ... )
-
             >>> # Alternative syntax for multiple embedding functions
             >>> deeplake_vector_store.add(
             ...     text = texts,
@@ -212,7 +205,6 @@ class VectorStore:
             ...     embedding_tensor_1 = (embedding_fn, texts),
             ...     embedding_tensor_2 = (embedding_fn_2, texts),
             ... )
-
             >>> # Add data to fully custom tensors
             >>> deeplake_vector_store.add(
             ...     tensor_A = [1, 2],
@@ -224,95 +216,41 @@ class VectorStore:
             ... )
 
         Args:
-            embedding_function (Optional[Callable]): embedding function used to convert ``embedding_data`` into embeddings. Overrides the ``embedding_function`` specified when initializing the Vector Store.
+            embedding_function (Optional[Callable]): embedding function used to convert ``embedding_data`` into embeddings. Input to `embedding_function` is a list of data and output is a list of embeddings. Overrides the ``embedding_function`` specified when initializing the Vector Store.
             embedding_data (Optional[List]): Data to be converted into embeddings using the provided ``embedding_function``. Defaults to None.
             embedding_tensor (Optional[str]): Tensor where results from the embedding function will be stored. If None, the embedding tensor is automatically inferred (when possible). Defaults to None.
-            total_samples_processed (int): Total number of samples processed before ingestion stopped. When specified.
             return_ids (bool): Whether to return added ids as an ouput of the method. Defaults to False.
-            num_workers (int): Number of workers to use for parallel ingestion. Overrides the ``num_workers`` specified when initializing the Vector Store.
-            ingestion_batch_size (int): Batch size to use for parallel ingestion. Defaults to 1000. Overrides the ``ingestion_batch_size`` specified when initializing the Vector Store.
+            rate_limiter (Dict): Rate limiter configuration. Defaults to ``{"enabled": False, "bytes_per_minute": MAX_BYTES_PER_MINUTE, "batch_byte_size": TARGET_BYTE_SIZE}``.
             **tensors: Keyword arguments where the key is the tensor name, and the value is a list of samples that should be uploaded to that tensor.
 
         Returns:
             Optional[List[str]]: List of ids if ``return_ids`` is set to True. Otherwise, None.
         """
-
-        deeplake_reporter.feature_report(
-            feature_name="vs.add",
-            parameters={
-                "tensors": list(tensors.keys()) if tensors else None,
-                "embedding_tensor": embedding_tensor,
-                "total_samples_processed": total_samples_processed,
-                "return_ids": return_ids,
-                "embedding_function": True if embedding_function is not None else False,
-                "embedding_data": True if embedding_data is not None else False,
-            },
-        )
-
-        (
-            embedding_function,
-            embedding_data,
-            embedding_tensor,
-            tensors,
-        ) = utils.parse_tensors_kwargs(
-            tensors, embedding_function, embedding_data, embedding_tensor
-        )
-
-        (
-            embedding_function,
-            embedding_data,
-            embedding_tensor,
-            tensors,
-        ) = utils.parse_add_arguments(
-            dataset=self.dataset,
-            initial_embedding_function=self.embedding_function,
+        return self.dataset_handler.add(
             embedding_function=embedding_function,
             embedding_data=embedding_data,
             embedding_tensor=embedding_tensor,
+            return_ids=return_ids,
+            rate_limiter=rate_limiter,
             **tensors,
         )
 
-        processed_tensors, id_ = dataset_utils.preprocess_tensors(
-            embedding_data, embedding_tensor, self.dataset, **tensors
-        )
-
-        assert id_ is not None
-        utils.check_length_of_each_tensor(processed_tensors)
-
-        dataset_utils.extend_or_ingest_dataset(
-            processed_tensors=processed_tensors,
-            dataset=self.dataset,
-            embedding_function=embedding_function,
-            embedding_data=embedding_data,
-            embedding_tensor=embedding_tensor,
-            ingestion_batch_size=ingestion_batch_size or self.ingestion_batch_size,
-            num_workers=num_workers or self.num_workers,
-            total_samples_processed=total_samples_processed,
-            logger=logger,
-        )
-
-        self.dataset.commit(allow_empty=True)
-        if self.verbose:
-            self.dataset.summary()
-
-        if return_ids:
-            return id_
-        return None
-
     def search(
         self,
-        embedding_data=None,
+        embedding_data: Union[str, List[str], None] = None,
         embedding_function: Optional[Callable] = None,
         embedding: Optional[Union[List[float], np.ndarray]] = None,
         k: int = 4,
-        distance_metric: str = "COS",
+        distance_metric: Optional[str] = None,
         query: Optional[str] = None,
         filter: Optional[Union[Dict, Callable]] = None,
         exec_option: Optional[str] = None,
         embedding_tensor: str = "embedding",
         return_tensors: Optional[List[str]] = None,
         return_view: bool = False,
-    ) -> Union[Dict, deeplake.core.dataset.Dataset]:
+        deep_memory: bool = False,
+        return_tql: bool = False,
+    ) -> Union[Dict, Dataset]:
         """VectorStore search method that combines embedding search, metadata search, and custom TQL search.
 
         Examples:
@@ -321,21 +259,18 @@ class VectorStore:
             ...        embedding = [1, 2, 3],
             ...        exec_option = "python",
             ... )
-
             >>> # Search using an embedding function and data for embedding
             >>> data = vector_store.search(
             ...        embedding_data = "What does this chatbot do?",
             ...        embedding_function = query_embedding_fn,
             ...        exec_option = "compute_engine",
             ... )
-
             >>> # Add a filter to your search
             >>> data = vector_store.search(
             ...        embedding = np.ones(3),
             ...        exec_option = "python",
             ...        filter = {"json_tensor_name": {"key: value"}, "json_tensor_name_2": {"key_2: value_2"},...}, # Only valid for exec_option = "python"
             ... )
-
             >>> # Search using TQL
             >>> data = vector_store.search(
             ...        query = "select * where ..... <add TQL syntax>",
@@ -344,10 +279,10 @@ class VectorStore:
 
         Args:
             embedding (Union[np.ndarray, List[float]], optional): Embedding representation for performing the search. Defaults to None. The ``embedding_data`` and ``embedding`` cannot both be specified.
-            embedding_data: Data against which the search will be performed by embedding it using the `embedding_function`. Defaults to None. The `embedding_data` and `embedding` cannot both be specified.
-            embedding_function (Optional[Callable], optional): function for converting `embedding_data` into embedding. Only valid if `embedding_data` is specified
+            embedding_data (List[str]): Data against which the search will be performed by embedding it using the `embedding_function`. Defaults to None. The `embedding_data` and `embedding` cannot both be specified.
+            embedding_function (Optional[Callable], optional): function for converting `embedding_data` into embedding. Only valid if `embedding_data` is specified. Input to `embedding_function` is a list of data and output is a list of embeddings.
             k (int): Number of elements to return after running query. Defaults to 4.
-            distance_metric (str): Type of distance metric to use for sorting the data. Avaliable options are: ``"L1", "L2", "COS", "MAX"``. Defaults to ``"COS"``.
+            distance_metric (str): Distance metric to use for sorting the data. Avaliable options are: ``"L1", "L2", "COS", "MAX"``. Defaults to None, which uses same distance metric specified in ``index_params``. If there is no index, it performs linear search using ``DEFAULT_VECTORSTORE_DISTANCE_METRIC``.
             query (Optional[str]):  TQL Query string for direct evaluation, without application of additional filters or vector search.
             filter (Union[Dict, Callable], optional): Additional filter evaluated prior to the embedding search.
 
@@ -359,50 +294,31 @@ class VectorStore:
                 - ``python`` - Pure-python implementation that runs on the client and can be used for data stored anywhere. WARNING: using this option with big datasets is discouraged because it can lead to memory issues.
                 - ``compute_engine`` - Performant C++ implementation of the Deep Lake Compute Engine that runs on the client and can be used for any data stored in or connected to Deep Lake. It cannot be used with in-memory or local datasets.
                 - ``tensor_db`` - Performant and fully-hosted Managed Tensor Database that is responsible for storage and query execution. Only available for data stored in the Deep Lake Managed Database. Store datasets in this database by specifying runtime = {"tensor_db": True} during dataset creation.
+
             embedding_tensor (str): Name of tensor with embeddings. Defaults to "embedding".
             return_tensors (Optional[List[str]]): List of tensors to return data for. Defaults to None, which returns data for all tensors except the embedding tensor (in order to minimize payload). To return data for all tensors, specify return_tensors = "*".
             return_view (bool): Return a Deep Lake dataset view that satisfied the search parameters, instead of a dictionary with data. Defaults to False. If ``True`` return_tensors is set to "*" beucase data is lazy-loaded and there is no cost to including all tensors in the view.
+            deep_memory (bool): Whether to use the Deep Memory model for improving search results. Defaults to False if deep_memory is not specified in the Vector Store initialization.
+                If True, the distance metric is set to "deepmemory_distance", which represents the metric with which the model was trained. The search is performed using the Deep Memory model. If False, the distance metric is set to "COS" or whatever distance metric user specifies.
+            return_tql (bool): Whether to return the TQL query string used for the search. Defaults to False.
 
         ..
             # noqa: DAR101
 
         Raises:
             ValueError: When invalid parameters are specified.
+            ValueError: when deep_memory is True. Deep Memory is only available for datasets stored in the Deep Lake Managed Database for paid accounts.
+            DeepMemoryAccessError: if user does not have access to deep_memory.
 
         Returns:
             Dict: Dictionary where keys are tensor names and values are the results of the search
         """
+        if deep_memory and not self.deep_memory:
+            raise DeepMemoryAccessError()
 
-        deeplake_reporter.feature_report(
-            feature_name="vs.search",
-            parameters={
-                "embedding_data": True if embedding_data is not None else False,
-                "embedding_function": True if embedding_function is not None else False,
-                "k": k,
-                "distance_metric": distance_metric,
-                "query": query[0:100] if query is not None else False,
-                "filter": True if filter is not None else False,
-                "exec_option": exec_option,
-                "embedding_tensor": embedding_tensor,
-                "embedding": True if embedding is not None else False,
-                "return_tensors": return_tensors,
-                "return_view": return_view,
-            },
-        )
-
-        if exec_option is None and self.exec_option != "python" and callable(filter):
-            logger.warning(
-                'Switching exec_option to "python" (runs on client) because filter is specified as a function. '
-                f'To continue using the original exec_option "{self.exec_option}", please specify the filter as a dictionary or use the "query" parameter to specify a TQL query.'
-            )
-            exec_option = "python"
-
-        exec_option = exec_option or self.exec_option
-
-        utils.parse_search_args(
+        return self.dataset_handler.search(
             embedding_data=embedding_data,
             embedding_function=embedding_function,
-            initial_embedding_function=self.embedding_function,
             embedding=embedding,
             k=k,
             distance_metric=distance_metric,
@@ -411,41 +327,14 @@ class VectorStore:
             exec_option=exec_option,
             embedding_tensor=embedding_tensor,
             return_tensors=return_tensors,
-        )
-
-        return_tensors = utils.parse_return_tensors(
-            self.dataset, return_tensors, embedding_tensor, return_view
-        )
-
-        query_emb: Optional[Union[List[float], np.ndarray[Any, Any]]] = None
-        if query is None:
-            query_emb = dataset_utils.get_embedding(
-                embedding,
-                embedding_data,
-                embedding_function=embedding_function or self.embedding_function,
-            )
-            if isinstance(query_emb, np.ndarray):
-                assert (
-                    query_emb.ndim == 1 or query_emb.shape[0] == 1
-                ), "Query embedding must be 1-dimensional. Please consider using another embedding function for converting query string to embedding."
-
-        return vector_search.search(
-            query=query,
-            logger=logger,
-            filter=filter,
-            query_embedding=query_emb,
-            k=k,
-            distance_metric=distance_metric,
-            exec_option=exec_option,
-            deeplake_dataset=self.dataset,
-            embedding_tensor=embedding_tensor,
-            return_tensors=return_tensors,
             return_view=return_view,
+            return_tql=return_tql,
+            deep_memory=deep_memory,
         )
 
     def delete(
         self,
-        row_ids: Optional[List[str]] = None,
+        row_ids: Optional[List[int]] = None,
         ids: Optional[List[str]] = None,
         filter: Optional[Union[Dict, Callable]] = None,
         query: Optional[str] = None,
@@ -457,12 +346,10 @@ class VectorStore:
         Examples:
             >>> # Delete using ids:
             >>> data = vector_store.delete(ids)
-
             >>> # Delete data using filter
             >>> data = vector_store.delete(
             ...        filter = {"json_tensor_name": {"key: value"}, "json_tensor_name_2": {"key_2: value_2"}},
             ... )
-
             >>> # Delete data using TQL
             >>> data = vector_store.delete(
             ...        query = "select * where ..... <add TQL syntax>",
@@ -471,7 +358,7 @@ class VectorStore:
 
         Args:
             ids (Optional[List[str]]): List of unique ids. Defaults to None.
-            row_ids (Optional[List[str]]): List of absolute row indices from the dataset. Defaults to None.
+            row_ids (Optional[List[int]]): List of absolute row indices from the dataset. Defaults to None.
             filter (Union[Dict, Callable], optional): Filter for finding samples for deletion.
                 - ``Dict`` - Key-value search on tensors of htype json, evaluated on an AND basis (a sample must satisfy all key-value filters to be True) Dict = {"tensor_name_1": {"key": value}, "tensor_name_2": {"key": value}}
                 - ``Function`` - Any function that is compatible with `deeplake.filter`.
@@ -492,42 +379,15 @@ class VectorStore:
             ValueError: If neither ``ids``, ``filter``, ``query``, nor ``delete_all`` are specified, or if an invalid ``exec_option`` is provided.
         """
 
-        deeplake_reporter.feature_report(
-            feature_name="vs.delete",
-            parameters={
-                "ids": True if ids is not None else False,
-                "row_ids": True if row_ids is not None else False,
-                "query": query[0:100] if query is not None else False,
-                "filter": True if filter is not None else False,
-                "exec_option": exec_option,
-                "delete_all": delete_all,
-            },
+        return self.dataset_handler.delete(
+            row_ids=row_ids,
+            ids=ids,
+            filter=filter,
+            query=query,
+            exec_option=exec_option,
+            delete_all=delete_all,
         )
 
-        if not row_ids:
-            row_ids = dataset_utils.search_row_ids(
-                dataset=self.dataset,
-                search_fn=self.search,
-                ids=ids,
-                filter=filter,
-                query=query,
-                select_all=delete_all,
-                exec_option=exec_option or self.exec_option,
-            )
-
-        (
-            self.dataset,
-            dataset_deleted,
-        ) = dataset_utils.delete_all_samples_if_specified(
-            self.dataset,
-            delete_all,
-        )
-        if dataset_deleted:
-            return True
-
-        dataset_utils.delete_and_commit(self.dataset, row_ids)
-        return True
-
     def update_embedding(
         self,
         row_ids: Optional[List[str]] = None,
@@ -547,9 +407,8 @@ class VectorStore:
             ...    ids,
             ...    embedding_source_tensor = "text",
             ...    embedding_tensor = "embedding",
-            ...    embedding_unction = embedding_function,
+            ...    embedding_function = embedding_function,
             ... )
-
             >>> # Update data using filter and several embedding_tensors, several embedding_source_tensors
             >>> # and several embedding_functions:
             >>> data = vector_store.update(
@@ -558,7 +417,6 @@ class VectorStore:
             ...     filter = {"json_tensor_name": {"key: value"}, "json_tensor_name_2": {"key_2: value_2"}},
             ...     embedding_tensor = ["text_embedding", "metadata_embedding"]
             ... )
-
             >>> # Update data using TQL, if new embedding function is not specified the embedding_function used
             >>> # during initialization will be used
             >>> data = vector_store.update(
@@ -586,84 +444,89 @@ class VectorStore:
             embedding_source_tensor (Union[str, List[str]], optional): Name of tensor with data that needs to be converted to embeddings. Defaults to `text`.
             embedding_tensor (Optional[Union[str, List[str]]], optional): Name of the tensor with embeddings. Defaults to None.
         """
-        deeplake_reporter.feature_report(
-            feature_name="vs.delete",
-            parameters={
-                "ids": True if ids is not None else False,
-                "row_ids": True if row_ids is not None else False,
-                "query": query[0:100] if query is not None else False,
-                "filter": True if filter is not None else False,
-                "exec_option": exec_option,
-            },
-        )
-
-        (
-            embedding_function,
-            embedding_source_tensor,
-            embedding_tensor,
-        ) = utils.parse_update_arguments(
-            dataset=self.dataset,
-            embedding_function=embedding_function,
-            initial_embedding_function=self.embedding_function,
-            embedding_source_tensor=embedding_source_tensor,
-            embedding_tensor=embedding_tensor,
-        )
-
-        if not row_ids:
-            row_ids = dataset_utils.search_row_ids(
-                dataset=self.dataset,
-                search_fn=self.search,
-                ids=ids,
-                filter=filter,
-                query=query,
-                exec_option=exec_option or self.exec_option,
-            )
-
-        embedding_tensor_data = utils.convert_embedding_source_tensor_to_embeddings(
-            dataset=self.dataset,
-            embedding_source_tensor=embedding_source_tensor,
-            embedding_tensor=embedding_tensor,
-            embedding_function=embedding_function,
+        self.dataset_handler.update_embedding(
             row_ids=row_ids,
+            ids=ids,
+            filter=filter,
+            query=query,
+            exec_option=exec_option,
+            embedding_function=embedding_function,
+            embedding_source_tensor=embedding_source_tensor,
+            embedding_tensor=embedding_tensor,
         )
 
-        self.dataset[row_ids].update(embedding_tensor_data)
-        self.dataset.commit(allow_empty=True)
-
     @staticmethod
     def delete_by_path(
         path: Union[str, pathlib.Path],
         token: Optional[str] = None,
+        force: bool = False,
+        creds: Optional[Union[Dict, str]] = None,
     ) -> None:
         """Deleted the Vector Store at the specified path.
 
         Args:
             path (str, pathlib.Path): The full path to the Deep Lake Vector Store.
             token (str, optional): Activeloop token, used for fetching user credentials. This is optional, as tokens are normally autogenerated. Defaults to ``None``.
+            creds (dict, str, optional): The string ``ENV`` or a dictionary containing credentials used to access the dataset at the path.
+                - If 'aws_access_key_id', 'aws_secret_access_key', 'aws_session_token' are present, these take precedence over credentials present in the environment or in credentials file. Currently only works with s3 paths.
+                - It supports 'aws_access_key_id', 'aws_secret_access_key', 'aws_session_token', 'endpoint_url', 'aws_region', 'profile_name' as keys.
+                - If 'ENV' is passed, credentials are fetched from the environment variables. This is also the case when creds is not passed for cloud datasets. For datasets connected to hub cloud, specifying 'ENV' will override the credentials fetched from Activeloop and use local ones.
+            force (bool): delete the path in a forced manner without rising an exception. Defaults to ``True``.
 
         Danger:
             This method permanently deletes all of your data if the Vector Store exists! Be very careful when using this method.
         """
-
         feature_report_path(
             path,
             "vs.delete_by_path",
-            {},
+            parameters={
+                "path": path,
+                "token": token,
+                "force": force,
+                "creds": creds,
+            },
             token=token,
         )
-        deeplake.delete(path, large_ok=True, token=token)
+        deeplake.delete(path, large_ok=True, token=token, force=force, creds=creds)
+
+    def commit(self, allow_empty: bool = True) -> None:
+        """Commits the Vector Store.
+
+        Args:
+            allow_empty (bool): Whether to allow empty commits. Defaults to True.
+        """
+        self.dataset_handler.commit(allow_empty=allow_empty)
+
+    def checkout(self, branch: str = "main", create=False) -> None:
+        """Checkout the Vector Store to a specific branch.
+
+        Args:
+            branch (str): Branch name to checkout. Defaults to "main".
+            create (bool): Whether to create the branch if it doesn't exist. Defaults to False.
+        """
+        self.dataset_handler.checkout(branch, create=create)
 
     def tensors(self):
         """Returns the list of tensors present in the dataset"""
-        return self.dataset.tensors
+        return self.dataset_handler.tensors()
 
     def summary(self):
         """Prints a summary of the dataset"""
-        return self.dataset.summary()
+        return self.dataset_handler.summary()
+
+    @property
+    def dataset(self):
+        """Returns the dataset"""
+        try:
+            return self.dataset_handler.dataset
+        except AttributeError:
+            raise AttributeError(
+                "Acessing the dataset is not available for managed Vector Store."
+            )
 
     def __len__(self):
         """Length of the dataset"""
-        return len(self.dataset)
+        return len(self.dataset_handler)
 
 
 DeepLakeVectorStore = VectorStore
