@@ -1,6 +1,7 @@
 from deeplake.core.vectorstore.vector_search.dataset.dataset import (
     create_or_load_dataset,
     delete_and_commit,
+    delete_and_without_commit,
     delete_all_samples_if_specified,
     fetch_embeddings,
     get_embedding,
@@ -9,4 +10,6 @@ from deeplake.core.vectorstore.vector_search.dataset.dataset import (
     extend_or_ingest_dataset,
     convert_id_to_row_id,
     search_row_ids,
+    extend,
+    populate_rate_limiter,
 )
