@@ -1,18 +1,13 @@
 import uuid
+import sys
+import time
+from math import ceil
 from typing import List, Dict, Any, Optional, Callable, Union
+from tqdm import tqdm
 
 import numpy as np
-from math import ceil
-
-try:
-    from indra import api  # type: ignore
-
-    _INDRA_INSTALLED = True  # pragma: no cover
-except ImportError:  # pragma: no cover
-    _INDRA_INSTALLED = False  # pragma: no cover
 
 import deeplake
-from deeplake.constants import MB
 from deeplake.core.vectorstore.vector_search import utils
 from deeplake.core.vectorstore.vector_search.ingestion import ingest_data
 from deeplake.constants import (
@@ -20,6 +15,10 @@ from deeplake.constants import (
     VECTORSTORE_EXTEND_MAX_SIZE,
     DEFAULT_VECTORSTORE_TENSORS,
     VECTORSTORE_EXTEND_MAX_SIZE_BY_HTYPE,
+    MAX_BYTES_PER_MINUTE,
+    TARGET_BYTE_SIZE,
+    VECTORSTORE_EXTEND_BATCH_SIZE,
+    DEFAULT_RATE_LIMITER_KEY_TO_VALUE,
 )
 from deeplake.util.exceptions import IncorrectEmbeddingShapeError
 
@@ -35,11 +34,11 @@ def create_or_load_dataset(
     embedding_function,
     overwrite,
     runtime,
+    org_id,
+    branch="main",
     **kwargs,
 ):
-    utils.check_indra_installation(
-        exec_option=exec_option, indra_installed=_INDRA_INSTALLED
-    )
+    utils.check_indra_installation(exec_option=exec_option)
 
     if not overwrite and dataset_exists(dataset_path, token, creds, **kwargs):
         if tensor_params is not None and tensor_params != DEFAULT_VECTORSTORE_TENSORS:
@@ -53,7 +52,8 @@ def create_or_load_dataset(
             creds,
             logger,
             read_only,
-            runtime,
+            org_id,
+            branch,
             **kwargs,
         )
 
@@ -67,6 +67,8 @@ def create_or_load_dataset(
         overwrite,
         creds,
         runtime,
+        org_id,
+        branch,
         **kwargs,
     )
 
@@ -84,7 +86,8 @@ def load_dataset(
     creds,
     logger,
     read_only,
-    runtime,
+    org_id,
+    branch,
     **kwargs,
 ):
     if dataset_path == DEFAULT_VECTORSTORE_DEEPLAKE_PATH:
@@ -99,24 +102,16 @@ def load_dataset(
         read_only=read_only,
         creds=creds,
         verbose=False,
+        org_id=org_id,
         **kwargs,
     )
+    dataset.checkout(branch)
     check_tensors(dataset)
 
     logger.warning(
         f"Deep Lake Dataset in {dataset_path} already exists, "
         f"loading from the storage"
     )
-
-    if runtime is not None and runtime["tensor_db"] == True:
-        logger.warning(
-            "Specifying runtime option when loading a Vector Store is not supported and this parameter will "
-            "be ignored. If you wanted to create a new Vector Store, please specify a path to a Vector Store "
-            "that does not already exist. To transfer an existing Vector Store to the Managed Tensor Database, "
-            "use the steps in the link below: "
-            "(https://docs.activeloop.ai/enterprise-features/managed-database/migrating-datasets-to-the-tensor-database)."
-        )
-
     return dataset
 
 
@@ -167,6 +162,8 @@ def create_dataset(
     overwrite,
     creds,
     runtime,
+    org_id,
+    branch,
     **kwargs,
 ):
     if exec_option == "tensor_db" and (
@@ -187,8 +184,10 @@ def create_dataset(
         verbose=False,
         overwrite=overwrite,
         creds=creds,
+        org_id=org_id,
         **kwargs,
     )
+    dataset.checkout(branch)
     create_tensors(tensor_params, dataset, logger, embedding_function)
 
     return dataset
@@ -223,8 +222,18 @@ def delete_and_commit(dataset, ids):
     return True
 
 
+def delete_and_without_commit(dataset, ids, index_maintenance):
+    with dataset:
+        for id in sorted(ids)[::-1]:
+            dataset.pop(id, index_maintenance=index_maintenance)
+
+
 def delete_all_samples_if_specified(dataset, delete_all):
     if delete_all:
+        # delete any indexes linked to any tensors.
+        for t in dataset.tensors:
+            dataset[t]._verify_and_delete_vdb_indexes()
+
         dataset = deeplake.like(
             dataset.path,
             dataset,
@@ -246,13 +255,29 @@ def get_embedding(embedding, embedding_data, embedding_function=None):
         and embedding_function is not None
         and embedding_data is not None
     ):
-        embedding = embedding_function(embedding_data)  # type: ignore
+        if isinstance(embedding_data, list):
+            if len(embedding_data) > 1:
+                raise NotImplementedError("Batched quering is not supported yet.")
+            elif len(embedding_data) == 0:
+                raise ValueError("embedding_data must not be empty.")
+            else:
+                embedding_data = embedding_data[0]
+
+        if not isinstance(embedding_data, str):
+            raise ValueError("embedding_data must be a string.")
+
+        embedding = embedding_function.embed_query(embedding_data)  # type: ignore
 
     if embedding is not None and (
         isinstance(embedding, list) or embedding.dtype != "float32"
     ):
         embedding = np.array(embedding, dtype=np.float32)
 
+    if isinstance(embedding, np.ndarray):
+        assert (
+            embedding.ndim == 1 or embedding.shape[0] == 1
+        ), "Query embedding must be 1-dimensional. Please consider using another embedding function for converting query string to embedding."
+
     return embedding
 
 
@@ -379,76 +404,161 @@ def update_embedding_info(logger, dataset, embedding_function):
     set_embedding_info(dataset[embeddings_tensors[0]], embedding_function)
 
 
+def _compute_batched_embeddings(
+    embedding_function,
+    embedding_data,
+    embedding_tensor,
+    start_idx,
+    end_idx,
+    rate_limiter,
+):
+    """
+    Computes embeddings for a given slice of data.
+    """
+    batched_processed_tensors = {}
+
+    for func, data, tensor in zip(embedding_function, embedding_data, embedding_tensor):
+        data_slice = data[start_idx:end_idx]
+        embedded_data = func(data_slice, rate_limiter=rate_limiter)
+
+        try:
+            return_embedded_data = np.vstack(embedded_data).astype(dtype=np.float32)
+        except ValueError:
+            raise IncorrectEmbeddingShapeError()
+
+        if len(return_embedded_data) == 0:
+            raise ValueError("embedding function returned empty list")
+
+        batched_processed_tensors[tensor] = return_embedded_data
+
+    return batched_processed_tensors
+
+
+def _slice_non_embedding_tensors(
+    processed_tensors, embedding_tensor, start_idx, end_idx
+):
+    """
+    Slices tensors that are not embeddings for a given range.
+    """
+    batched_processed_tensors = {}
+
+    for tensor_name, tensor_data in processed_tensors.items():
+        if tensor_name not in embedding_tensor:
+            batched_processed_tensors[tensor_name] = tensor_data[start_idx:end_idx]
+
+    return batched_processed_tensors
+
+
+def extend(
+    embedding_function: List[Callable],
+    embedding_data: List[Any],
+    embedding_tensor: Union[str, List[str]],
+    processed_tensors: Dict[str, Union[List[Any], np.ndarray]],
+    dataset: deeplake.core.dataset.Dataset,
+    rate_limiter: Dict,
+    _extend_batch_size: int = VECTORSTORE_EXTEND_BATCH_SIZE,
+    logger=None,
+):
+    """
+    Function to extend the dataset with new data.
+    """
+    if embedding_data and not isinstance(embedding_data[0], list):
+        embedding_data = [embedding_data]
+
+    if embedding_function:
+        number_of_batches = ceil(len(embedding_data[0]) / _extend_batch_size)
+        progressbar_str = (
+            f"Creating {len(embedding_data[0])} embeddings in "
+            f"{number_of_batches} batches of size {min(_extend_batch_size, len(embedding_data[0]))}:"
+        )
+
+        for idx in tqdm(
+            range(0, len(embedding_data[0]), _extend_batch_size),
+            progressbar_str,
+        ):
+            batch_start, batch_end = idx, idx + _extend_batch_size
+
+            batched_embeddings = _compute_batched_embeddings(
+                embedding_function,
+                embedding_data,
+                embedding_tensor,
+                batch_start,
+                batch_end,
+                rate_limiter,
+            )
+
+            batched_tensors = _slice_non_embedding_tensors(
+                processed_tensors, embedding_tensor, batch_start, batch_end
+            )
+
+            batched_processed_tensors = {**batched_embeddings, **batched_tensors}
+
+            dataset.extend(batched_processed_tensors, progressbar=False)
+    else:
+        logger.info("Uploading data to deeplake dataset.")
+        dataset.extend(processed_tensors, progressbar=True)
+
+
+def populate_rate_limiter(rate_limiter):
+    if rate_limiter is None or rate_limiter == {}:
+        return {
+            "enabled": False,
+            "bytes_per_minute": MAX_BYTES_PER_MINUTE,
+            "batch_byte_size": TARGET_BYTE_SIZE,
+        }
+    else:
+        rate_limiter_keys = ["enabled", "bytes_per_minute", "batch_byte_size"]
+
+        for key in rate_limiter_keys:
+            if key not in rate_limiter:
+                rate_limiter[key] = DEFAULT_RATE_LIMITER_KEY_TO_VALUE[key]
+
+        for item in rate_limiter:
+            if item not in rate_limiter_keys:
+                raise ValueError(
+                    f"Invalid rate_limiter key: {item}. Valid keys are: 'enabled', 'bytes_per_minute', 'batch_byte_size'."
+                )
+        return rate_limiter
+
+
 def extend_or_ingest_dataset(
     processed_tensors,
     dataset,
     embedding_function,
     embedding_tensor,
     embedding_data,
-    ingestion_batch_size,
-    num_workers,
-    total_samples_processed,
+    rate_limiter,
     logger,
 ):
-    first_item = next(iter(processed_tensors))
-
-    htypes = [
-        dataset[item].meta.htype for item in dataset.tensors
-    ]  # Inspect raw htype (not parsed htype like tensor.htype) in order to avoid parsing links and sequences separately.
-    threshold_by_htype = [
-        VECTORSTORE_EXTEND_MAX_SIZE_BY_HTYPE.get(h, int(1e10)) for h in htypes
-    ]
-    extend_threshold = min(threshold_by_htype + [VECTORSTORE_EXTEND_MAX_SIZE])
-
-    if len(processed_tensors[first_item]) <= extend_threshold:
-        if embedding_function:
-            for func, data, tensor in zip(
-                embedding_function, embedding_data, embedding_tensor
-            ):
-                embedded_data = func(data)
-                try:
-                    embedded_data = np.array(embedded_data, dtype=np.float32)
-                except ValueError:
-                    raise IncorrectEmbeddingShapeError()
-
-                if len(embedded_data) == 0:
-                    raise ValueError("embedding function returned empty list")
-
-                processed_tensors[tensor] = embedded_data
-
-        dataset.extend(processed_tensors)
-    else:
-        elements = create_elements(processed_tensors)
-
-        num_workers_auto = ceil(len(elements) / ingestion_batch_size)
-        if num_workers_auto < num_workers:
-            logger.warning(
-                f"Number of workers is {num_workers}, but {len(elements)} rows of data are being added and the ingestion_batch_size is {ingestion_batch_size}. "
-                f"Setting the number of workers to {num_workers_auto} instead, in order reduce overhead from excessive workers that will not accelerate ingestion."
-                f"If you want to parallelizing using more workers, please reduce ``ingestion_batch_size``."
-            )
-            num_workers = min(num_workers_auto, num_workers)
-
-        ingest_data.run_data_ingestion(
-            elements=elements,
-            dataset=dataset,
-            embedding_function=embedding_function,
-            embedding_tensor=embedding_tensor,
-            ingestion_batch_size=ingestion_batch_size,
-            num_workers=num_workers,
-            total_samples_processed=total_samples_processed,
-            logger=logger,
-        )
+    rate_limiter = populate_rate_limiter(rate_limiter)
+    # TODO: Add back the old logic with checkpointing after indexing is fixed
+    extend(
+        embedding_function,
+        embedding_data,
+        embedding_tensor,
+        processed_tensors,
+        dataset,
+        rate_limiter,
+        logger=logger,
+    )
 
 
 def convert_id_to_row_id(ids, dataset, search_fn, query, exec_option, filter):
     if ids is None:
         delete_view = search_fn(
+            embedding_data=None,
+            embedding_function=None,
+            embedding=None,
+            distance_metric=None,
+            embedding_tensor=None,
             filter=filter,
             query=query,
             exec_option=exec_option,
+            return_tensors=False,
             return_view=True,
             k=int(1e9),
+            deep_memory=False,
+            return_tql=False,
         )
 
     else:
