@@ -1,13 +1,6 @@
 from typing import Dict, Union, Callable
+
 from deeplake.core.dataset import Dataset as DeepLakeDataset
-
-try:
-    from indra import api  # type: ignore
-
-    _INDRA_INSTALLED = True  # pragma: no cover
-except ImportError:  # pragma: no cover
-    _INDRA_INSTALLED = False  # pragma: no cover
-
 from deeplake.core.vectorstore.vector_search import utils
 from deeplake.core.vectorstore.vector_search import filter as filter_utils
 from deeplake.core import vectorstore
@@ -25,6 +18,9 @@ def vector_search(
     k,
     return_tensors,
     return_view,
+    token,
+    org_id,
+    return_tql,
 ) -> Union[Dict, DeepLakeDataset]:
     runtime = utils.get_runtime_from_exec_option(exec_option)
 
@@ -33,7 +29,7 @@ def vector_search(
             f"UDF filter functions are not supported with the current `exec_option`={exec_option}. "
         )
 
-    utils.check_indra_installation(exec_option, indra_installed=_INDRA_INSTALLED)
+    utils.check_indra_installation(exec_option)
 
     view, tql_filter = filter_utils.attribute_based_filtering_tql(
         view=dataset,
@@ -52,4 +48,7 @@ def vector_search(
         runtime=runtime,
         return_tensors=return_tensors,
         return_view=return_view,
+        token=token,
+        org_id=org_id,
+        return_tql=return_tql,
     )
