@@ -1,14 +1,22 @@
-from deeplake.constants import MB
-from deeplake.enterprise.util import raise_indra_installation_error
-from deeplake.util.warnings import always_warn
+import functools
+import types
+import random
+import string
+from abc import ABC, abstractmethod
+from typing import Optional, List, Dict, Tuple
 
-from deeplake.core.dataset import DeepLakeCloudDataset
+import deeplake
+from deeplake.constants import MB, DEFAULT_VECTORSTORE_INDEX_PARAMS, TARGET_BYTE_SIZE
+from deeplake.enterprise.util import INDRA_INSTALLED
+from deeplake.util.exceptions import TensorDoesNotExistError
+from deeplake.util.warnings import always_warn
+from deeplake.core.dataset import DeepLakeCloudDataset, Dataset
+from deeplake.core.vectorstore.embeddings.embedder import DeepLakeEmbedder
+from deeplake.client.client import DeepLakeBackendClient
+from deeplake.util.path import get_path_type
 
 import numpy as np
 
-import random
-import string
-from typing import Optional, List, Dict, Callable
 
 EXEC_OPTION_TO_RUNTIME: Dict[str, Optional[Dict]] = {
     "compute_engine": None,
@@ -21,21 +29,88 @@ def parse_tensor_return(tensor):
     return tensor.data(aslist=True)["value"]
 
 
-def parse_exec_option(dataset, exec_option, indra_installed):
-    """Select the best available exec_option for the given dataset and environment"""
+class ExecOptionBase(ABC):
+    def get_token(self, token):
+        user_profile = self.client.get_user_profile()
+        if user_profile["name"] != "public":
+            token = token or self.client.get_token()
+        return token
 
-    if exec_option is None or exec_option == "auto":
-        if isinstance(dataset, DeepLakeCloudDataset):
-            if "vector_db/" in dataset.base_storage.path:
-                return "tensor_db"
-            elif indra_installed:
-                return "compute_engine"
-            else:
-                return "python"
+    @abstractmethod
+    def get_exec_option(self):
+        return NotImplementedError()
+
+
+class ExecOptionCloudDataset(ExecOptionBase):
+    def __init__(self, dataset, username, path_type):
+        self.dataset = dataset
+        self.client = dataset.client
+        self.token = self.dataset.token
+        self.username = username
+        self.path_type = path_type
+
+    def get_exec_option(self):
+        # option 1: dataset is created in vector_db:
+        if (
+            isinstance(self.dataset, DeepLakeCloudDataset)
+            and "vectordb/" in self.dataset.base_storage.root
+            and self.token is not None
+        ):
+            return "tensor_db"
+        # option 2: dataset is created in a linked storage or locally,
+        # indra is installed user/org has access to indra
+        elif self.path_type == "hub" and INDRA_INSTALLED and self.username != "public":
+            return "compute_engine"
         else:
             return "python"
-    else:
-        return exec_option
+
+
+class ExecOptionLocalDataset(ExecOptionBase):
+    def __init__(self, dataset, username):
+        self.dataset = dataset
+        self.token = self.dataset.token
+        self.username = username
+
+    def get_exec_option(self):
+        if self.token is None:
+            return "python"
+
+        if "mem://" in self.dataset.path:
+            return "python"
+
+        if INDRA_INSTALLED and self.username != "public":
+            return "compute_engine"
+        return "python"
+
+
+def exec_option_factory(dataset, username):
+    path_type = get_path_type(dataset.path)
+    if path_type == "local":
+        return ExecOptionLocalDataset(dataset, username)
+    return ExecOptionCloudDataset(dataset, username, path_type)
+
+
+def parse_exec_option(dataset, exec_option, username):
+    if exec_option is None or exec_option == "auto":
+        exec_option = exec_option_factory(dataset, username)
+        return exec_option.get_exec_option()
+    return exec_option
+
+
+def parse_index_params(index_params):
+    ip = DEFAULT_VECTORSTORE_INDEX_PARAMS.copy()
+    valid_keys = ip.keys()
+
+    if index_params:
+        for key, value in index_params.items():
+            if key not in valid_keys:
+                raise ValueError(
+                    f"Invalid key '{key}' in index_params. Valid keys are: {valid_keys}"
+                )
+
+            ip[key] = value
+
+    return ip
 
 
 def parse_return_tensors(dataset, return_tensors, embedding_tensor, return_view):
@@ -49,11 +124,17 @@ def parse_return_tensors(dataset, return_tensors, embedding_tensor, return_view)
             for tensor in dataset.tensors
             if (tensor != embedding_tensor or return_tensors == "*")
         ]
+    for tensor in return_tensors:
+        if tensor not in dataset.tensors:
+            raise TensorDoesNotExistError(tensor)
+
     return return_tensors
 
 
-def check_indra_installation(exec_option, indra_installed):
-    if exec_option == "compute_engine" and not indra_installed:
+def check_indra_installation(exec_option):
+    if exec_option == "compute_engine" and not INDRA_INSTALLED:
+        from deeplake.enterprise.util import raise_indra_installation_error
+
         raise raise_indra_installation_error(
             indra_import_error=False
         )  # pragma: no cover
@@ -75,6 +156,8 @@ def check_length_of_each_tensor(tensors):
                 f"All of the tensors should have equal length. Currently tensors have different length: {tensor_lengths}"
             )
 
+    return tensor_length
+
 
 def create_tensor_to_length_str(tensors):
     tensor_lengths = "\n"
@@ -85,10 +168,6 @@ def create_tensor_to_length_str(tensors):
     return tensor_lengths
 
 
-random.seed(0)
-np.random.seed(0)
-
-
 def generate_random_string(length):
     # Define the character set to include letters (both lowercase and uppercase) and digits
     characters = string.ascii_letters + string.digits
@@ -98,18 +177,19 @@ def generate_random_string(length):
     return random_string
 
 
-def generate_json(value):
-    key = "abc"
+def generate_json(value, key):
     return {key: value}
 
 
-def create_data(number_of_data, embedding_dim=100):
+def create_data(
+    number_of_data, embedding_dim=100, metadata_key="abc", string_length=1000
+):
     embeddings = np.random.uniform(
         low=-10, high=10, size=(number_of_data, embedding_dim)
     ).astype(np.float32)
-    texts = [generate_random_string(1000) for i in range(number_of_data)]
+    texts = [generate_random_string(string_length) for i in range(number_of_data)]
     ids = [f"{i}" for i in range(number_of_data)]
-    metadata = [generate_json(i) for i in range(number_of_data)]
+    metadata = [generate_json(i, metadata_key) for i in range(number_of_data)]
     images = ["deeplake/tests/dummy_data/images/car.jpg" for i in range(number_of_data)]
     return texts, embeddings, ids, metadata, images
 
@@ -189,7 +269,12 @@ def get_embedding_tensor(embedding_tensor, embedding_source_tensor, dataset):
     return embedding_tensor
 
 
-def parse_tensors_kwargs(tensors, embedding_function, embedding_data, embedding_tensor):
+def parse_tensors_kwargs(
+    tensors,
+    embedding_function,
+    embedding_data,
+    embedding_tensor,
+):
     tensors = tensors.copy()
 
     # embedding_tensor = (embedding_function, embedding_data) syntax
@@ -222,11 +307,18 @@ def parse_tensors_kwargs(tensors, embedding_function, embedding_data, embedding_
                 "Cannot specify embedding tensors in both `tensors` and `embedding_tensor`."
             )
     else:
+        if isinstance(embedding_function, list):
+            embedding_function = [
+                create_embedding_function(fn_i) for fn_i in embedding_function
+            ]
+        else:
+            embedding_function = create_embedding_function(embedding_function)
         return embedding_function, embedding_data, embedding_tensor, tensors
 
     # separate embedding functions, data and tensors
     for k, v in filtered.items():
-        funcs.append(v[0])
+        func = create_embedding_function(v[0])
+        funcs.append(func)
         data.append(v[1])
         tensors_.append(k)
         # remove embedding tensors (tuple format) from tensors
@@ -235,6 +327,54 @@ def parse_tensors_kwargs(tensors, embedding_function, embedding_data, embedding_
     return funcs, data, tensors_, tensors
 
 
+def _validate_embedding_functions(embedding_function, initial_embedding_function):
+    if embedding_function is None and initial_embedding_function is None:
+        raise ValueError(
+            "`embedding_function` was not specified during initialization of vector store or the update call"
+        )
+
+
+def _get_single_value_from_list(data):
+    if isinstance(data, list) and len(data) == 1:
+        return data[0]
+    return data
+
+
+def _validate_source_and_embedding_tensors(embedding_source_tensor, embedding_tensor):
+    if isinstance(embedding_source_tensor, str) and isinstance(embedding_tensor, list):
+        raise ValueError(
+            "Multiple `embedding_tensor` were specified while a single `embedding_source_tensor` was given."
+        )
+
+    if (
+        isinstance(embedding_source_tensor, list)
+        and len(embedding_source_tensor) > 1
+        and isinstance(embedding_tensor, str)
+    ):
+        raise ValueError(
+            "Multiple `embedding_source_tensor` were specified while a single `embedding_tensor` was given."
+        )
+
+
+def _convert_to_embedder_list(embedding_function):
+    if isinstance(embedding_function, list):
+        return [DeepLakeEmbedder(embedding_function=fn) for fn in embedding_function]
+
+    valid_function_types = (
+        types.MethodType,
+        types.FunctionType,
+        types.LambdaType,
+        functools.partial,
+    )
+    if isinstance(embedding_function, valid_function_types):
+        return DeepLakeEmbedder(embedding_function=embedding_function)
+
+    if embedding_function is not None:
+        raise ValueError(
+            "Invalid `embedding_function` type. It should be either a function or a list of functions."
+        )
+
+
 def parse_update_arguments(
     dataset,
     embedding_function=None,
@@ -242,37 +382,30 @@ def parse_update_arguments(
     embedding_source_tensor=None,
     embedding_tensor=None,
 ):
-    if embedding_function is None and initial_embedding_function is None:
-        raise ValueError(
-            "`embedding_function` was not specified during initialization of vector store or the update call"
-        )
+    _validate_embedding_functions(embedding_function, initial_embedding_function)
 
     embedding_tensor = get_embedding_tensor(
         embedding_tensor, embedding_source_tensor, dataset
     )
-    if isinstance(embedding_tensor, list) and len(embedding_tensor) == 1:
-        embedding_tensor = embedding_tensor[0]
+    embedding_tensor = _get_single_value_from_list(embedding_tensor)
 
-    if isinstance(embedding_source_tensor, str) and isinstance(embedding_tensor, list):
-        raise ValueError(
-            "Multiple `embedding_tensor` were specifed. "
-            "While single `embedding_source_tensor` was given. "
-        )
-    elif (
-        isinstance(embedding_source_tensor, list)
-        and len(embedding_source_tensor) > 1
-        and isinstance(embedding_tensor, str)
-    ):
-        raise ValueError(
-            "Multiple `embedding_source_tensor` were specifed. "
-            "While single `embedding_tensor` was given. "
-        )
+    _validate_source_and_embedding_tensors(embedding_source_tensor, embedding_tensor)
 
+    embedding_function = _convert_to_embedder_list(embedding_function)
     final_embedding_function = embedding_function or initial_embedding_function
 
-    if isinstance(embedding_tensor, list) and callable(final_embedding_function):
+    if isinstance(embedding_tensor, list) and not isinstance(
+        final_embedding_function, list
+    ):
         final_embedding_function = [final_embedding_function] * len(embedding_tensor)
 
+    if isinstance(final_embedding_function, list):
+        final_embedding_function = [
+            fn.embed_documents for fn in final_embedding_function
+        ]
+    else:
+        final_embedding_function = final_embedding_function.embed_documents
+
     if isinstance(embedding_tensor, list) and isinstance(embedding_source_tensor, list):
         assert len(embedding_tensor) == len(embedding_source_tensor), (
             "The length of the `embedding_tensor` does not match the length of "
@@ -334,7 +467,12 @@ def parse_add_arguments(
             tensors,
             dataset,
         )
-        return (embedding_function, embedding_data, embedding_tensor, tensors)
+        return (
+            [fn.embed_documents for fn in embedding_function],
+            embedding_data,
+            embedding_tensor,
+            tensors,
+        )
 
     if initial_embedding_function:
         if not embedding_data:
@@ -351,7 +489,12 @@ def parse_add_arguments(
             tensors,
             dataset,
         )
-        return (initial_embedding_function, embedding_data, embedding_tensor, tensors)
+        return (
+            [fn.embed_documents for fn in initial_embedding_function],
+            embedding_data,
+            embedding_tensor,
+            tensors,
+        )
 
     if embedding_tensor:
         raise ValueError(
@@ -456,12 +599,33 @@ def get_embedding_tensors(embedding_tensor, tensor_args, dataset) -> List[str]:
 def find_embedding_tensors(dataset) -> List[str]:
     """Find all the embedding tensors in a dataset."""
     matching_tensors = []
-    for tensor in dataset.tensors:
-        if (
-            dataset[tensor].htype == "embedding"
-            or tensor == "embedding"
-            or tensor == "embeddings"
-        ):
-            matching_tensors.append(tensor)
+    for tensor in dataset.tensors.values():
+        if is_embedding_tensor(tensor):
+            matching_tensors.append(tensor.key)
 
     return matching_tensors
+
+
+def is_embedding_tensor(tensor):
+    """Check if a tensor is an embedding tensor."""
+
+    valid_names = ["embedding", "embeddings"]
+
+    return (
+        tensor.htype == "embedding"
+        or tensor.meta.name in valid_names
+        or tensor.key in valid_names
+    )
+
+
+def index_used(exec_option):
+    """Check if the index is used for the exec_option"""
+    return exec_option in ("tensor_db", "compute_engine")
+
+
+def create_embedding_function(embedding_function):
+    if embedding_function:
+        return DeepLakeEmbedder(
+            embedding_function=embedding_function,
+        )
+    return None
