@@ -1,5 +1,9 @@
 import glob
+import json
 from collections import OrderedDict
+
+from deeplake.constants import FIRST_COMMIT_ID
+
 import deeplake
 import pytest
 import numpy as np
@@ -8,6 +12,7 @@ from deeplake.util.diff import (
     get_lowest_common_ancestor,
     sanitize_commit,
 )
+from deeplake.tests.common import disabale_hidden_tensors_config
 from deeplake.util.remove_cache import get_base_storage
 from deeplake.util.exceptions import (
     CheckoutError,
@@ -1418,9 +1423,9 @@ def test_rename_diff_branch(local_ds, capsys):
     with local_ds:
         local_ds.rename_tensor("abc", "xyz")
         expected_dataset_diff_from_a_on_alt["renamed"]["abc"] = "xyz"
-        expected_tensor_diff_from_a_on_alt[
-            "xyz"
-        ] = expected_tensor_diff_from_a_on_alt.pop("abc")
+        expected_tensor_diff_from_a_on_alt["xyz"] = (
+            expected_tensor_diff_from_a_on_alt.pop("abc")
+        )
         local_ds.xyz.append([4, 5, 6])
         expected_tensor_diff_from_a_on_alt["xyz"]["data_added"] = [1, 2]
 
@@ -1455,18 +1460,18 @@ def test_rename_diff_branch(local_ds, capsys):
     with local_ds:
         local_ds.rename_tensor("abc", "efg")
         expected_dataset_diff_from_c_on_alt2["renamed"]["abc"] = "efg"
-        expected_tensor_diff_from_c_on_alt2[
-            "efg"
-        ] = expected_tensor_diff_from_c_on_alt2.pop("abc")
+        expected_tensor_diff_from_c_on_alt2["efg"] = (
+            expected_tensor_diff_from_c_on_alt2.pop("abc")
+        )
         local_ds.efg.append([5, 6, 7])
         expected_tensor_diff_from_c_on_alt2["efg"]["data_added"] = [2, 3]
         local_ds.efg.info["hello"] = "world"
         expected_tensor_diff_from_c_on_alt2["efg"]["info_updated"] = True
         local_ds.rename_tensor("red", "blue")
         expected_dataset_diff_from_c_on_alt2["renamed"]["red"] = "blue"
-        expected_tensor_diff_from_c_on_alt2[
-            "blue"
-        ] = expected_tensor_diff_from_c_on_alt2.pop("red")
+        expected_tensor_diff_from_c_on_alt2["blue"] = (
+            expected_tensor_diff_from_c_on_alt2.pop("red")
+        )
 
     d = local_ds.commit()
     expected_tensor_diff_from_d_on_alt2 = {
@@ -2361,15 +2366,15 @@ def test_reset_create_delete_tensors(local_ds):
     "ds_generator",
     [
         "local_ds_generator",
-        "s3_ds_generator",
-        "gcs_ds_generator",
-        "hub_cloud_ds_generator",
+        pytest.param("s3_ds_generator", marks=pytest.mark.slow),
+        pytest.param("gcs_ds_generator", marks=pytest.mark.slow),
+        pytest.param("hub_cloud_ds_generator", marks=pytest.mark.slow),
     ],
     indirect=True,
 )
 def test_reset_bug(ds_generator):
     ds = ds_generator()
-    ds.create_tensor("abc")
+    ds.create_tensor("abc", **disabale_hidden_tensors_config)
     ds.abc.append([1, 2, 3])
     assert len(ds.abc) == 1
     a = ds.commit()
@@ -2440,6 +2445,7 @@ def test_version_in_path(local_path):
         deeplake.exists(f"{local_path}@main")
 
 
+@pytest.mark.slow
 def test_branch_delete(local_ds_generator):
     local_ds = local_ds_generator()
     local_ds.create_tensor("test")
@@ -2550,3 +2556,88 @@ def test_branch_delete(local_ds_generator):
 
     local_ds.delete_branch("alt1_sub1")
     assert len(local_ds.branches) == 2
+
+
+def test_squash_main_has_branch(local_ds_generator):
+    local_ds = local_ds_generator()
+    local_ds.create_tensor("test")
+    with local_ds:
+        local_ds.test.append("main 1")
+        local_ds.commit("first main commit")
+    local_ds.checkout("alt", create=True)
+
+    with pytest.raises(VersionControlError) as e:
+        local_ds._squash_main()
+    assert "Cannot squash commits if there are multiple branches" in str(e.value)
+
+
+def test_squash_main_has_view(local_ds_generator):
+    local_ds = local_ds_generator()
+    local_ds.create_tensor("test")
+    with local_ds:
+        local_ds.test.append("main 1")
+        local_ds.commit("first main commit")
+    query = local_ds.filter("test == 'a'")
+    query.save_view("test_view")
+
+    with pytest.raises(VersionControlError) as e:
+        local_ds._squash_main()
+    assert "Cannot squash commits if there are views present" in str(e.value)
+
+
+def test_squash_main(local_ds_generator):
+    local_ds = local_ds_generator()
+    local_ds.create_tensor("test")
+
+    with local_ds:
+        # Add commits to main
+        local_ds.test.append("main 1")
+        local_ds.test.append("main 2")
+        local_ds.commit("first main commit")
+        local_ds.test.append("main 3")
+        local_ds.commit("second main commit")
+        local_ds.test.append("main 4")
+        local_ds.commit("third main commit")
+        local_ds.test.append("main uncommitted")
+
+    assert len(local_ds.branches) == 1
+    assert len(glob.glob(local_ds.path + "/versions/*")) > 0
+    assert len(local_ds.test) == 5
+    assert [i.data()["value"] for i in local_ds.test] == [
+        "main 1",
+        "main 2",
+        "main 3",
+        "main 4",
+        "main uncommitted",
+    ]
+    assert [i["message"] for i in local_ds.commits] == [
+        "third main commit",
+        "second main commit",
+        "first main commit",
+    ]
+
+    local_ds._squash_main()
+
+    assert len(local_ds.branches) == 1
+    assert len(glob.glob(local_ds.path + "/versions/*")) == 1
+    assert [commit["message"] for commit in local_ds.commits] == ["Squashed commits"]
+    assert local_ds.pending_commit_id != FIRST_COMMIT_ID
+
+    with open(local_ds.path + "/version_control_info.json", "r") as f:
+        data = json.load(f)
+        assert len(data["commits"]) == 1
+        assert data["commits"][FIRST_COMMIT_ID]["commit_message"] == None
+        assert data["commits"][FIRST_COMMIT_ID]["commit_time"] == None
+        assert data["commits"][FIRST_COMMIT_ID]["commit_user_name"] == None
+        assert len(data["commits"][FIRST_COMMIT_ID]["children"]) == 0
+        assert data["commits"][FIRST_COMMIT_ID]["parent"] == None
+
+    assert [i.data()["value"] for i in local_ds.test] == [
+        "main 1",
+        "main 2",
+        "main 3",
+        "main 4",
+        "main uncommitted",
+    ]
+    assert [i["message"] for i in local_ds.commits] == ["Squashed commits"]
+    assert local_ds.pending_commit_id != FIRST_COMMIT_ID
