@@ -3,6 +3,7 @@ import deeplake
 from deeplake.enterprise.convert_to_libdeeplake import dataset_to_libdeeplake
 from deeplake.enterprise.dummy_dataloader import DummyDataloader  # type: ignore
 from deeplake.util.scheduling import create_fetching_schedule, find_primary_tensor
+from deeplake.core.seed import DeeplakeRandom
 from deeplake.enterprise.util import (
     handle_mode,
     raise_indra_installation_error,
@@ -25,16 +26,21 @@ import importlib
 try:
     from torch.utils.data.dataloader import DataLoader, _InfiniteConstantSampler
     from torch.utils.data.distributed import DistributedSampler
+    from torch.utils.data import BatchSampler
+
 except ImportError:
     DataLoader = object  # type: ignore
     _InfiniteConstantSampler = None  # type: ignore
     DistributedSampler = None  # type: ignore
+    BatchSampler = None  # type: ignore
 
 import numpy as np
+import warnings
 
 import math
 
 import itertools
+import multiprocessing
 
 original_islice = itertools.islice
 
@@ -49,7 +55,7 @@ itertools.islice = deeplake_islice  # type: ignore
 
 
 # Load lazy to avoid cycylic import.
-INDRA_LOADER = None
+INDRA_LOADER = None  # type: ignore
 
 
 def indra_available() -> bool:
@@ -62,6 +68,7 @@ def indra_available() -> bool:
 
 def import_indra_loader():
     global INDRA_LOADER
+    global INDRA_API
     if INDRA_LOADER:
         return INDRA_LOADER
     if not importlib.util.find_spec("indra"):
@@ -70,11 +77,13 @@ def import_indra_loader():
         from indra import api  # type: ignore
         from indra.pytorch.loader import Loader  # type:ignore
 
+        INDRA_API = api
         INDRA_LOADER = Loader
         return Loader
     except Exception as e:
         if not deeplake.constants.RETURN_DUMMY_DATA_FOR_DATALOADER:
             raise_indra_installation_error(e)
+        INDRA_API = None
         INDRA_LOADER = None
 
 
@@ -102,6 +111,9 @@ class DeepLakeDataLoader(DataLoader):
         _dataloader=None,
         _world_size=1,
         _ignore_errors=False,
+        _verbose=False,
+        _offset=None,
+        _sampler=None,
         **kwargs,
     ):
         import_indra_loader()
@@ -126,6 +138,9 @@ class DeepLakeDataLoader(DataLoader):
         self._dataloader = _dataloader
         self._world_size = _world_size
         self._ignore_errors = _ignore_errors
+        self._verbose = _verbose
+        self._offset = _offset
+        self._sampler = _sampler
         for k, v in kwargs.items():
             setattr(self, k, v)
 
@@ -133,6 +148,8 @@ class DeepLakeDataLoader(DataLoader):
         self.__initialized = True
         self._IterableDataset_len_called = None
         self._iterator = None
+        self._worker_init_fn = None
+        self.__multiprocessing_context = None
 
     @property
     def batch_size(self):
@@ -164,11 +181,56 @@ class DeepLakeDataLoader(DataLoader):
 
     @property
     def worker_init_fn(self):
-        return None
+        return self._worker_init_fn
+
+    @worker_init_fn.setter
+    def worker_init_fn(self, fn):
+        self._worker_init_fn = fn
+        if self._dataloader is not None:
+            self._dataloader.worker_init_fn = fn
 
     @property  # type: ignore
     def multiprocessing_context(self):
-        return None
+        return self.__multiprocessing_context
+
+    @multiprocessing_context.setter
+    def multiprocessing_context(self, multiprocessing_context):
+        if multiprocessing_context is not None:
+            if self.num_workers > 0:
+                if isinstance(multiprocessing_context, str):
+                    valid_start_methods = multiprocessing.get_all_start_methods()
+                    if multiprocessing_context not in valid_start_methods:
+                        raise ValueError(
+                            (
+                                "multiprocessing_context option "
+                                "should specify a valid start method in {!r}, but got "
+                                "multiprocessing_context={!r}"
+                            ).format(valid_start_methods, multiprocessing_context)
+                        )
+                    multiprocessing_context = multiprocessing.get_context(
+                        multiprocessing_context
+                    )
+
+                if not isinstance(
+                    multiprocessing_context, multiprocessing.context.BaseContext
+                ):
+                    raise TypeError(
+                        (
+                            "multiprocessing_context option should be a valid context "
+                            "object or a string specifying the start method, but got "
+                            "multiprocessing_context={}"
+                        ).format(multiprocessing_context)
+                    )
+            else:
+                raise ValueError(
+                    (
+                        "multiprocessing_context can only be used with "
+                        "multi-process loading (num_workers > 0), but got "
+                        "num_workers={}"
+                    ).format(self.num_workers)
+                )
+
+        self.__multiprocessing_context = multiprocessing_context
 
     @property
     def _dataset_kind(self):
@@ -176,15 +238,25 @@ class DeepLakeDataLoader(DataLoader):
 
     @property
     def sampler(self):
+        if self._sampler is not None:
+            return self._sampler
         return (
-            DistributedSampler(self.dataset)
+            DistributedSampler(self._orig_dataset)
             if self._distributed
             else _InfiniteConstantSampler()
         )
 
+    @sampler.setter
+    def sampler(self, value):
+        self._sampler = value
+
     @property
     def batch_sampler(self):
-        return DistributedSampler(self.dataset) if self._distributed else None
+        return (
+            BatchSampler(self.sampler, self.batch_size, self.drop_last)
+            if BatchSampler is not None
+            else None
+        )
 
     @property
     def generator(self):
@@ -214,10 +286,13 @@ class DeepLakeDataLoader(DataLoader):
         return get_collate_fn(self._collate, self._mode)
 
     def __len__(self):
-        round_fn = math.floor if self._drop_last else math.ceil
-        return round_fn(
-            len(self._orig_dataset) / ((self.batch_size) * self._world_size)
+        len_ds = (
+            len(self._orig_dataset[self._tensors])
+            if self._tensors is not None
+            else len(self._orig_dataset)
         )
+        round_fn = math.floor if self._drop_last else math.ceil
+        return round_fn(len_ds / ((self.batch_size) * self._world_size))
 
     def batch(self, batch_size: int, drop_last: bool = False):
         """Returns a batched :class:`DeepLakeDataLoader` object.
@@ -240,6 +315,25 @@ class DeepLakeDataLoader(DataLoader):
         all_vars["_drop_last"] = drop_last
         return self.__class__(**all_vars)
 
+    def offset(self, off: int = 0):
+        """Returns a shifted :class:`DeepLakeDataLoader` object.
+
+        Args:
+            off (int): index that the dataloadee will start to iterate.
+
+        Returns:
+            DeepLakeDataLoader: A :class:`DeepLakeDataLoader` object.
+
+        Raises:
+            ValueError: If .offset() has already been called.
+        """
+        if self._offset is not None:
+            raise ValueError("offset is already set")
+
+        all_vars = self.__dict__.copy()
+        all_vars["_offset"] = off
+        return self.__class__(**all_vars)
+
     def shuffle(self, shuffle: bool = True, buffer_size: int = 2048):
         """Returns a shuffled :class:`DeepLakeDataLoader` object.
 
@@ -260,10 +354,12 @@ class DeepLakeDataLoader(DataLoader):
         all_vars["_shuffle"] = shuffle
         all_vars["_buffer_size"] = buffer_size
         if shuffle:
-            schedule = create_fetching_schedule(self.dataset, self._primary_tensor_name)
+            schedule = create_fetching_schedule(
+                self._orig_dataset, self._primary_tensor_name
+            )
             if schedule is not None:
-                ds = self.dataset.no_view_dataset  # type: ignore
-                all_vars["dataset"] = ds[schedule]
+                ds = self._orig_dataset  # type: ignore
+                all_vars["_orig_dataset"] = ds[schedule]
         all_vars["_dataloader"] = None
         return self.__class__(**all_vars)
 
@@ -290,12 +386,12 @@ class DeepLakeDataLoader(DataLoader):
         all_vars = self.__dict__.copy()
         if isinstance(transform, dict):
             tensors = [k for k in transform.keys() if k != "index"]
-            tensors = map_tensor_keys(self.dataset, tensors)
+            tensors = map_tensor_keys(self._orig_dataset, tensors)
             if self._tensors:
                 raise ValueError(
                     f"Tensors have already been specified in the .{self._mode} method."
                 )
-            all_vars["_tensors"] = map_tensor_keys(self.dataset, tensors)
+            all_vars["_tensors"] = map_tensor_keys(self._orig_dataset, tensors)
             transform = PytorchTransformFunction(transform_dict=transform)
         else:
             if kwargs:
@@ -326,7 +422,7 @@ class DeepLakeDataLoader(DataLoader):
             >>> query_ds_train = ds_train.dataloader().query("(select * where contains(categories, 'car') limit 1000) union (select * where contains(categories, 'motorcycle') limit 1000)")
         """
         all_vars = self.__dict__.copy()
-        all_vars["dataset"] = query(self.dataset, query_string)
+        all_vars["_orig_dataset"] = query(self._orig_dataset, query_string)
         all_vars["_dataloader"] = None
         return self.__class__(**all_vars)
 
@@ -371,16 +467,17 @@ class DeepLakeDataLoader(DataLoader):
 
         """
         all_vars = self.__dict__.copy()
-        all_vars["dataset"] = sample_by(
-            self.dataset, weights, replace=replace, size=size
+        all_vars["_orig_dataset"] = sample_by(
+            self._orig_dataset, weights, replace=replace, size=size
         )
         all_vars["_dataloader"] = None
         return self.__class__(**all_vars)
 
     def close(self):
         """Shuts down the workers and releases the resources."""
-        if self._dataloader is not None:
-            self._dataloader.close()
+        if hasattr(self, "_iterator") and self._iterator is not None:
+            self._iterator.close()
+        if hasattr(self, "_dataloader") and self._dataloader is not None:
             self._dataloader = None
 
     def pytorch(
@@ -395,13 +492,15 @@ class DeepLakeDataLoader(DataLoader):
         decode_method: Optional[Dict[str, str]] = None,
         persistent_workers: bool = False,
     ):
-        """Returns a :class:`DeepLakeDataLoader` object.
-
+        """Creates a PyTorch Dataloader on top of the ``DeepLakeDataLoader`` from the Deep Lake dataset. During iteration, the data from all tensors will be streamed on-the-fly from the storage location.
+        Understanding the parameters below is critical for achieving fast streaming for your use-case
 
         Args:
             num_workers (int): Number of workers to use for transforming and processing the data. Defaults to 0.
             collate_fn (Callable, Optional): merges a list of samples to form a mini-batch of Tensor(s).
-            tensors (List[str], Optional): List of tensors to load. If None, all tensors are loaded. Defaults to ``None``.
+            tensors (List, Optional): List of tensors to load. If ``None``, all tensors are loaded. Defaults to ``None``.
+                For datasets with many tensors, its extremely important to stream only the data that is needed for training the model, in order to avoid bottlenecks associated with streaming unused data.
+                For example, if you have a dataset that has ``image``, ``label``, and ``metadata`` tensors, if ``tensors=["image", "label"]``, the Data Loader will only stream the ``image`` and ``label`` tensors.
             num_threads (int, Optional): Number of threads to use for fetching and decompressing the data. If ``None``, the number of threads is automatically determined. Defaults to ``None``.
             prefetch_factor (int): Number of batches to transform and collate in advance per worker. Defaults to 2.
             distributed (bool): Used for DDP training. Distributes different sections of the dataset to different ranks. Defaults to ``False``.
@@ -454,7 +553,7 @@ class DeepLakeDataLoader(DataLoader):
         all_vars = self.__dict__.copy()
         all_vars["_num_workers"] = num_workers
         all_vars["_collate"] = collate_fn
-        validate_tensors(tensors, self.dataset, all_vars)
+        validate_tensors(tensors, self._orig_dataset, all_vars)
         all_vars["_decode_method"] = decode_method
         all_vars["_num_threads"] = num_threads
         all_vars["_prefetch_factor"] = prefetch_factor
@@ -484,18 +583,19 @@ class DeepLakeDataLoader(DataLoader):
         Args:
             num_workers (int): Number of workers to use for transforming and processing the data. Defaults to 0.
             collate_fn (Callable, Optional): merges a list of samples to form a mini-batch of Tensor(s).
-            tensors (List[str], Optional): List of tensors to load. If None, all tensors are loaded. Defaults to ``None``.
+            tensors (List, Optional): List of tensors to load. If ``None``, all tensors are loaded. Defaults to ``None``.
+                For datasets with many tensors, its extremely important to stream only the data that is needed for training the model, in order to avoid bottlenecks associated with streaming unused data.
+                For example, if you have a dataset that has ``image``, ``label``, and ``metadata`` tensors, if ``tensors=["image", "label"]``, the Data Loader will only stream the ``image`` and ``label`` tensors.
             num_threads (int, Optional): Number of threads to use for fetching and decompressing the data. If ``None``, the number of threads is automatically determined. Defaults to ``None``.
             prefetch_factor (int): Number of batches to transform and collate in advance per worker. Defaults to 2.
-            return_index (bool): Used to idnetify where loader needs to retur sample index or not. Defaults to ``True``.
+            return_index (bool): If ``True``, the returned dataloader will have a key "index" that contains the index of the sample(s) in the original dataset. Default value is True.
             persistent_workers (bool): If ``True``, the data loader will not shutdown the worker processes after a dataset has been consumed once. Defaults to ``False``.
-            decode_method (Dict[str, str], Optional): A dictionary of decode methods for each tensor. Defaults to ``None``.
-
+            decode_method (Dict[str, str], Optional): The method for decoding the Deep Lake tensor data, the result of which is passed to the transform. Decoding occurs outside of the transform so that it can be performed in parallel and as rapidly as possible as per Deep Lake optimizations.
 
                 - Supported decode methods are:
-
-                    :'numpy': Default behaviour. Returns samples as numpy arrays.
-                    :'tobytes': Returns raw bytes of the samples.
+                    :'numpy': Default behaviour. Returns samples as numpy arrays, the same as ds.tensor[i].numpy()
+                    :'tobytes': Returns raw bytes of the samples the same as ds.tensor[i].tobytes()
+                    :'data': Returns a dictionary with keys,values depending on htype, the same as ds.tensor[i].data()
                     :'pil': Returns samples as PIL images. Especially useful when transformation use torchvision transforms, that
                             require PIL images as input. Only supported for tensors with ``sample_compression='jpeg'`` or ``'png'``.
 
@@ -529,7 +629,7 @@ class DeepLakeDataLoader(DataLoader):
         all_vars = self.__dict__.copy()
         all_vars["_num_workers"] = num_workers
         all_vars["_collate"] = collate_fn
-        validate_tensors(tensors, self.dataset, all_vars)
+        validate_tensors(tensors, self._orig_dataset, all_vars)
         all_vars["_decode_method"] = decode_method
         all_vars["_num_threads"] = num_threads
         all_vars["_prefetch_factor"] = prefetch_factor
@@ -574,7 +674,7 @@ class DeepLakeDataLoader(DataLoader):
         handle_mode(self._mode, mode)
         all_vars = self.__dict__.copy()
         all_vars["_num_workers"] = num_workers
-        validate_tensors(tensors, self.dataset, all_vars)
+        validate_tensors(tensors, self._orig_dataset, all_vars)
         all_vars["_decode_method"] = decode_method
         all_vars["_tensors"] = self._tensors or tensors
         all_vars["_num_threads"] = num_threads
@@ -584,15 +684,162 @@ class DeepLakeDataLoader(DataLoader):
         all_vars["_dataloader"] = None
         return self.__class__(**all_vars)
 
+    def _get_suboptimal_thread_count(self) -> Optional[int]:
+        assert self._distributed
+
+        if self._num_threads is None and self._mode == "pytorch":
+            import torch
+
+            num_devices = (
+                torch.cuda.device_count() if torch.cuda.is_available() else None
+            )
+
+            num_suboptimal_threads = (
+                int(INDRA_API.num_available_threads() / num_devices)  # type: ignore [name-defined]
+                if INDRA_API is not None and num_devices is not None  # type: ignore [name-defined]
+                else None
+            )
+            return num_suboptimal_threads
+        return self._num_threads
+
+    def __is_upcast_needed(self, dataset, tensors) -> bool:
+        for tensor_name in tensors:
+            tensor = dataset._get_tensor_from_root(tensor_name)
+            dtype = tensor.dtype
+            if dtype is None:
+                return True
+
+            if dtype.type in [np.uint16, np.uint32, np.uint64]:
+                return True
+
+            if hasattr(dtype, "type"):
+                if isinstance(
+                    dtype.type, (type(np.uint16), type(np.uint32), type(np.uint64))
+                ):
+                    return True
+        return False
+
+    def __create_dummy_dataloader(
+        self,
+        dataset,
+        tensors: Optional[List[str]] = None,
+        raw_tensors: Optional[List[str]] = None,
+        pil_compressed_tensors: Optional[List[str]] = None,
+    ) -> DummyDataloader:
+        return DummyDataloader(
+            deeplake_dataset=dataset,
+            batch_size=self._batch_size,
+            shuffle=self._shuffle,
+            num_workers=self._num_workers,
+            collate_fn=self.collate_fn,
+            transform_fn=self._transform,
+            distributed=self._distributed,
+            prefetch_factor=self._prefetch_factor,
+            tensors=tensors,
+            drop_last=self._drop_last,
+            upcast=self._mode == "pytorch"
+            and self.__is_upcast_needed(
+                dataset, tensors
+            ),  # upcast to handle unsupported dtypes,
+            return_index=self._return_index,
+            raw_tensors=raw_tensors,
+            pil_compressed_tensors=pil_compressed_tensors,
+            persistent_workers=self._persistent_workers,
+        )
+
+    def __get_indra_dataloader(
+        self,
+        dataset,
+        indra_dataset,
+        tensors: Optional[List[str]] = None,
+        raw_tensors: Optional[List[str]] = None,
+        pil_compressed_tensors: Optional[List[str]] = None,
+        json_tensors: Optional[List[str]] = None,
+        list_tensors: Optional[List[str]] = None,
+        htype_dict: Optional[dict] = None,
+        ndim_dict: Optional[dict] = None,
+        tensor_info_dict: Optional[dict] = None,
+    ):
+        num_threads = (
+            self._get_suboptimal_thread_count()
+            if self._distributed
+            else self._num_threads
+        )
+        seed = DeeplakeRandom().get_seed()
+        if self._offset is not None and self._shuffle and seed is None:
+            warnings.warn(
+                "offset and shuffle parameters are set without a random seed. This means that the ordering of the samples are "
+                "not equal after each initialization and iteration through the dataloader. If you intend to shuffle data while "
+                "preserving the offset for resuming iteration at a predictable index and order, please set a random seed using deeplake.random()"
+            )
+        from indra.pytorch.tensorinfo import TensorsInfo, LoaderMetaInfo  # type:ignore
+
+        info = TensorsInfo(
+            raw_tensors=raw_tensors or [],
+            htype_dict=htype_dict or {},
+            ndim_dict=ndim_dict or {},
+            tensor_info_dict=tensor_info_dict or {},
+            pil_compressed_tensors=pil_compressed_tensors or [],
+            json_tensors=json_tensors or [],
+            list_tensors=list_tensors or [],
+        )
+
+        loader_meta = LoaderMetaInfo(
+            context=self.multiprocessing_context,
+            distributed=self._distributed,
+            upcast=self._mode == "pytorch"
+            and self.__is_upcast_needed(
+                dataset, tensors
+            ),  # upcast to handle unsupported dtypes,
+            return_index=self._return_index,
+            verbose=self._verbose,
+            ignore_errors=self._ignore_errors,
+            prefetch_factor=self._prefetch_factor,
+            offset=self._offset,
+            primary_tensor=self._primary_tensor_name,
+            worker_init_fn=self.worker_init_fn,
+        )
+
+        return INDRA_LOADER(  # type: ignore [misc]
+            indra_dataset,
+            batch_size=self._batch_size,
+            num_threads=num_threads,
+            shuffle=self._shuffle,
+            num_workers=self._num_workers,
+            collate_fn=self.collate_fn,
+            transform_fn=self._transform,
+            tensors=tensors,
+            drop_last=self._drop_last,
+            buffer_size=self._buffer_size,
+            persistent_workers=self._persistent_workers,
+            loader_meta=loader_meta,
+            info=info,
+        )
+
+    def _fill_sample_info_tensors(
+        self,
+        dataset,
+        sample_info_tensors,
+        json_tensors,
+        list_tensors,
+    ):
+        for tensor_name in sample_info_tensors:
+            tensor = dataset._get_tensor_from_root(tensor_name)
+            if len(tensor) == 0:
+                raise EmptyTensorError(
+                    f" the dataset has an empty tensor {tensor_name}, pytorch dataloader can't be created."
+                    f" Please either populate the tensor or pass tensors argument to .pytorch that excludes this"
+                    f" tensor."
+                )
+            meta = tensor.meta
+            if meta.htype == "json":
+                json_tensors.append(tensor_name)
+            elif meta.htype in ["list", "tag"]:
+                list_tensors.append(tensor_name)
+
     def __iter__(self):
         if self._dataloader is None:
             dataset = self._orig_dataset
-            collate_fn = self.collate_fn
-            upcast = self._mode == "pytorch"  # upcast to handle unsupported dtypes
-
-            primary_tensor_name = self._primary_tensor_name
-            buffer_size = self._buffer_size
-
             tensors = self._tensors or map_tensor_keys(dataset, None)
 
             jpeg_png_compressed_tensors, json_tensors, list_tensors = check_tensors(
@@ -614,70 +861,78 @@ class DeepLakeDataLoader(DataLoader):
             sample_info_tensors, tensor_info_tensors = find_additional_tensors_and_info(
                 dataset, data_tensors
             )
+            self._fill_sample_info_tensors(
+                dataset, sample_info_tensors, json_tensors, list_tensors
+            )
             tensors.extend(sample_info_tensors)
             htype_dict, ndim_dict, tensor_info_dict = get_htype_ndim_tensor_info_dicts(
                 dataset, data_tensors, tensor_info_tensors
             )
             if deeplake.constants.RETURN_DUMMY_DATA_FOR_DATALOADER:
-                self._dataloader = DummyDataloader(
-                    deeplake_dataset=dataset,
-                    batch_size=self._batch_size,
-                    shuffle=self._shuffle,
-                    num_workers=self._num_workers,
-                    collate_fn=collate_fn,
-                    transform_fn=self._transform,
-                    distributed=self._distributed,
-                    prefetch_factor=self._prefetch_factor,
+                self._dataloader = self.__create_dummy_dataloader(
+                    dataset,
                     tensors=tensors,
-                    drop_last=self._drop_last,
-                    upcast=upcast,
-                    return_index=self._return_index,
                     raw_tensors=raw_tensors,
                     pil_compressed_tensors=pil_compressed_tensors,
-                    persistent_workers=self._persistent_workers,
                 )
             else:
                 if not hasattr(self, "_indra_dataset"):
                     indra_dataset = dataset_to_libdeeplake(dataset)
                 else:
                     indra_dataset = self._indra_dataset
-                self._dataloader = INDRA_LOADER(
+
+                self._dataloader = self.__get_indra_dataloader(
+                    dataset,
                     indra_dataset,
-                    batch_size=self._batch_size,
-                    num_threads=self._num_threads,
-                    shuffle=self._shuffle,
-                    num_workers=self._num_workers,
-                    collate_fn=collate_fn,
-                    transform_fn=self._transform,
-                    distributed=self._distributed,
-                    prefetch_factor=self._prefetch_factor,
                     tensors=tensors,
-                    drop_last=self._drop_last,
-                    ignore_errors=self._ignore_errors,
-                    upcast=upcast,
-                    return_index=self._return_index,
-                    primary_tensor=primary_tensor_name,
-                    buffer_size=buffer_size,
                     raw_tensors=raw_tensors,
                     pil_compressed_tensors=pil_compressed_tensors,
                     json_tensors=json_tensors,
                     list_tensors=list_tensors,
-                    persistent_workers=self._persistent_workers,
                     htype_dict=htype_dict,
                     ndim_dict=ndim_dict,
                     tensor_info_dict=tensor_info_dict,
                 )
+
         dataset_read(self._orig_dataset)
-        return iter(self._dataloader)
+
+        if self._iterator is not None:
+            self._iterator = iter(self._dataloader)
+
+        return self
+
+    def __setattr__(self, attr, val):
+        if (
+            attr == "_iterator"
+            and val is None
+            and hasattr(self, "_iterator")
+            and self._iterator is not None
+        ):
+            self._iterator.close()
+        else:
+            super().__setattr__(attr, val)
+
+    def __next__(self):
+        if self._dataloader is None:
+            self.__iter__()
+        if self._iterator is None:
+            self._iterator = iter(self._dataloader)
+        return next(self._iterator)
+
+    def __del__(self):
+        self.close()
 
 
-def dataloader(dataset, ignore_errors: bool = False) -> DeepLakeDataLoader:
+def dataloader(
+    dataset, ignore_errors: bool = False, verbose: bool = False
+) -> DeepLakeDataLoader:
     """Returns a :class:`~deeplake.enterprise.dataloader.DeepLakeDataLoader` object which can be transformed to either pytorch dataloader or numpy.
 
 
     Args:
         dataset: :class:`~deeplake.core.dataset.Dataset` object on which dataloader needs to be built
-        ignore_errors (bool): If ``True``, the data loader will ignore errors apperaing during dataloading otherwise it will collct the statistics and report appeard errors. Default value is ``False``
+        ignore_errors (bool): If ``True``, the data loader will ignore errors appeared during data iteration otherwise it will collect the statistics and report appeared errors. Default value is ``False``
+        verbose (bool): If ``True``, the data loader will dump verbose logs of it's steps. Default value is ``False``
 
 
     Returns:
@@ -691,7 +946,7 @@ def dataloader(dataset, ignore_errors: bool = False) -> DeepLakeDataLoader:
 
 
         >>> import deeplake
-        >>> from deeplake.enterprise import dataloader
+        >>> from deeplake.enterprise.dataloader import dataloader
         >>>
         >>> ds_train = deeplake.load('hub://activeloop/fashion-mnist-train')
         >>> train_loader = dataloader(ds_train).numpy()
@@ -727,7 +982,7 @@ def dataloader(dataset, ignore_errors: bool = False) -> DeepLakeDataLoader:
         ...     # custom logic on data
         ...     pass
 
-        Creating dataloader and chaning with query
+        Creating dataloader and chaining with query
 
         >>> ds = deeplake.load('hub://activeloop/coco-train')
         >>> dl = dataloader(ds_train)
@@ -740,7 +995,7 @@ def dataloader(dataset, ignore_errors: bool = False) -> DeepLakeDataLoader:
         ...     pass
     """
     verify_base_storage(dataset)
-    return DeepLakeDataLoader(dataset, _ignore_errors=ignore_errors)
+    return DeepLakeDataLoader(dataset, _ignore_errors=ignore_errors, _verbose=verbose)
 
 
 def validate_tensors(tensors, dataset, all_vars):
