@@ -2,6 +2,7 @@ import pickle
 import deeplake
 import numpy as np
 import pytest
+from functools import partial
 from deeplake.util.exceptions import EmptyTensorError, TensorDoesNotExistError
 
 from deeplake.util.remove_cache import get_base_storage
@@ -9,6 +10,7 @@ from deeplake.core.index.index import IndexEntry
 from deeplake.tests.common import (
     requires_torch,
     requires_libdeeplake,
+    disabale_hidden_tensors_config,
     convert_data_according_to_torch_version,
 )
 from deeplake.core.dataset import Dataset
@@ -62,19 +64,69 @@ def index_transform(sample):
     return sample["index"], sample["xyz"]
 
 
+def dummy_init_fn(arg):
+    return f"function called with arg {arg}"
+
+
+@requires_libdeeplake
+def test_setting_woker_init_function(local_auth_ds):
+    dl = local_auth_ds.dataloader().pytorch()
+
+    assert dl.worker_init_fn == None
+    dl.worker_init_fn = partial(dummy_init_fn, 1024)
+    assert dl.worker_init_fn() == "function called with arg 1024"
+
+
+@requires_torch
+@requires_libdeeplake
+def test_offset_ds_iteration(local_auth_ds):
+    with local_auth_ds as ds:
+        ds.create_tensor(
+            "abc", htype="generic", dtype="uint16", **disabale_hidden_tensors_config
+        )
+        ds.abc.extend([i for i in range(10)])
+
+    dl = (
+        local_auth_ds.dataloader()
+        .offset(4)
+        .transform(identity)
+        .pytorch(collate_fn=identity)
+    )
+
+    idx_table = [4, 5, 6, 7, 8, 9, 0, 1, 2, 3]
+    for i, item in enumerate(dl):
+        assert idx_table[i] == item[0]["index"].astype(int)
+
+
 @requires_torch
 @requires_libdeeplake
 @pytest.mark.parametrize(
     "ds",
-    ["hub_cloud_ds", "local_auth_ds"],
+    [
+        pytest.param(
+            "hub_cloud_ds",
+            marks=[pytest.mark.slow, pytest.mark.skip("Causing lockups")],
+        ),
+        "local_auth_ds",
+    ],
     indirect=True,
 )
+@pytest.mark.flaky
+@pytest.mark.skip("causing lockups")
 def test_pytorch_small(ds):
     with ds:
-        ds.create_tensor("image", max_chunk_size=PYTORCH_TESTS_MAX_CHUNK_SIZE)
+        ds.create_tensor(
+            "image",
+            max_chunk_size=PYTORCH_TESTS_MAX_CHUNK_SIZE,
+            **disabale_hidden_tensors_config,
+        )
         ds.image.extend(([i * np.ones((i + 1, i + 1)) for i in range(16)]))
         ds.commit()
-        ds.create_tensor("image2", max_chunk_size=PYTORCH_TESTS_MAX_CHUNK_SIZE)
+        ds.create_tensor(
+            "image2",
+            max_chunk_size=PYTORCH_TESTS_MAX_CHUNK_SIZE,
+            **disabale_hidden_tensors_config,
+        )
         ds.image2.extend(np.array([i * np.ones((12, 12)) for i in range(16)]))
     dl = ds.dataloader().batch(1).pytorch(num_workers=2)
 
@@ -127,18 +179,28 @@ def test_pytorch_small(ds):
 
 @requires_torch
 @requires_libdeeplake
-def test_pytorch_transform(hub_cloud_ds):
-    with hub_cloud_ds:
-        hub_cloud_ds.create_tensor("image", max_chunk_size=PYTORCH_TESTS_MAX_CHUNK_SIZE)
-        hub_cloud_ds.image.extend(([i * np.ones((i + 1, i + 1)) for i in range(16)]))
-        hub_cloud_ds.checkout("alt", create=True)
-        hub_cloud_ds.create_tensor(
-            "image2", max_chunk_size=PYTORCH_TESTS_MAX_CHUNK_SIZE
+@pytest.mark.flaky(retry_count=3)
+@pytest.mark.slow
+@pytest.mark.timeout(10)
+@pytest.mark.skip("causing lockups")
+def test_pytorch_transform(local_auth_ds):
+    with local_auth_ds as ds:
+        ds.create_tensor(
+            "image",
+            max_chunk_size=PYTORCH_TESTS_MAX_CHUNK_SIZE,
+            **disabale_hidden_tensors_config,
         )
-        hub_cloud_ds.image2.extend(np.array([i * np.ones((12, 12)) for i in range(16)]))
+        ds.image.extend(([i * np.ones((i + 1, i + 1)) for i in range(16)]))
+        ds.checkout("alt", create=True)
+        ds.create_tensor(
+            "image2",
+            max_chunk_size=PYTORCH_TESTS_MAX_CHUNK_SIZE,
+            **disabale_hidden_tensors_config,
+        )
+        ds.image2.extend(np.array([i * np.ones((12, 12)) for i in range(16)]))
 
     dl = (
-        hub_cloud_ds.dataloader()
+        ds.dataloader()
         .batch(1)
         .transform(to_tuple, t1="image", t2="image2")
         .pytorch(num_workers=2, collate_fn=identity_collate)
@@ -153,24 +215,46 @@ def test_pytorch_transform(hub_cloud_ds):
             np.testing.assert_array_equal(actual_image2, expected_image2)
 
 
+@requires_libdeeplake
+@pytest.mark.flaky
+def test_inequal_tensors_dataloader_length(local_auth_ds):
+    with local_auth_ds as ds:
+        ds.create_tensor("images", **disabale_hidden_tensors_config)
+        ds.create_tensor("label", **disabale_hidden_tensors_config)
+        ds.images.extend(([i * np.ones((i + 1, i + 1)) for i in range(16)]))
+
+    ld = local_auth_ds.dataloader().batch(1).pytorch()
+    assert len(ld) == 0
+    ld1 = local_auth_ds.dataloader().batch(2).pytorch(tensors=["images"])
+    assert len(ld1) == 8
+
+
 @requires_torch
 @requires_libdeeplake
-def test_pytorch_transform_dict(hub_cloud_ds):
-    with hub_cloud_ds:
-        hub_cloud_ds.create_tensor("image", max_chunk_size=PYTORCH_TESTS_MAX_CHUNK_SIZE)
-        hub_cloud_ds.image.extend(([i * np.ones((i + 1, i + 1)) for i in range(16)]))
-        hub_cloud_ds.create_tensor(
-            "image2", max_chunk_size=PYTORCH_TESTS_MAX_CHUNK_SIZE
+@pytest.mark.flaky
+@pytest.mark.slow
+def test_pytorch_transform_dict(local_auth_ds):
+    with local_auth_ds as ds:
+        ds.create_tensor(
+            "image",
+            max_chunk_size=PYTORCH_TESTS_MAX_CHUNK_SIZE,
+            **disabale_hidden_tensors_config,
         )
-        hub_cloud_ds.image2.extend(np.array([i * np.ones((12, 12)) for i in range(16)]))
-        hub_cloud_ds.create_tensor(
-            "image3", max_chunk_size=PYTORCH_TESTS_MAX_CHUNK_SIZE
+        ds.image.extend(([i * np.ones((i + 1, i + 1)) for i in range(16)]))
+        ds.create_tensor(
+            "image2",
+            max_chunk_size=PYTORCH_TESTS_MAX_CHUNK_SIZE,
+            **disabale_hidden_tensors_config,
         )
-        hub_cloud_ds.image3.extend(np.array([i * np.ones((12, 12)) for i in range(16)]))
+        ds.image2.extend(np.array([i * np.ones((12, 12)) for i in range(16)]))
+        ds.create_tensor(
+            "image3",
+            max_chunk_size=PYTORCH_TESTS_MAX_CHUNK_SIZE,
+            **disabale_hidden_tensors_config,
+        )
+        ds.image3.extend(np.array([i * np.ones((12, 12)) for i in range(16)]))
 
-    dl = (
-        hub_cloud_ds.dataloader().transform({"image": double, "image2": None}).pytorch()
-    )
+    dl = ds.dataloader().transform({"image": double, "image2": None}).pytorch()
 
     assert len(dl.dataset) == 16
 
@@ -192,19 +276,25 @@ def test_pytorch_transform_dict(hub_cloud_ds):
             np.testing.assert_array_equal(image2.numpy(), i * np.ones((1, 12, 12)))
 
 
+@pytest.mark.slow
 @requires_torch
 @requires_libdeeplake
-def test_pytorch_with_compression(hub_cloud_ds: Dataset):
+@pytest.mark.flaky
+def test_pytorch_with_compression(local_auth_ds: Dataset):
     # TODO: chunk-wise compression for labels (right now they are uncompressed)
-    with hub_cloud_ds:
-        images = hub_cloud_ds.create_tensor(
+    with local_auth_ds as ds:
+        images = ds.create_tensor(
             "images",
             htype="image",
             sample_compression="png",
             max_chunk_size=PYTORCH_TESTS_MAX_CHUNK_SIZE,
+            **disabale_hidden_tensors_config,
         )
-        labels = hub_cloud_ds.create_tensor(
-            "labels", htype="class_label", max_chunk_size=PYTORCH_TESTS_MAX_CHUNK_SIZE
+        labels = ds.create_tensor(
+            "labels",
+            htype="class_label",
+            max_chunk_size=PYTORCH_TESTS_MAX_CHUNK_SIZE,
+            **disabale_hidden_tensors_config,
         )
 
         assert images.meta.sample_compression == "png"
@@ -212,7 +302,7 @@ def test_pytorch_with_compression(hub_cloud_ds: Dataset):
         images.extend(np.ones((16, 12, 12, 3), dtype="uint8"))
         labels.extend(np.ones((16, 1), dtype="uint32"))
 
-    dl = hub_cloud_ds.dataloader().pytorch(num_workers=0)
+    dl = ds.dataloader().pytorch(num_workers=0)
 
     for _ in range(2):
         for batch in dl:
@@ -222,19 +312,25 @@ def test_pytorch_with_compression(hub_cloud_ds: Dataset):
             assert T.shape == (1, 1)
 
 
+@pytest.mark.slow
 @requires_torch
 @requires_libdeeplake
-def test_custom_tensor_order(hub_cloud_ds):
-    with hub_cloud_ds:
+@pytest.mark.flaky
+def test_custom_tensor_order(local_auth_ds):
+    with local_auth_ds as ds:
         tensors = ["a", "b", "c", "d"]
         for t in tensors:
-            hub_cloud_ds.create_tensor(t, max_chunk_size=PYTORCH_TESTS_MAX_CHUNK_SIZE)
-            hub_cloud_ds[t].extend(np.random.random((3, 4, 5)))
+            ds.create_tensor(
+                t,
+                max_chunk_size=PYTORCH_TESTS_MAX_CHUNK_SIZE,
+                **disabale_hidden_tensors_config,
+            )
+            ds[t].extend(np.random.random((3, 4, 5)))
 
     with pytest.raises(TensorDoesNotExistError):
-        dl = hub_cloud_ds.dataloader().pytorch(tensors=["c", "d", "e"])
+        ds.dataloader().pytorch(tensors=["c", "d", "e"])
 
-    dl = hub_cloud_ds.dataloader().pytorch(tensors=["c", "d", "a"], return_index=False)
+    dl = ds.dataloader().pytorch(tensors=["c", "d", "a"], return_index=False)
 
     for i, batch in enumerate(dl):
         c1, d1, a1 = batch
@@ -245,9 +341,9 @@ def test_custom_tensor_order(hub_cloud_ds):
         np.testing.assert_array_equal(a1, a2)
         np.testing.assert_array_equal(c1, c2)
         np.testing.assert_array_equal(d1, d2)
-        np.testing.assert_array_equal(a1[0], hub_cloud_ds.a.numpy()[i])
-        np.testing.assert_array_equal(c1[0], hub_cloud_ds.c.numpy()[i])
-        np.testing.assert_array_equal(d1[0], hub_cloud_ds.d.numpy()[i])
+        np.testing.assert_array_equal(a1[0], ds.a.numpy()[i])
+        np.testing.assert_array_equal(c1[0], ds.c.numpy()[i])
+        np.testing.assert_array_equal(d1[0], ds.d.numpy()[i])
         batch = pickle.loads(pickle.dumps(batch))
         c1, d1, a1 = batch
         a2 = batch["a"]
@@ -256,30 +352,38 @@ def test_custom_tensor_order(hub_cloud_ds):
         np.testing.assert_array_equal(a1, a2)
         np.testing.assert_array_equal(c1, c2)
         np.testing.assert_array_equal(d1, d2)
-        np.testing.assert_array_equal(a1[0], hub_cloud_ds.a.numpy()[i])
-        np.testing.assert_array_equal(c1[0], hub_cloud_ds.c.numpy()[i])
-        np.testing.assert_array_equal(d1[0], hub_cloud_ds.d.numpy()[i])
+        np.testing.assert_array_equal(a1[0], ds.a.numpy()[i])
+        np.testing.assert_array_equal(c1[0], ds.c.numpy()[i])
+        np.testing.assert_array_equal(d1[0], ds.d.numpy()[i])
 
 
+@pytest.mark.slow
 @requires_torch
 @requires_libdeeplake
-def test_readonly_with_two_workers(hub_cloud_ds):
-    with hub_cloud_ds:
-        hub_cloud_ds.create_tensor(
-            "images", max_chunk_size=PYTORCH_TESTS_MAX_CHUNK_SIZE
+@pytest.mark.flaky
+@pytest.mark.skip("causing lockups")
+@pytest.mark.timeout(10)
+def test_readonly_with_two_workers(local_auth_ds):
+    with local_auth_ds as ds:
+        ds.create_tensor(
+            "images",
+            max_chunk_size=PYTORCH_TESTS_MAX_CHUNK_SIZE,
+            **disabale_hidden_tensors_config,
         )
-        hub_cloud_ds.create_tensor(
-            "labels", max_chunk_size=PYTORCH_TESTS_MAX_CHUNK_SIZE
+        ds.create_tensor(
+            "labels",
+            max_chunk_size=PYTORCH_TESTS_MAX_CHUNK_SIZE,
+            **disabale_hidden_tensors_config,
         )
-        hub_cloud_ds.images.extend(np.ones((10, 12, 12)))
-        hub_cloud_ds.labels.extend(np.ones(10))
+        ds.images.extend(np.ones((10, 12, 12)))
+        ds.labels.extend(np.ones(10))
 
-    base_storage = get_base_storage(hub_cloud_ds.storage)
+    base_storage = get_base_storage(ds.storage)
     base_storage.flush()
     base_storage.enable_readonly()
     ds = Dataset(
-        storage=hub_cloud_ds.storage,
-        token=hub_cloud_ds.token,
+        storage=ds.storage,
+        token=ds.token,
         read_only=True,
         verbose=False,
     )
@@ -287,36 +391,37 @@ def test_readonly_with_two_workers(hub_cloud_ds):
     ptds = ds.dataloader().pytorch(num_workers=2)
     # no need to check input, only care that readonly works
     for _ in ptds:
-        pass
-
-
-@pytest.mark.xfail(raises=NotImplementedError, strict=True)
-def test_corrupt_dataset():
-    raise NotImplementedError
-
-
-@pytest.mark.xfail(raises=NotImplementedError, strict=True)
-def test_pytorch_local_cache():
-    raise NotImplementedError
+        continue
 
 
 @requires_torch
 @requires_libdeeplake
-def test_groups(hub_cloud_ds, compressed_image_paths):
+@pytest.mark.slow
+@pytest.mark.flaky
+def test_groups(local_auth_ds, compressed_image_paths):
     img1 = deeplake.read(compressed_image_paths["jpeg"][0])
     img2 = deeplake.read(compressed_image_paths["png"][0])
-    with hub_cloud_ds:
-        hub_cloud_ds.create_tensor(
-            "images/jpegs/cats", htype="image", sample_compression="jpeg"
+    with local_auth_ds as ds:
+        ds.create_tensor(
+            "images/jpegs/cats",
+            htype="image",
+            sample_compression="jpeg",
+            **disabale_hidden_tensors_config,
         )
-        hub_cloud_ds.create_tensor(
-            "images/pngs/flowers", htype="image", sample_compression="png"
+        ds.create_tensor(
+            "images/pngs/flowers",
+            htype="image",
+            sample_compression="png",
+            **disabale_hidden_tensors_config,
         )
-        for _ in range(10):
-            hub_cloud_ds.images.jpegs.cats.append(img1)
-            hub_cloud_ds.images.pngs.flowers.append(img2)
 
-    another_ds = deeplake.dataset(hub_cloud_ds.path, token=hub_cloud_ds.token)
+        ds.images.jpegs.cats.extend(10 * [img1])
+        ds.images.pngs.flowers.extend(10 * [img2])
+
+    another_ds = deeplake.dataset(
+        ds.path,
+        token=ds.token,
+    )
     dl = another_ds.dataloader().pytorch(return_index=False)
     for i, (cat, flower) in enumerate(dl):
         assert cat[0].shape == another_ds.images.jpegs.cats[i].numpy().shape
@@ -330,18 +435,49 @@ def test_groups(hub_cloud_ds, compressed_image_paths):
         np.testing.assert_array_equal(flower[0], img2.array)
 
 
+@pytest.mark.slow
 @requires_torch
 @requires_libdeeplake
-def test_string_tensors(hub_cloud_ds):
-    with hub_cloud_ds:
-        hub_cloud_ds.create_tensor("strings", htype="text")
-        hub_cloud_ds.strings.extend([f"string{idx}" for idx in range(5)])
+@pytest.mark.flaky
+def test_string_tensors(local_auth_ds):
+    with local_auth_ds as ds:
+        ds.create_tensor("strings", htype="text", **disabale_hidden_tensors_config)
+        ds.strings.extend([f"string{idx}" for idx in range(5)])
 
-    ptds = hub_cloud_ds.dataloader().pytorch()
+    ptds = ds.dataloader().pytorch()
     for idx, batch in enumerate(ptds):
         np.testing.assert_array_equal(batch["strings"], f"string{idx}")
 
 
+@pytest.mark.slow
+@requires_torch
+@requires_libdeeplake
+@pytest.mark.flaky
+def test_tag_tensors(local_auth_ds):
+    with local_auth_ds as ds:
+        ds.create_tensor("tags", htype="tag", **disabale_hidden_tensors_config)
+        ds.tags.extend(
+            [
+                f"tag{idx}" if idx % 2 == 0 else [f"tag{idx}", f"tag{idx}"]
+                for idx in range(5)
+            ]
+        )
+
+    ptds = ds.pytorch(batch_size=1)
+    for idx, batch in enumerate(ptds):
+        if idx % 2 == 0:
+            np.testing.assert_array_equal(batch["tags"], [[f"tag{idx}"]])
+        else:
+            np.testing.assert_array_equal(batch["tags"], [[f"tag{idx}", f"tag{idx}"]])
+
+    ptds2 = ds.pytorch(batch_size=None)
+    for idx, batch in enumerate(ptds2):
+        if idx % 2 == 0:
+            np.testing.assert_array_equal(batch["tags"], [f"tag{idx}"])
+        else:
+            np.testing.assert_array_equal(batch["tags"], [f"tag{idx}", f"tag{idx}"])
+
+
 @pytest.mark.xfail(raises=NotImplementedError, strict=True)
 def test_pytorch_large():
     raise NotImplementedError
@@ -363,21 +499,23 @@ def test_pytorch_large():
         [4, 3, 2, 1],
     ],
 )
-def test_pytorch_view(hub_cloud_ds, index):
+@pytest.mark.slow
+@pytest.mark.flaky
+def test_pytorch_view(local_auth_ds, index):
     arr_list_1 = [np.random.randn(15, 15, i) for i in range(10)]
     arr_list_2 = [np.random.randn(40, 15, 4, i) for i in range(10)]
     label_list = list(range(10))
 
-    with hub_cloud_ds as ds:
-        ds.create_tensor("img1")
-        ds.create_tensor("img2")
-        ds.create_tensor("label")
+    with local_auth_ds as ds:
+        ds.create_tensor("img1", **disabale_hidden_tensors_config)
+        ds.create_tensor("img2", **disabale_hidden_tensors_config)
+        ds.create_tensor("label", **disabale_hidden_tensors_config)
         ds.img1.extend(arr_list_1)
         ds.img2.extend(arr_list_2)
         ds.label.extend(label_list)
 
-    ptds = hub_cloud_ds[index].dataloader().pytorch()
-    idxs = list(IndexEntry(index).indices(len(hub_cloud_ds)))
+    ptds = ds[index].dataloader().pytorch()
+    idxs = list(IndexEntry(index).indices(len(ds)))
     for idx, batch in enumerate(ptds):
         idx = idxs[idx]
         np.testing.assert_array_equal(batch["img1"][0], arr_list_1[idx])
@@ -388,17 +526,18 @@ def test_pytorch_view(hub_cloud_ds, index):
 @requires_torch
 @requires_libdeeplake
 @pytest.mark.parametrize("shuffle", [True, False])
-def test_pytorch_collate(hub_cloud_ds, shuffle):
-    with hub_cloud_ds:
-        hub_cloud_ds.create_tensor("a")
-        hub_cloud_ds.create_tensor("b")
-        hub_cloud_ds.create_tensor("c")
-        for _ in range(100):
-            hub_cloud_ds.a.append(0)
-            hub_cloud_ds.b.append(1)
-            hub_cloud_ds.c.append(2)
+@pytest.mark.slow
+@pytest.mark.flaky
+def test_pytorch_collate(local_auth_ds, shuffle):
+    with local_auth_ds as ds:
+        ds.create_tensor("a", **disabale_hidden_tensors_config)
+        ds.create_tensor("b", **disabale_hidden_tensors_config)
+        ds.create_tensor("c", **disabale_hidden_tensors_config)
+        ds.a.extend(100 * [0])
+        ds.b.extend(100 * [1])
+        ds.c.extend(100 * [2])
 
-    ptds = hub_cloud_ds.dataloader().batch(4).pytorch(collate_fn=reorder_collate)
+    ptds = ds.dataloader().batch(4).pytorch(collate_fn=reorder_collate)
     if shuffle:
         ptds = ptds.shuffle()
     for batch in ptds:
@@ -412,18 +551,19 @@ def test_pytorch_collate(hub_cloud_ds, shuffle):
 @requires_torch
 @requires_libdeeplake
 @pytest.mark.parametrize("shuffle", [True, False])
-def test_pytorch_transform_collate(hub_cloud_ds, shuffle):
-    with hub_cloud_ds:
-        hub_cloud_ds.create_tensor("a")
-        hub_cloud_ds.create_tensor("b")
-        hub_cloud_ds.create_tensor("c")
-        for _ in range(100):
-            hub_cloud_ds.a.append(0 * np.ones((300, 300)))
-            hub_cloud_ds.b.append(1 * np.ones((300, 300)))
-            hub_cloud_ds.c.append(2 * np.ones((300, 300)))
+@pytest.mark.slow
+@pytest.mark.flaky
+def test_pytorch_transform_collate(local_auth_ds, shuffle):
+    with local_auth_ds as ds:
+        ds.create_tensor("a", **disabale_hidden_tensors_config)
+        ds.create_tensor("b", **disabale_hidden_tensors_config)
+        ds.create_tensor("c", **disabale_hidden_tensors_config)
+        ds.a.extend(100 * [0 * np.ones((300, 300))])
+        ds.b.extend(100 * [1 * np.ones((300, 300))])
+        ds.c.extend(100 * [2 * np.ones((300, 300))])
 
     ptds = (
-        hub_cloud_ds.dataloader()
+        ds.dataloader()
         .batch(4)
         .pytorch(
             collate_fn=my_transform_collate,
@@ -441,25 +581,20 @@ def test_pytorch_transform_collate(hub_cloud_ds, shuffle):
         np.testing.assert_array_equal(batch[2], 1 * np.ones((4, 300, 300)))
 
 
-@pytest.mark.xfail(raises=NotImplementedError, strict=True)
-def test_pytorch_ddp():
-    raise NotImplementedError
-
-
 @requires_torch
 @requires_libdeeplake
 @pytest.mark.parametrize("compression", [None, "jpeg"])
-def test_pytorch_decode(hub_cloud_ds, compressed_image_paths, compression):
-    with hub_cloud_ds:
-        hub_cloud_ds.create_tensor("image", sample_compression=compression)
-        hub_cloud_ds.image.extend(
+@pytest.mark.slow
+@pytest.mark.flaky
+def test_pytorch_decode(local_auth_ds, compressed_image_paths, compression):
+    with local_auth_ds as ds:
+        ds.create_tensor("image", sample_compression=compression)
+        ds.image.extend(
             np.array([i * np.ones((10, 10, 3), dtype=np.uint8) for i in range(5)])
         )
-        hub_cloud_ds.image.extend(
-            [deeplake.read(compressed_image_paths["jpeg"][0])] * 5
-        )
+        ds.image.extend([deeplake.read(compressed_image_paths["jpeg"][0])] * 5)
 
-    ptds = hub_cloud_ds.dataloader().pytorch(decode_method={"image": "tobytes"})
+    ptds = ds.dataloader().pytorch(decode_method={"image": "tobytes"})
 
     for i, batch in enumerate(ptds):
         image = convert_data_according_to_torch_version(batch["image"])
@@ -474,7 +609,7 @@ def test_pytorch_decode(hub_cloud_ds, compressed_image_paths, compression):
                 assert f.read() == image
 
     if compression:
-        ptds = hub_cloud_ds.dataloader().numpy(decode_method={"image": "pil"})
+        ptds = ds.dataloader().numpy(decode_method={"image": "pil"})
         for i, batch in enumerate(ptds):
             image = batch[0]["image"]
             assert isinstance(image, Image.Image)
@@ -489,40 +624,45 @@ def test_pytorch_decode(hub_cloud_ds, compressed_image_paths, compression):
 
 @requires_torch
 @requires_libdeeplake
-def test_rename(hub_cloud_ds):
-    with hub_cloud_ds as ds:
-        ds.create_tensor("abc")
-        ds.create_tensor("blue/green")
+@pytest.mark.flaky
+@pytest.mark.slow
+def test_rename(local_auth_ds):
+    group_name = "red/green"
+    with local_auth_ds as ds:
+        ds.create_tensor("abc", **disabale_hidden_tensors_config)
+        ds.create_tensor("blue/green", **disabale_hidden_tensors_config)
         ds.abc.append([1, 2, 3])
         ds.rename_tensor("abc", "xyz")
         ds.rename_group("blue", "red")
-        ds["red/green"].append([1, 2, 3, 4])
+        ds[group_name].append([1, 2, 3, 4])
     loader = ds.dataloader().pytorch(return_index=False)
     for sample in loader:
-        assert set(sample.keys()) == {"xyz", "red/green"}
+        assert set(sample.keys()) == {"xyz", group_name}
         np.testing.assert_array_equal(np.array(sample["xyz"]), np.array([[1, 2, 3]]))
         np.testing.assert_array_equal(
-            np.array(sample["red/green"]), np.array([[1, 2, 3, 4]])
+            np.array(sample[group_name]), np.array([[1, 2, 3, 4]])
         )
 
 
 @requires_torch
 @requires_libdeeplake
-@pytest.mark.parametrize("num_workers", [0, 2])
-def test_indexes(hub_cloud_ds, num_workers):
-    shuffle = False
-    with hub_cloud_ds as ds:
-        ds.create_tensor("xyz")
+@pytest.mark.parametrize(
+    "num_workers",
+    [
+        0,
+        pytest.param(2, marks=pytest.mark.skip(reason="causing lockups")),
+    ],
+)
+@pytest.mark.slow
+@pytest.mark.flaky
+def test_indexes(local_auth_ds, num_workers):
+    with local_auth_ds as ds:
+        ds.create_tensor("xyz", **disabale_hidden_tensors_config)
         for i in range(8):
             ds.xyz.append(i * np.ones((2, 2)))
 
-    ptds = (
-        hub_cloud_ds.dataloader()
-        .batch(4)
-        .pytorch(num_workers=num_workers, return_index=True)
-    )
-    if shuffle:
-        ptds = ptds.shuffle()
+    ptds = ds.dataloader().batch(4).pytorch(num_workers=num_workers, return_index=True)
+    ptds = ptds.shuffle()
 
     for batch in ptds:
         assert batch.keys() == {"xyz", "index"}
@@ -532,24 +672,29 @@ def test_indexes(hub_cloud_ds, num_workers):
 
 @requires_torch
 @requires_libdeeplake
-@pytest.mark.parametrize("num_workers", [0, 2])
-def test_indexes_transform(hub_cloud_ds, num_workers):
-    shuffle = False
-    with hub_cloud_ds as ds:
-        ds.create_tensor("xyz")
+@pytest.mark.slow
+@pytest.mark.parametrize(
+    "num_workers",
+    [
+        0,
+        pytest.param(2, marks=pytest.mark.skip("causing lockups")),
+    ],
+)
+@pytest.mark.flaky
+def test_indexes_transform(local_auth_ds, num_workers):
+    with local_auth_ds as ds:
+        ds.create_tensor("xyz", **disabale_hidden_tensors_config)
         for i in range(8):
             ds.xyz.append(i * np.ones((2, 2)))
 
     ptds = (
-        hub_cloud_ds.dataloader()
+        ds.dataloader()
         .batch(4)
         .transform(index_transform)
         .pytorch(
             num_workers=num_workers, return_index=True, collate_fn=identity_collate
         )
     )
-    if shuffle:
-        ptds = ptds.shuffle()
 
     for batch in ptds:
         assert len(batch) == 4
@@ -559,22 +704,23 @@ def test_indexes_transform(hub_cloud_ds, num_workers):
 
 @requires_torch
 @requires_libdeeplake
-@pytest.mark.parametrize("num_workers", [0, 2])
-def test_indexes_transform_dict(hub_cloud_ds, num_workers):
-    shuffle = False
-    with hub_cloud_ds as ds:
-        ds.create_tensor("xyz")
+@pytest.mark.parametrize(
+    "num_workers", [0, pytest.param(2, marks=pytest.mark.skip("causing lockups"))]
+)
+@pytest.mark.slow
+@pytest.mark.flaky
+def test_indexes_transform_dict(local_auth_ds, num_workers):
+    with local_auth_ds as ds:
+        ds.create_tensor("xyz", **disabale_hidden_tensors_config)
         for i in range(8):
             ds.xyz.append(i * np.ones((2, 2)))
 
     ptds = (
-        hub_cloud_ds.dataloader()
+        ds.dataloader()
         .batch(4)
         .transform({"xyz": double, "index": None})
         .pytorch(num_workers=num_workers, return_index=True)
     )
-    if shuffle:
-        ptds = ptds.shuffle()
 
     for batch in ptds:
         assert batch.keys() == {"xyz", "index"}
@@ -582,13 +728,11 @@ def test_indexes_transform_dict(hub_cloud_ds, num_workers):
             np.testing.assert_array_equal(2 * batch["index"][i], batch["xyz"][i][0, 0])
 
     ptds = (
-        hub_cloud_ds.dataloader()
+        ds.dataloader()
         .batch(4)
         .transform({"xyz": double})
         .pytorch(num_workers=num_workers, return_index=True)
     )
-    if shuffle:
-        ptds = ptds.shuffle()
 
     for batch in ptds:
         assert batch.keys() == {"xyz"}
@@ -596,17 +740,20 @@ def test_indexes_transform_dict(hub_cloud_ds, num_workers):
 
 @requires_torch
 @requires_libdeeplake
-@pytest.mark.parametrize("num_workers", [0, 2])
-def test_indexes_tensors(hub_cloud_ds, num_workers):
-    shuffle = False
-    with hub_cloud_ds as ds:
-        ds.create_tensor("xyz")
+@pytest.mark.parametrize(
+    "num_workers", [0, pytest.param(2, marks=pytest.mark.skip("causing lockups"))]
+)
+@pytest.mark.slow
+@pytest.mark.flaky
+def test_indexes_tensors(local_auth_ds, num_workers):
+    with local_auth_ds as ds:
+        ds.create_tensor("xyz", **disabale_hidden_tensors_config)
         for i in range(8):
             ds.xyz.append(i * np.ones((2, 2)))
 
     with pytest.raises(ValueError):
-        ptds = (
-            hub_cloud_ds.dataloader()
+        (
+            ds.dataloader()
             .batch(4)
             .pytorch(
                 num_workers=num_workers, return_index=True, tensors=["xyz", "index"]
@@ -614,12 +761,10 @@ def test_indexes_tensors(hub_cloud_ds, num_workers):
         )
 
     ptds = (
-        hub_cloud_ds.dataloader()
+        ds.dataloader()
         .batch(4)
         .pytorch(num_workers=num_workers, return_index=True, tensors=["xyz"])
     )
-    if shuffle:
-        ptds = ptds.shuffle()
 
     for batch in ptds:
         assert batch.keys() == {"xyz", "index"}
@@ -627,10 +772,12 @@ def test_indexes_tensors(hub_cloud_ds, num_workers):
 
 @requires_libdeeplake
 @requires_torch
-def test_uneven_iteration(hub_cloud_ds):
-    with hub_cloud_ds as ds:
-        ds.create_tensor("x")
-        ds.create_tensor("y")
+@pytest.mark.flaky
+@pytest.mark.slow
+def test_uneven_iteration(local_auth_ds):
+    with local_auth_ds as ds:
+        ds.create_tensor("x", **disabale_hidden_tensors_config)
+        ds.create_tensor("y", **disabale_hidden_tensors_config)
         ds.x.extend(list(range(5)))
         ds.y.extend(list(range(10)))
     ptds = ds.dataloader().pytorch()
@@ -642,31 +789,45 @@ def test_uneven_iteration(hub_cloud_ds):
 
 @requires_libdeeplake
 @requires_torch
-def test_pytorch_error_handling(hub_cloud_ds):
-    with hub_cloud_ds as ds:
-        ds.create_tensor("x")
-        ds.create_tensor("y")
+@pytest.mark.slow
+def test_pytorch_error_handling(local_auth_ds):
+    with local_auth_ds as ds:
+        ds.create_tensor("x", **disabale_hidden_tensors_config)
+        ds.create_tensor("y", **disabale_hidden_tensors_config)
         ds.x.extend(list(range(5)))
 
     ptds = ds.dataloader().pytorch()
     with pytest.raises(EmptyTensorError):
         for _ in ptds:
-            pass
+            continue
 
     ptds = ds.dataloader().pytorch(tensors=["x", "y"])
     with pytest.raises(EmptyTensorError):
         for _ in ptds:
-            pass
+            continue
 
     ptds = ds.dataloader().pytorch(tensors=["x"])
     for _ in ptds:
-        pass
+        continue
 
 
 @requires_libdeeplake
 @requires_torch
-def test_pil_decode_method(hub_cloud_ds):
-    with hub_cloud_ds as ds:
+def test_batch_sampler_attribute(local_auth_ds):
+    ld = local_auth_ds.dataloader().pytorch()
+
+    from torch.utils.data import BatchSampler
+
+    assert isinstance(ld.batch_sampler, BatchSampler)
+    assert ld.batch_sampler.sampler is not None
+
+
+@requires_libdeeplake
+@requires_torch
+@pytest.mark.slow
+@pytest.mark.flaky
+def test_pil_decode_method(local_auth_ds):
+    with local_auth_ds as ds:
         ds.create_tensor("x", htype="image", sample_compression="jpeg")
         ds.x.extend(np.random.randint(0, 255, (10, 10, 10, 3), np.uint8))
 
@@ -676,11 +837,6 @@ def test_pil_decode_method(hub_cloud_ds):
         assert "x" in batch.keys()
         assert batch["x"].shape == (1, 10, 10, 3)
 
-    ptds = ds.dataloader().pytorch(decode_method={"x": "pil"})
-    with pytest.raises(TypeError):
-        for _ in ptds:
-            pass
-
     def custom_transform(batch):
         batch["x"] = np.array(batch["x"])
         return batch
@@ -699,6 +855,7 @@ def test_pil_decode_method(hub_cloud_ds):
 @patch("deeplake.constants.RETURN_DUMMY_DATA_FOR_DATALOADER", True)
 @requires_torch
 @requires_libdeeplake
+@pytest.mark.flaky
 def test_pytorch_dummy_data(local_auth_ds):
     x_data = [
         np.random.randint(0, 255, (100, 100, 3), dtype="uint8"),
@@ -732,13 +889,15 @@ def test_pytorch_dummy_data(local_auth_ds):
 
 @requires_libdeeplake
 @requires_torch
-def test_json_data_loader(hub_cloud_ds):
-    ds = hub_cloud_ds
-    with ds:
+@pytest.mark.flaky
+@pytest.mark.slow
+def test_json_data_loader(local_auth_ds):
+    with local_auth_ds as ds:
         ds.create_tensor(
             "json",
             htype="json",
             sample_compression=None,
+            **disabale_hidden_tensors_config,
         )
         d = {"x": 1, "y": 2, "z": 3}
         for _ in range(10):
@@ -756,13 +915,15 @@ def test_json_data_loader(hub_cloud_ds):
 
 @requires_libdeeplake
 @requires_torch
-def test_list_data_loader(hub_cloud_ds):
-    ds = hub_cloud_ds
-    with ds:
+@pytest.mark.flaky
+@pytest.mark.slow
+def test_list_data_loader(local_auth_ds):
+    with local_auth_ds as ds:
         ds.create_tensor(
             "list",
             htype="list",
             sample_compression=None,
+            **disabale_hidden_tensors_config,
         )
         l = [1, 2, 3]
         for _ in range(10):
@@ -779,8 +940,10 @@ def test_list_data_loader(hub_cloud_ds):
 
 @requires_libdeeplake
 @requires_torch
-def test_pytorch_data_decode(hub_cloud_ds, cat_path):
-    with hub_cloud_ds as ds:
+@pytest.mark.flaky
+@pytest.mark.slow
+def test_pytorch_data_decode(local_auth_ds, cat_path):
+    with local_auth_ds as ds:
         ds.create_tensor("generic")
         for i in range(10):
             ds.generic.append(i)
