@@ -51,21 +51,22 @@ def index_transform(sample):
 
 @requires_tensorflow
 @requires_libdeeplake
-def test_tensorflow_small(hub_cloud_ds):
-    with hub_cloud_ds:
-        hub_cloud_ds.create_tensor("image", max_chunk_size=TF_TESTS_MAX_CHUNK_SIZE)
-        hub_cloud_ds.image.extend(([i * np.ones((i + 1, i + 1)) for i in range(16)]))
-        hub_cloud_ds.commit()
-        hub_cloud_ds.create_tensor("image2", max_chunk_size=TF_TESTS_MAX_CHUNK_SIZE)
-        hub_cloud_ds.image2.extend(np.array([i * np.ones((12, 12)) for i in range(16)]))
+@pytest.mark.slow
+@pytest.mark.flaky
+@pytest.mark.skip("causing lockups")
+def test_tensorflow_small(local_auth_ds):
+    with local_auth_ds as ds:
+        ds.create_tensor("image", max_chunk_size=TF_TESTS_MAX_CHUNK_SIZE)
+        ds.image.extend(([i * np.ones((i + 1, i + 1)) for i in range(16)]))
+        ds.commit()
+        ds.create_tensor("image2", max_chunk_size=TF_TESTS_MAX_CHUNK_SIZE)
+        ds.image2.extend(np.array([i * np.ones((12, 12)) for i in range(16)]))
 
-    if isinstance(
-        get_base_storage(hub_cloud_ds.storage), (MemoryProvider, GCSProvider)
-    ):
+    if isinstance(get_base_storage(ds.storage), (MemoryProvider, GCSProvider)):
         with pytest.raises(ValueError):
-            dl = hub_cloud_ds.dataloader()
+            dl = ds.dataloader()
         return
-    dl = hub_cloud_ds.dataloader().batch(1).tensorflow(num_workers=2)
+    dl = ds.dataloader().batch(1).tensorflow(num_workers=2)
 
     assert len(dl.dataset) == 16
 
@@ -78,7 +79,7 @@ def test_tensorflow_small(hub_cloud_ds):
                 batch["image2"].numpy(), i * np.ones((1, 12, 12))
             )
 
-    sub_ds = hub_cloud_ds[5:]
+    sub_ds = ds[5:]
     sub_dl = sub_ds.dataloader().tensorflow(num_workers=0)
 
     for i, batch in enumerate(sub_dl):
@@ -89,7 +90,7 @@ def test_tensorflow_small(hub_cloud_ds):
             batch["image2"].numpy(), (5 + i) * np.ones((1, 12, 12))
         )
 
-    sub_ds2 = hub_cloud_ds[8:12]
+    sub_ds2 = ds[8:12]
     sub_dl2 = sub_ds2.dataloader().tensorflow(num_workers=0)
 
     for _ in range(2):
@@ -101,7 +102,7 @@ def test_tensorflow_small(hub_cloud_ds):
                 batch["image2"].numpy(), (8 + i) * np.ones((1, 12, 12))
             )
 
-    sub_ds3 = hub_cloud_ds[:5]
+    sub_ds3 = ds[:5]
     sub_dl3 = sub_ds3.dataloader().tensorflow(num_workers=0)
 
     for _ in range(2):
@@ -116,23 +117,24 @@ def test_tensorflow_small(hub_cloud_ds):
 
 @requires_tensorflow
 @requires_libdeeplake
-def test_tensorflow_transform(hub_cloud_ds):
-    with hub_cloud_ds:
-        hub_cloud_ds.create_tensor("image", max_chunk_size=TF_TESTS_MAX_CHUNK_SIZE)
-        hub_cloud_ds.image.extend(([i * np.ones((i + 1, i + 1)) for i in range(16)]))
-        hub_cloud_ds.checkout("alt", create=True)
-        hub_cloud_ds.create_tensor("image2", max_chunk_size=TF_TESTS_MAX_CHUNK_SIZE)
-        hub_cloud_ds.image2.extend(np.array([i * np.ones((12, 12)) for i in range(16)]))
+@pytest.mark.slow
+@pytest.mark.flaky
+@pytest.mark.skip("causing lockups")
+def test_tensorflow_transform(local_auth_ds):
+    with local_auth_ds as ds:
+        ds.create_tensor("image", max_chunk_size=TF_TESTS_MAX_CHUNK_SIZE)
+        ds.image.extend(([i * np.ones((i + 1, i + 1)) for i in range(16)]))
+        ds.checkout("alt", create=True)
+        ds.create_tensor("image2", max_chunk_size=TF_TESTS_MAX_CHUNK_SIZE)
+        ds.image2.extend(np.array([i * np.ones((12, 12)) for i in range(16)]))
 
-    if isinstance(
-        get_base_storage(hub_cloud_ds.storage), (MemoryProvider, GCSProvider)
-    ):
+    if isinstance(get_base_storage(ds.storage), (MemoryProvider, GCSProvider)):
         with pytest.raises(ValueError):
-            dl = hub_cloud_ds.dataloader()
+            dl = ds.dataloader()
         return
 
     dl = (
-        hub_cloud_ds.dataloader()
+        ds.dataloader()
         .batch(1)
         .transform(to_tuple, t1="image", t2="image2")
         .tensorflow(num_workers=2)
@@ -150,27 +152,23 @@ def test_tensorflow_transform(hub_cloud_ds):
 
 @requires_tensorflow
 @requires_libdeeplake
-def test_tensorflow_transform_dict(hub_cloud_ds):
-    with hub_cloud_ds:
-        hub_cloud_ds.create_tensor("image", max_chunk_size=TF_TESTS_MAX_CHUNK_SIZE)
-        hub_cloud_ds.image.extend(([i * np.ones((i + 1, i + 1)) for i in range(16)]))
-        hub_cloud_ds.create_tensor("image2", max_chunk_size=TF_TESTS_MAX_CHUNK_SIZE)
-        hub_cloud_ds.image2.extend(np.array([i * np.ones((12, 12)) for i in range(16)]))
-        hub_cloud_ds.create_tensor("image3", max_chunk_size=TF_TESTS_MAX_CHUNK_SIZE)
-        hub_cloud_ds.image3.extend(np.array([i * np.ones((12, 12)) for i in range(16)]))
+@pytest.mark.slow
+@pytest.mark.flaky
+def test_tensorflow_transform_dict(local_auth_ds):
+    with local_auth_ds as ds:
+        ds.create_tensor("image", max_chunk_size=TF_TESTS_MAX_CHUNK_SIZE)
+        ds.image.extend(([i * np.ones((i + 1, i + 1)) for i in range(16)]))
+        ds.create_tensor("image2", max_chunk_size=TF_TESTS_MAX_CHUNK_SIZE)
+        ds.image2.extend(np.array([i * np.ones((12, 12)) for i in range(16)]))
+        ds.create_tensor("image3", max_chunk_size=TF_TESTS_MAX_CHUNK_SIZE)
+        ds.image3.extend(np.array([i * np.ones((12, 12)) for i in range(16)]))
 
-    if isinstance(
-        get_base_storage(hub_cloud_ds.storage), (MemoryProvider, GCSProvider)
-    ):
+    if isinstance(get_base_storage(ds.storage), (MemoryProvider, GCSProvider)):
         with pytest.raises(ValueError):
-            dl = hub_cloud_ds.dataloader()
+            dl = ds.dataloader()
         return
 
-    dl = (
-        hub_cloud_ds.dataloader()
-        .transform({"image": double, "image2": None})
-        .tensorflow()
-    )
+    dl = ds.dataloader().transform({"image": double, "image2": None}).tensorflow()
 
     assert len(dl.dataset) == 16
 
@@ -194,16 +192,18 @@ def test_tensorflow_transform_dict(hub_cloud_ds):
 
 @requires_tensorflow
 @requires_libdeeplake
-def test_tensorflow_with_compression(hub_cloud_ds: Dataset):
+@pytest.mark.slow
+@pytest.mark.flaky
+def test_tensorflow_with_compression(local_auth_ds: Dataset):
     # TODO: chunk-wise compression for labels (right now they are uncompressed)
-    with hub_cloud_ds:
-        images = hub_cloud_ds.create_tensor(
+    with local_auth_ds as ds:
+        images = ds.create_tensor(
             "images",
             htype="image",
             sample_compression="png",
             max_chunk_size=TF_TESTS_MAX_CHUNK_SIZE,
         )
-        labels = hub_cloud_ds.create_tensor(
+        labels = ds.create_tensor(
             "labels", htype="class_label", max_chunk_size=TF_TESTS_MAX_CHUNK_SIZE
         )
 
@@ -212,14 +212,12 @@ def test_tensorflow_with_compression(hub_cloud_ds: Dataset):
         images.extend(np.ones((16, 12, 12, 3), dtype="uint8"))
         labels.extend(np.ones((16, 1), dtype="uint32"))
 
-    if isinstance(
-        get_base_storage(hub_cloud_ds.storage), (MemoryProvider, GCSProvider)
-    ):
+    if isinstance(get_base_storage(ds.storage), (MemoryProvider, GCSProvider)):
         with pytest.raises(ValueError):
-            dl = hub_cloud_ds.dataloader()
+            dl = ds.dataloader()
         return
 
-    dl = hub_cloud_ds.dataloader().tensorflow(num_workers=0)
+    dl = ds.dataloader().tensorflow(num_workers=0)
 
     for _ in range(2):
         for batch in dl:
@@ -231,26 +229,24 @@ def test_tensorflow_with_compression(hub_cloud_ds: Dataset):
 
 @requires_tensorflow
 @requires_libdeeplake
-def test_custom_tensor_order(hub_cloud_ds):
-    with hub_cloud_ds:
+@pytest.mark.slow
+@pytest.mark.flaky
+def test_custom_tensor_order(local_auth_ds):
+    with local_auth_ds as ds:
         tensors = ["a", "b", "c", "d"]
         for t in tensors:
-            hub_cloud_ds.create_tensor(t, max_chunk_size=TF_TESTS_MAX_CHUNK_SIZE)
-            hub_cloud_ds[t].extend(np.random.random((3, 4, 5)))
+            ds.create_tensor(t, max_chunk_size=TF_TESTS_MAX_CHUNK_SIZE)
+            ds[t].extend(np.random.random((3, 4, 5)))
 
-    if isinstance(
-        get_base_storage(hub_cloud_ds.storage), (MemoryProvider, GCSProvider)
-    ):
+    if isinstance(get_base_storage(ds.storage), (MemoryProvider, GCSProvider)):
         with pytest.raises(ValueError):
-            dl = hub_cloud_ds.dataloader()
+            dl = ds.dataloader()
         return
 
     with pytest.raises(TensorDoesNotExistError):
-        dl = hub_cloud_ds.dataloader().tensorflow(tensors=["c", "d", "e"])
+        dl = ds.dataloader().tensorflow(tensors=["c", "d", "e"])
 
-    dl = hub_cloud_ds.dataloader().tensorflow(
-        tensors=["c", "d", "a"], return_index=False
-    )
+    dl = ds.dataloader().tensorflow(tensors=["c", "d", "a"], return_index=False)
 
     for i, batch in enumerate(dl):
         c1, d1, a1 = batch
@@ -261,9 +257,9 @@ def test_custom_tensor_order(hub_cloud_ds):
         np.testing.assert_array_equal(a1, a2)
         np.testing.assert_array_equal(c1, c2)
         np.testing.assert_array_equal(d1, d2)
-        np.testing.assert_array_equal(a1[0], hub_cloud_ds.a.numpy()[i])
-        np.testing.assert_array_equal(c1[0], hub_cloud_ds.c.numpy()[i])
-        np.testing.assert_array_equal(d1[0], hub_cloud_ds.d.numpy()[i])
+        np.testing.assert_array_equal(a1[0], ds.a.numpy()[i])
+        np.testing.assert_array_equal(c1[0], ds.c.numpy()[i])
+        np.testing.assert_array_equal(d1[0], ds.d.numpy()[i])
         batch = pickle.loads(pickle.dumps(batch))
         c1, d1, a1 = batch
         a2 = batch["a"]
@@ -272,25 +268,28 @@ def test_custom_tensor_order(hub_cloud_ds):
         np.testing.assert_array_equal(a1, a2)
         np.testing.assert_array_equal(c1, c2)
         np.testing.assert_array_equal(d1, d2)
-        np.testing.assert_array_equal(a1[0], hub_cloud_ds.a.numpy()[i])
-        np.testing.assert_array_equal(c1[0], hub_cloud_ds.c.numpy()[i])
-        np.testing.assert_array_equal(d1[0], hub_cloud_ds.d.numpy()[i])
+        np.testing.assert_array_equal(a1[0], ds.a.numpy()[i])
+        np.testing.assert_array_equal(c1[0], ds.c.numpy()[i])
+        np.testing.assert_array_equal(d1[0], ds.d.numpy()[i])
 
 
 @requires_tensorflow
 @requires_libdeeplake
-def test_readonly_with_two_workers(hub_cloud_ds):
-    hub_cloud_ds.create_tensor("images", max_chunk_size=TF_TESTS_MAX_CHUNK_SIZE)
-    hub_cloud_ds.create_tensor("labels", max_chunk_size=TF_TESTS_MAX_CHUNK_SIZE)
-    hub_cloud_ds.images.extend(np.ones((10, 12, 12)))
-    hub_cloud_ds.labels.extend(np.ones(10))
+@pytest.mark.slow
+@pytest.mark.flaky
+@pytest.mark.skip("causing lockups")
+def test_readonly_with_two_workers(local_auth_ds):
+    local_auth_ds.create_tensor("images", max_chunk_size=TF_TESTS_MAX_CHUNK_SIZE)
+    local_auth_ds.create_tensor("labels", max_chunk_size=TF_TESTS_MAX_CHUNK_SIZE)
+    local_auth_ds.images.extend(np.ones((10, 12, 12)))
+    local_auth_ds.labels.extend(np.ones(10))
 
-    base_storage = get_base_storage(hub_cloud_ds.storage)
+    base_storage = get_base_storage(local_auth_ds.storage)
     base_storage.flush()
     base_storage.enable_readonly()
     ds = Dataset(
-        storage=hub_cloud_ds.storage,
-        token=hub_cloud_ds.token,
+        storage=local_auth_ds.storage,
+        token=local_auth_ds.token,
         read_only=True,
         verbose=False,
     )
@@ -313,23 +312,21 @@ def test_tensorflow_local_cache():
 
 @requires_tensorflow
 @requires_libdeeplake
-def test_groups(hub_cloud_ds, compressed_image_paths):
+@pytest.mark.slow
+@pytest.mark.flaky
+def test_groups(local_auth_ds, compressed_image_paths):
     img1 = deeplake.read(compressed_image_paths["jpeg"][0])
     img2 = deeplake.read(compressed_image_paths["png"][0])
-    with hub_cloud_ds:
-        hub_cloud_ds.create_tensor(
-            "images/jpegs/cats", htype="image", sample_compression="jpeg"
-        )
-        hub_cloud_ds.create_tensor(
-            "images/pngs/flowers", htype="image", sample_compression="png"
-        )
+    with local_auth_ds as ds:
+        ds.create_tensor("images/jpegs/cats", htype="image", sample_compression="jpeg")
+        ds.create_tensor("images/pngs/flowers", htype="image", sample_compression="png")
         for _ in range(10):
-            hub_cloud_ds.images.jpegs.cats.append(img1)
-            hub_cloud_ds.images.pngs.flowers.append(img2)
+            ds.images.jpegs.cats.append(img1)
+            ds.images.pngs.flowers.append(img2)
 
     another_ds = deeplake.dataset(
-        hub_cloud_ds.path,
-        token=hub_cloud_ds.token,
+        ds.path,
+        token=ds.token,
     )
     dl = another_ds.dataloader().tensorflow(return_index=False)
     for i, (cat, flower) in enumerate(dl):
@@ -346,12 +343,14 @@ def test_groups(hub_cloud_ds, compressed_image_paths):
 
 @requires_tensorflow
 @requires_libdeeplake
-def test_string_tensors(hub_cloud_ds):
-    with hub_cloud_ds:
-        hub_cloud_ds.create_tensor("strings", htype="text")
-        hub_cloud_ds.strings.extend([f"string{idx}" for idx in range(5)])
+@pytest.mark.slow
+@pytest.mark.flaky
+def test_string_tensors(local_auth_ds):
+    with local_auth_ds as ds:
+        ds.create_tensor("strings", htype="text")
+        ds.strings.extend([f"string{idx}" for idx in range(5)])
 
-    ptds = hub_cloud_ds.dataloader().tensorflow(collate_fn=identity_collate)
+    ptds = ds.dataloader().tensorflow(collate_fn=identity_collate)
     for idx, batch in enumerate(ptds):
         np.testing.assert_array_equal(batch[0]["strings"], f"string{idx}")
 
@@ -377,12 +376,14 @@ def test_tensorflow_large():
         [4, 3, 2, 1],
     ],
 )
-def test_tensorflow_view(hub_cloud_ds, index):
+@pytest.mark.slow
+@pytest.mark.flaky
+def test_tensorflow_view(local_auth_ds, index):
     arr_list_1 = [np.random.randn(15, 15, i) for i in range(10)]
     arr_list_2 = [np.random.randn(40, 15, 4, i) for i in range(10)]
     label_list = list(range(10))
 
-    with hub_cloud_ds as ds:
+    with local_auth_ds as ds:
         ds.create_tensor("img1")
         ds.create_tensor("img2")
         ds.create_tensor("label")
@@ -390,8 +391,8 @@ def test_tensorflow_view(hub_cloud_ds, index):
         ds.img2.extend(arr_list_2)
         ds.label.extend(label_list)
 
-    ptds = hub_cloud_ds[index].dataloader().tensorflow()
-    idxs = list(IndexEntry(index).indices(len(hub_cloud_ds)))
+    ptds = ds[index].dataloader().tensorflow()
+    idxs = list(IndexEntry(index).indices(len(ds)))
     for idx, batch in enumerate(ptds):
         idx = idxs[idx]
         np.testing.assert_array_equal(batch["img1"][0], arr_list_1[idx])
@@ -402,17 +403,19 @@ def test_tensorflow_view(hub_cloud_ds, index):
 @requires_tensorflow
 @requires_libdeeplake
 @pytest.mark.parametrize("shuffle", [True, False])
-def test_tensorflow_collate(hub_cloud_ds, shuffle):
-    with hub_cloud_ds:
-        hub_cloud_ds.create_tensor("a")
-        hub_cloud_ds.create_tensor("b")
-        hub_cloud_ds.create_tensor("c")
+@pytest.mark.slow
+@pytest.mark.flaky
+def test_tensorflow_collate(local_auth_ds, shuffle):
+    with local_auth_ds as ds:
+        ds.create_tensor("a")
+        ds.create_tensor("b")
+        ds.create_tensor("c")
         for _ in range(100):
-            hub_cloud_ds.a.append(0)
-            hub_cloud_ds.b.append(1)
-            hub_cloud_ds.c.append(2)
+            ds.a.append(0)
+            ds.b.append(1)
+            ds.c.append(2)
 
-    ptds = hub_cloud_ds.dataloader().batch(4).tensorflow(collate_fn=reorder_collate)
+    ptds = ds.dataloader().batch(4).tensorflow(collate_fn=reorder_collate)
     if shuffle:
         ptds = ptds.shuffle()
     for batch in ptds:
@@ -426,18 +429,20 @@ def test_tensorflow_collate(hub_cloud_ds, shuffle):
 @requires_tensorflow
 @requires_libdeeplake
 @pytest.mark.parametrize("shuffle", [True, False])
-def test_tensorflow_transform_collate(hub_cloud_ds, shuffle):
-    with hub_cloud_ds:
-        hub_cloud_ds.create_tensor("a")
-        hub_cloud_ds.create_tensor("b")
-        hub_cloud_ds.create_tensor("c")
+@pytest.mark.slow
+@pytest.mark.flaky
+def test_tensorflow_transform_collate(local_auth_ds, shuffle):
+    with local_auth_ds as ds:
+        local_auth_ds.create_tensor("a")
+        local_auth_ds.create_tensor("b")
+        local_auth_ds.create_tensor("c")
         for _ in range(100):
-            hub_cloud_ds.a.append(0 * np.ones((300, 300)))
-            hub_cloud_ds.b.append(1 * np.ones((300, 300)))
-            hub_cloud_ds.c.append(2 * np.ones((300, 300)))
+            local_auth_ds.a.append(0 * np.ones((300, 300)))
+            local_auth_ds.b.append(1 * np.ones((300, 300)))
+            local_auth_ds.c.append(2 * np.ones((300, 300)))
 
     ptds = (
-        hub_cloud_ds.dataloader()
+        local_auth_ds.dataloader()
         .batch(4)
         .tensorflow(
             collate_fn=my_transform_collate,
@@ -463,6 +468,8 @@ def test_tensorflow_ddp():
 @requires_tensorflow
 @requires_libdeeplake
 @pytest.mark.parametrize("compression", [None, "jpeg"])
+@pytest.mark.slow
+@pytest.mark.flaky
 def test_tensorflow_decode(local_auth_ds, compressed_image_paths, compression):
     with local_auth_ds:
         local_auth_ds.create_tensor("image", sample_compression=compression)
@@ -511,37 +518,46 @@ def test_tensorflow_decode(local_auth_ds, compressed_image_paths, compression):
 
 @requires_tensorflow
 @requires_libdeeplake
-def test_rename(hub_cloud_ds):
-    with hub_cloud_ds as ds:
+@pytest.mark.slow
+@pytest.mark.flaky
+def test_rename(local_auth_ds):
+    tensor_name = "red/green"
+    with local_auth_ds as ds:
         ds.create_tensor("abc")
         ds.create_tensor("blue/green")
         ds.abc.append([1, 2, 3])
         ds.rename_tensor("abc", "xyz")
         ds.rename_group("blue", "red")
-        ds["red/green"].append([1, 2, 3, 4])
+        ds[tensor_name].append([1, 2, 3, 4])
     loader = ds.dataloader().tensorflow(return_index=False)
     for sample in loader:
-        assert set(sample.keys()) == {"xyz", "red/green"}
+        assert set(sample.keys()) == {"xyz", tensor_name}
         np.testing.assert_array_equal(np.array(sample["xyz"]), np.array([[1, 2, 3]]))
         np.testing.assert_array_equal(
-            np.array(sample["red/green"]), np.array([[1, 2, 3, 4]])
+            np.array(sample[tensor_name]), np.array([[1, 2, 3, 4]])
         )
 
 
 @requires_tensorflow
 @requires_libdeeplake
-@pytest.mark.parametrize("num_workers", [0, 2])
-def test_indexes(hub_cloud_ds, num_workers):
+@pytest.mark.parametrize(
+    "num_workers",
+    [
+        0,
+        pytest.param(2, marks=pytest.mark.skip("causing lockups")),
+    ],
+)
+@pytest.mark.slow
+@pytest.mark.flaky
+def test_indexes(local_auth_ds, num_workers):
     shuffle = False
-    with hub_cloud_ds as ds:
+    with local_auth_ds as ds:
         ds.create_tensor("xyz")
         for i in range(8):
             ds.xyz.append(i * np.ones((2, 2)))
 
     ptds = (
-        hub_cloud_ds.dataloader()
-        .batch(4)
-        .tensorflow(num_workers=num_workers, return_index=True)
+        ds.dataloader().batch(4).tensorflow(num_workers=num_workers, return_index=True)
     )
     if shuffle:
         ptds = ptds.shuffle()
@@ -554,16 +570,24 @@ def test_indexes(hub_cloud_ds, num_workers):
 
 @requires_tensorflow
 @requires_libdeeplake
-@pytest.mark.parametrize("num_workers", [0, 2])
-def test_indexes_transform(hub_cloud_ds, num_workers):
+@pytest.mark.parametrize(
+    "num_workers",
+    [
+        0,
+        pytest.param(2, marks=pytest.mark.skip("causing lockups")),
+    ],
+)
+@pytest.mark.slow
+@pytest.mark.flaky
+def test_indexes_transform(local_auth_ds, num_workers):
     shuffle = False
-    with hub_cloud_ds as ds:
+    with local_auth_ds as ds:
         ds.create_tensor("xyz")
         for i in range(8):
             ds.xyz.append(i * np.ones((2, 2)))
 
     ptds = (
-        hub_cloud_ds.dataloader()
+        ds.dataloader()
         .batch(4)
         .transform(index_transform)
         .tensorflow(num_workers=num_workers, return_index=True)
@@ -583,15 +607,17 @@ def test_indexes_transform(hub_cloud_ds, num_workers):
 @requires_tensorflow
 @requires_libdeeplake
 @pytest.mark.parametrize("num_workers", [0, 2])
-def test_indexes_transform_dict(hub_cloud_ds, num_workers):
+@pytest.mark.slow
+@pytest.mark.flaky
+def test_indexes_transform_dict(local_auth_ds, num_workers):
     shuffle = False
-    with hub_cloud_ds as ds:
+    with local_auth_ds as ds:
         ds.create_tensor("xyz")
         for i in range(8):
             ds.xyz.append(i * np.ones((2, 2)))
 
     ptds = (
-        hub_cloud_ds.dataloader()
+        ds.dataloader()
         .batch(4)
         .transform({"xyz": double, "index": None})
         .tensorflow(num_workers=num_workers, return_index=True)
@@ -605,7 +631,7 @@ def test_indexes_transform_dict(hub_cloud_ds, num_workers):
             np.testing.assert_array_equal(2 * batch["index"][i], batch["xyz"][i][0, 0])
 
     ptds = (
-        hub_cloud_ds.dataloader()
+        ds.dataloader()
         .batch(4)
         .transform({"xyz": double})
         .tensorflow(num_workers=num_workers, return_index=True)
@@ -619,17 +645,20 @@ def test_indexes_transform_dict(hub_cloud_ds, num_workers):
 
 @requires_tensorflow
 @requires_libdeeplake
-@pytest.mark.parametrize("num_workers", [0, 2])
-def test_indexes_tensors(hub_cloud_ds, num_workers):
-    shuffle = False
-    with hub_cloud_ds as ds:
+@pytest.mark.parametrize(
+    "num_workers", [0, pytest.param(2, marks=pytest.mark.skip("causing lockups"))]
+)
+@pytest.mark.slow
+@pytest.mark.flaky
+def test_indexes_tensors(local_auth_ds, num_workers):
+    with local_auth_ds as ds:
         ds.create_tensor("xyz")
         for i in range(8):
             ds.xyz.append(i * np.ones((2, 2)))
 
     with pytest.raises(ValueError):
         (
-            hub_cloud_ds.dataloader()
+            ds.dataloader()
             .batch(4)
             .tensorflow(
                 num_workers=num_workers, return_index=True, tensors=["xyz", "index"]
@@ -637,12 +666,10 @@ def test_indexes_tensors(hub_cloud_ds, num_workers):
         )
 
     ptds = (
-        hub_cloud_ds.dataloader()
+        ds.dataloader()
         .batch(4)
         .tensorflow(num_workers=num_workers, return_index=True, tensors=["xyz"])
     )
-    if shuffle:
-        ptds = ptds.shuffle()
 
     for batch in ptds:
         assert batch.keys() == {"xyz", "index"}
@@ -650,6 +677,7 @@ def test_indexes_tensors(hub_cloud_ds, num_workers):
 
 @requires_libdeeplake
 @requires_tensorflow
+@pytest.mark.flaky
 def test_uneven_iteration(local_auth_ds):
     with local_auth_ds as ds:
         ds.create_tensor("x")
@@ -665,6 +693,7 @@ def test_uneven_iteration(local_auth_ds):
 
 @requires_libdeeplake
 @requires_tensorflow
+@pytest.mark.flaky
 def test_tensorflow_error_handling(local_auth_ds):
     with local_auth_ds as ds:
         ds.create_tensor("x")
@@ -674,13 +703,13 @@ def test_tensorflow_error_handling(local_auth_ds):
     ptds = ds.dataloader().tensorflow()
     with pytest.raises(EmptyTensorError):
         for _ in ptds:
-            pass
+            continue
 
     ptds = ds.dataloader().tensorflow(tensors=["x", "y"])
     with pytest.raises(EmptyTensorError):
         for _ in ptds:
-            pass
+            continue
 
     ptds = ds.dataloader().tensorflow(tensors=["x"])
     for _ in ptds:
-        pass
+        continue
