@@ -1,10 +1,9 @@
-from deeplake.core.storage.gcs import GCSProvider
-from deeplake.core.storage.google_drive import GDriveProvider
-from deeplake.util.storage import storage_provider_from_hub_path
-from deeplake.core.storage.s3 import S3Provider
-from deeplake.core.storage.local import LocalProvider
-from deeplake.core.storage.azure import AzureProvider
 import os
+import pickle
+import posixpath
+import pytest
+import sys
+
 import deeplake
 from deeplake.constants import (
     HUB_CLOUD_OPT,
@@ -27,16 +26,29 @@ from deeplake.constants import (
     ENV_GDRIVE_CLIENT_ID,
     ENV_GDRIVE_CLIENT_SECRET,
     ENV_GDRIVE_REFRESH_TOKEN,
+    HUB_CLOUD_DEV_USERNAME,
+    ENV_AZURE_CLIENT_ID,
+    ENV_AZURE_TENANT_ID,
+    ENV_AZURE_CLIENT_SECRET,
+    ENV_AWS_ACCESS_KEY,
+    ENV_AWS_SECRETS_ACCESS_KEY,
+    ENV_AWS_ENDPOINT_URL,
 )
-import posixpath
+from deeplake import VectorStore
+from deeplake.client.client import DeepMemoryBackendClient
+from deeplake.core.storage.gcs import GCSProvider
+from deeplake.core.storage.google_drive import GDriveProvider
+from deeplake.util.storage import storage_provider_from_hub_path
+from deeplake.core.storage.s3 import S3Provider
+from deeplake.core.storage.local import LocalProvider
+from deeplake.core.storage.azure import AzureProvider
+from deeplake.core.vectorstore import utils
 from deeplake.tests.common import (
     SESSION_ID,
     current_test_name,
     get_dummy_data_path,
     is_opt_true,
 )
-import pytest
-import sys
 
 
 MEMORY = "memory"
@@ -47,7 +59,15 @@ GCS = "gcs"
 AZURE = "azure"
 HUB_CLOUD = "hub_cloud"
 
-_GIT_CLONE_CACHE_DIR = ".test_resources"
+REPO_ROOT = os.path.abspath(".")
+while REPO_ROOT != os.path.dirname(REPO_ROOT):
+    if os.path.isfile(os.path.join(REPO_ROOT, "LICENSE")):
+        break
+    REPO_ROOT = os.path.dirname(REPO_ROOT)
+assert REPO_ROOT != "/"
+
+## .test_resources should always be in the root of the repo, regardless of where the tests were ran from
+_GIT_CLONE_CACHE_DIR = os.path.join(REPO_ROOT, ".test_resources")
 
 _HUB_TEST_RESOURCES_URL = "https://www.github.com/activeloopai/hub-test-resources.git"
 _PILLOW_URL = "https://www.github.com/python-pillow/Pillow.git"
@@ -253,7 +273,7 @@ def _get_storage_path(
 @pytest.fixture
 def memory_path(request):
     if is_opt_true(request, MEMORY_OPT):
-        pytest.skip()
+        pytest.skip(f"{MEMORY_OPT} flag not set")
         return
 
     # no need to clear memory paths
@@ -263,7 +283,7 @@ def memory_path(request):
 @pytest.fixture
 def local_path(request):
     if not is_opt_true(request, LOCAL_OPT):
-        pytest.skip()
+        pytest.skip(f"{LOCAL_OPT} flag not set")
         return
 
     path = _get_storage_path(request, LOCAL)
@@ -279,7 +299,7 @@ def local_path(request):
 @pytest.fixture
 def s3_path(request):
     if not is_opt_true(request, S3_OPT):
-        pytest.skip()
+        pytest.skip(f"{S3_OPT} flag not set")
         return
 
     path = _get_storage_path(request, S3)
@@ -292,10 +312,27 @@ def s3_path(request):
         S3Provider(path).clear()
 
 
+@pytest.fixture
+def s3_creds(request):
+    if not is_opt_true(request, S3_OPT):
+        pytest.skip(f"{S3_OPT} flag not set")
+        return
+
+    aws_access_key = os.environ.get(ENV_AWS_ACCESS_KEY)
+    aws_secrets_key = os.environ.get(ENV_AWS_SECRETS_ACCESS_KEY)
+    endpoint_url = os.environ.get(ENV_AWS_ENDPOINT_URL)
+    creds = {
+        "aws_access_key": aws_access_key,
+        "aws_secrets_key": aws_secrets_key,
+        "endpoint_url": endpoint_url,
+    }
+    return creds
+
+
 @pytest.fixture
 def s3_vstream_path(request):
     if not is_opt_true(request, S3_OPT):
-        pytest.skip()
+        pytest.skip(f"{S3_OPT} flag not set")
         return
 
     path = f"{PYTEST_S3_PROVIDER_BASE_ROOT}vstream_test"
@@ -310,23 +347,23 @@ def gcs_creds():
 @pytest.fixture
 def gcs_path(request, gcs_creds):
     if not is_opt_true(request, GCS_OPT):
-        pytest.skip()
+        pytest.skip(f"{GCS_OPT} flag not set")
         return
 
     path = _get_storage_path(request, GCS)
-    GCSProvider(path, token=gcs_creds).clear()
+    GCSProvider(path, creds=gcs_creds).clear()
 
     yield path
 
     # clear storage unless flagged otherwise
     if not is_opt_true(request, KEEP_STORAGE_OPT):
-        GCSProvider(path, token=gcs_creds).clear()
+        GCSProvider(path, creds=gcs_creds).clear()
 
 
 @pytest.fixture
 def gcs_vstream_path(request):
     if not is_opt_true(request, GCS_OPT):
-        pytest.skip()
+        pytest.skip(f"{GCS_OPT} flag not set")
         return
 
     path = f"{PYTEST_GCS_PROVIDER_BASE_ROOT}vstream_test"
@@ -336,7 +373,7 @@ def gcs_vstream_path(request):
 @pytest.fixture
 def azure_path(request):
     if not is_opt_true(request, AZURE_OPT):
-        pytest.skip()
+        pytest.skip(f"{AZURE_OPT} flag not set")
 
     path = _get_storage_path(request, AZURE)
     AzureProvider(path).clear()
@@ -351,7 +388,7 @@ def azure_path(request):
 @pytest.fixture
 def azure_vstream_path(request):
     if not is_opt_true(request, AZURE_OPT):
-        pytest.skip()
+        pytest.skip(f"{AZURE_OPT} flag not set")
 
     path = f"{PYTEST_AZURE_PROVIDER_BASE_ROOT}vstream_test"
     yield path
@@ -370,10 +407,27 @@ def gdrive_creds():
     return creds
 
 
+@pytest.fixture
+def azure_creds(request):
+    if not is_opt_true(request, AZURE_OPT):
+        pytest.skip(f"{AZURE_OPT} flag not set")
+        return
+
+    azure_client_id = os.environ.get(ENV_AZURE_CLIENT_ID)
+    azure_tenant_id = os.environ.get(ENV_AZURE_TENANT_ID)
+    azure_client_secret = os.environ.get(ENV_AZURE_CLIENT_SECRET)
+    creds = {
+        "azure_client_id": azure_client_id,
+        "azure_tenant_id": azure_tenant_id,
+        "azure_client_secret": azure_client_secret,
+    }
+    return creds
+
+
 @pytest.fixture
 def gdrive_path(request, gdrive_creds):
     if not is_opt_true(request, GDRIVE_OPT):
-        pytest.skip()
+        pytest.skip(f"{GDRIVE_OPT} flag not set")
         return
 
     path = _get_storage_path(request, GDRIVE, with_current_test_name=False)
@@ -388,7 +442,7 @@ def gdrive_path(request, gdrive_creds):
 @pytest.fixture
 def hub_cloud_path(request, hub_cloud_dev_token):
     if not is_opt_true(request, HUB_CLOUD_OPT):
-        pytest.skip()
+        pytest.skip(f"{HUB_CLOUD_OPT} flag not set")
         return
 
     path = _get_storage_path(request, HUB_CLOUD)
@@ -410,7 +464,7 @@ def hub_cloud_path(request, hub_cloud_dev_token):
 @pytest.fixture
 def hub_cloud_vstream_path(request, hub_cloud_dev_token):
     if not is_opt_true(request, HUB_CLOUD_OPT):
-        pytest.skip()
+        pytest.skip(f"{HUB_CLOUD_OPT} flag not set")
         return
 
     path = f"{PYTEST_HUB_CLOUD_PROVIDER_BASE_ROOT}vstream_test_dataset"
@@ -418,6 +472,75 @@ def hub_cloud_vstream_path(request, hub_cloud_dev_token):
     yield path
 
 
+@pytest.fixture
+def corpus_query_relevances_copy(request, hub_cloud_dev_token):
+    if not is_opt_true(request, HUB_CLOUD_OPT):
+        pytest.skip(f"{HUB_CLOUD_OPT} flag not set")
+        return
+
+    corpus = _get_storage_path(request, HUB_CLOUD)
+    query_vs = VectorStore(
+        path=f"hub://{HUB_CLOUD_DEV_USERNAME}/deepmemory_test_queries",
+        runtime={"tensor_db": True},
+        token=hub_cloud_dev_token,
+    )
+    queries = query_vs.dataset.text.data()["value"]
+    relevance = query_vs.dataset.metadata.data()["value"]
+    relevance = [rel["relevance"] for rel in relevance]
+
+    deeplake.deepcopy(
+        f"hub://{HUB_CLOUD_DEV_USERNAME}/test-deepmemory10",
+        corpus,
+        token=hub_cloud_dev_token,
+        overwrite=True,
+        runtime={"tensor_db": True},
+    )
+
+    queries_path = corpus + "_eval_queries"
+
+    yield corpus, queries, relevance, queries_path
+
+    delete_if_exists(corpus, hub_cloud_dev_token)
+    delete_if_exists(queries_path, hub_cloud_dev_token)
+
+
+@pytest.fixture
+def deep_memory_local_dataset(request, hub_cloud_dev_token):
+    if not is_opt_true(request, HUB_CLOUD_OPT):
+        pytest.skip(f"{HUB_CLOUD_OPT} flag not set")
+        return
+
+    corpus_path = _get_storage_path(request, LOCAL)
+
+    deeplake.deepcopy(
+        f"hub://{HUB_CLOUD_DEV_USERNAME}/test-deepmemory10",
+        corpus_path,
+        token=hub_cloud_dev_token,
+        overwrite=True,
+    )
+
+    yield corpus_path
+
+    delete_if_exists(corpus_path, hub_cloud_dev_token)
+
+
+def delete_if_exists(path, token):
+    try:
+        deeplake.delete(path, force=True, large_ok=True, token=token)
+    except Exception:
+        pass
+
+
+@pytest.fixture
+def corpus_query_pair_path(hub_cloud_dev_token):
+    corpus = f"hub://{HUB_CLOUD_DEV_USERNAME}/deepmemory_test_corpus_managed_2"
+    query = corpus + "_eval_queries_2"
+    delete_if_exists(query, hub_cloud_dev_token)
+    yield corpus, query
+
+    delete_if_exists(query, hub_cloud_dev_token)
+
+
 @pytest.fixture
 def cat_path():
     """Path to a cat image in the dummy data folder. Expected shape: (900, 900, 3)"""
@@ -468,6 +591,24 @@ def grayscale_image_paths():
     return paths
 
 
+@pytest.fixture
+def lfpw_links():
+    """Mix of working and broken links to images from the LFPW dataset."""
+    BROKEN_LINKS = [
+        "https://cm1.theinsider.com/media/0/428/93/spl41194_011.0.0.0x0.636x912.jpeg",
+        "https://cm1.theinsider.com/media/0/428/93/spl47823_060.0.0.0x0.633x912.jpeg",
+        "https://cm1.theinsider.com/media/0/428/90/spl91520_012.0.0.0x0.636x912.jpeg",
+        "https://blog.themavenreport.com/wp-content/uploads/2008/02/kimora_show_575.jpg",
+        "https://cache.thephoenix.com/secure/uploadedImages/The_Phoenix/Movies/Reviews/FILM_Queen_6.jpg",
+        "https://img2.timeinc.net/people/i/2008/features/theysaid/080331/kimora_lee_simmons400.jpg",
+        "https://img2.timeinc.net/people/i/cbb/2008/04/05/kylieminogue.jpg",
+        "https://i41.tinypic.com/2ih5b7q.png",
+        "https://www.todoelmundo.org/archivos/99/imagenes/En_america.jpg",
+        "https://image.toutlecine.com/photos/b/l/o/blood-diamond-2006-22-g.jpg",
+    ]
+    return BROKEN_LINKS
+
+
 @pytest.fixture(scope="session")
 def mmdet_path():
     return _git_clone_with_branch("dev-2.x", _MMDET_URL)
@@ -525,7 +666,7 @@ def corrupt_image_paths():
 @pytest.fixture
 def audio_paths():
     if sys.platform.startswith("linux") and sys.version_info[:2] == (3, 6):  # FixMe
-        pytest.skip()
+        pytest.skip("Skipping audio tests on linux 3.6")
         return
     paths = {"mp3": "samplemp3.mp3", "flac": "sampleflac.flac", "wav": "samplewav.wav"}
 
@@ -578,6 +719,19 @@ def mesh_paths():
     return paths
 
 
+@pytest.fixture
+def stl_mesh_paths():
+    paths = {
+        "ascii": "box_freecad_ascii.stl",
+        "bin": "box_freecad_binary.stl",
+    }
+
+    parent = get_dummy_data_path("mesh")
+    for k in paths:
+        paths[k] = os.path.join(parent, paths[k])
+    return paths
+
+
 @pytest.fixture
 def vstream_path(request):
     """Used with parametrize to use all video stream test datasets."""
@@ -590,6 +744,18 @@ def path(request):
     return request.getfixturevalue(request.param)
 
 
+@pytest.fixture
+def dest_path(request):
+    """Used with parametrize to get all dataset paths."""
+    return request.getfixturevalue(request.param)
+
+
+@pytest.fixture
+def creds(request):
+    """Used with parametrize to get all dataset creds."""
+    return request.getfixturevalue(request.param)
+
+
 @pytest.fixture
 def hub_token(request):
     """Used with parametrize to get hub_cloud_dev_token if hub-cloud option is True else None"""
@@ -615,24 +781,151 @@ def dataframe_ingestion_data():
 
 @pytest.fixture
 def vector_store_hash_ids(request):
-    return [f"{i}" for i in range(5)]
+    if getattr(request, "param", True):
+        return [f"{i}" for i in range(5)]
 
 
 @pytest.fixture
 def vector_store_row_ids(request):
-    return [i for i in range(5)]
+    if getattr(request, "param", True):
+        return [i for i in range(5)]
 
 
 @pytest.fixture
 def vector_store_filter_udf(request):
-    return "filter_udf"
+    def filter_udf(x):
+        return x["metadata"].data()["value"] == {"a": 1}
+
+    if getattr(request, "param", True):
+        return filter_udf
 
 
 @pytest.fixture
 def vector_store_filters(request):
-    return {"a": 1}
+    if getattr(request, "param", True):
+        return {"metadata": {"a": 1}}
 
 
 @pytest.fixture
 def vector_store_query(request):
-    return "select * where metadata=={'a': 1}"
+    if getattr(request, "param", True):
+        return "select * where metadata['a']==1"
+
+
+@pytest.fixture
+def jobs_list():
+    parent = get_dummy_data_path("deep_memory")
+
+    with open(os.path.join(parent, "jobs_list.txt"), "r") as f:
+        jobs = f.read()
+    return jobs
+
+
+@pytest.fixture
+def questions_embeddings_and_relevances():
+    parent = get_dummy_data_path("deep_memory")
+
+    with open(os.path.join(parent, "questions.pkl"), "rb") as f:
+        questions = pickle.load(f)
+    with open(os.path.join(parent, "questions_embeddings.pkl"), "rb") as f:
+        questions_embeddings = pickle.load(f)
+    with open(os.path.join(parent, "questions_relevances.pkl"), "rb") as f:
+        question_relevances = pickle.load(f)
+    return questions_embeddings, question_relevances, questions
+
+
+@pytest.fixture
+def testing_relevance_query_deepmemory():
+    parent = get_dummy_data_path("deep_memory")
+    with open(os.path.join(parent, "dm_rq.pkl"), "rb") as f:
+        dm_rq = pickle.load(f)
+
+    relevance = dm_rq["relevance"]
+    query = dm_rq["query_embedding"]
+    return relevance, query
+
+
+@pytest.fixture
+def job_id():
+    return "65198efcd28df3238c49a849"
+
+
+@pytest.fixture
+def precomputed_jobs_list():
+    parent = get_dummy_data_path("deep_memory")
+
+    with open(os.path.join(parent, "precomputed_jobs_list.txt"), "r") as f:
+        jobs = f.read()
+    return jobs
+
+
+@pytest.fixture
+def local_dmv2_dataset(request, hub_cloud_dev_token):
+    dmv2_path = f"hub://{HUB_CLOUD_DEV_USERNAME}/dmv2"
+
+    local_cache_path = ".deepmemory_tests_cache/"
+    if not os.path.exists(local_cache_path):
+        os.mkdir(local_cache_path)
+
+    dataset_cache_path = local_cache_path + "dmv2"
+    if not os.path.exists(dataset_cache_path):
+        deeplake.deepcopy(
+            dmv2_path,
+            dataset_cache_path,
+            token=hub_cloud_dev_token,
+            overwrite=True,
+        )
+
+    corpus = _get_storage_path(request, LOCAL)
+
+    deeplake.deepcopy(
+        dataset_cache_path,
+        corpus,
+        token=hub_cloud_dev_token,
+        overwrite=True,
+    )
+    yield corpus
+
+    delete_if_exists(corpus, hub_cloud_dev_token)
+
+
+@pytest.fixture
+def deepmemory_small_dataset_copy(request, hub_cloud_dev_token):
+    dm_path = f"hub://{HUB_CLOUD_DEV_USERNAME}/tiny_dm_dataset"
+    queries_path = f"hub://{HUB_CLOUD_DEV_USERNAME}/queries_vs"
+
+    local_cache_path = ".deepmemory_tests_cache/"
+    if not os.path.exists(local_cache_path):
+        os.mkdir(local_cache_path)
+
+    dataset_cache_path = local_cache_path + "tiny_dm_queries"
+    if not os.path.exists(dataset_cache_path):
+        deeplake.deepcopy(
+            queries_path,
+            dataset_cache_path,
+            token=hub_cloud_dev_token,
+            overwrite=True,
+        )
+
+    corpus = _get_storage_path(request, HUB_CLOUD)
+    query_vs = VectorStore(
+        path=dataset_cache_path,
+    )
+    queries = query_vs.dataset.text.data()["value"]
+    relevance = query_vs.dataset.metadata.data()["value"]
+    relevance = [rel["relevance"] for rel in relevance]
+
+    deeplake.deepcopy(
+        dm_path,
+        corpus,
+        token=hub_cloud_dev_token,
+        overwrite=True,
+        runtime={"tensor_db": True},
+    )
+
+    queries_path = corpus + "_eval_queries"
+
+    yield corpus, queries, relevance, queries_path
+
+    delete_if_exists(corpus, hub_cloud_dev_token)
+    delete_if_exists(queries_path, hub_cloud_dev_token)
