@@ -2,7 +2,6 @@ from deeplake.util.exceptions import (
     ModuleNotInstalledException,
     UnsupportedSchedulerError,
 )
-from deeplake.util.check_installation import ray_installed
 from deeplake.core.compute.provider import ComputeProvider
 
 
@@ -22,15 +21,6 @@ def get_compute_provider(
         from deeplake.core.compute.process import ProcessProvider
 
         compute = ProcessProvider(num_workers)
-    elif scheduler == "ray":
-        if not ray_installed():
-            raise ModuleNotInstalledException(
-                "'ray' should be installed to use ray scheduler."
-            )
-        from deeplake.core.compute.ray import RayProvider
-
-        compute = RayProvider(num_workers)
-
     else:
         raise UnsupportedSchedulerError(scheduler)
     return compute
