@@ -7,6 +7,8 @@ def try_flushing(ds):
         ds.storage.flush()
     except ReadOnlyModeError:
         pass
+    except:
+        pass
 
 
 def map_tensor_keys(dataset, tensor_keys: Optional[Sequence[str]] = None) -> List[str]:
@@ -27,7 +29,7 @@ def map_tensor_keys(dataset, tensor_keys: Optional[Sequence[str]] = None) -> Lis
     return [tensors[k].meta.name or tensors[k].key for k in tensor_keys]
 
 
-_invalid_chars = {"[", "]", "@", ".", ",", "?", "!", "/", "\\", "#", "'", '"'}
+_invalid_chars = {"[", "]", "@", ".", ",", "?", "!", "/", "\\", "#", "'", '"', " "}
 
 
 def sanitize_tensor_name(input: str) -> str:
