@@ -1,8 +1,9 @@
+import requests
 import numpy as np
-import deeplake
-from deeplake.htype import HTYPE_CONFIGURATIONS
 from typing import Any, List, Sequence, Tuple, Optional, Union
 
+import deeplake
+
 
 class ExternalCommandError(Exception):
     def __init__(self, command: str, status: int):
@@ -166,6 +167,11 @@ class ProviderListEmptyError(Exception):
         )
 
 
+class ClassNamesEmptyError(Exception):
+    def __init__(self, tensor_name: str):
+        super().__init__(f"The class_names of the tensor `{tensor_name}` are empty")
+
+
 class DirectoryAtPathException(Exception):
     def __init__(self):
         super().__init__(
@@ -190,22 +196,12 @@ class ModuleNotInstalledException(Exception):
         super().__init__(message)
 
 
-class LoginException(Exception):
-    def __init__(
-        self,
-        message="Error while logging in, invalid auth token. Please try logging in again.",
-    ):
-        super().__init__(message)
-
-
 class UserNotLoggedInException(Exception):
     def __init__(self):
         message = (
-            "You are not logged in and an API token was not found. To complete the operation, you can\n"
-            "1. Login with your username and password using the `activeloop login` CLI command.\n"
-            "2. Create an API token at https://app.activeloop.ai and use it in any of the following ways:\n"
+            "No API token found. To complete the operation, you must "
+            "create an API token at https://app.activeloop.ai and use it in any of the following ways:\n"
             "    - Set the environment variable `ACTIVELOOP_TOKEN` to the token value.\n"
-            "    - Use the CLI command `activeloop login -t <token>`.\n"
             "    - Pass the API token to the `token` parameter of this function.\n"
             "Visit https://docs.activeloop.ai/getting-started/using-activeloop-storage for more information."
         )
@@ -215,7 +211,7 @@ class UserNotLoggedInException(Exception):
 class InvalidHubPathException(Exception):
     def __init__(self, path):
         super().__init__(
-            f"The Dataset's path is an invalid Deep Lake cloud path. It should be of the form hub://username/dataset got {path}."
+            f"The Dataset's path is an invalid Deep Lake cloud path. It should be of the form hub://username/dataset, but got {path}."
         )
 
 
@@ -240,21 +236,14 @@ class AuthenticationException(Exception):
 class AuthorizationException(Exception):
     def __init__(
         self,
-        message="You are not authorized to access this resource on Activeloop Server.",
-        response=None,
+        message: Optional[str] = None,
+        response: Optional[requests.Response] = None,
     ):
+        self.original_message = message
         self.response = response
         super().__init__(message)
 
 
-class InvalidPasswordException(AuthorizationException):
-    def __init__(
-        self,
-        message="The password you provided was invalid.",
-    ):
-        super().__init__(message)
-
-
 class CouldNotCreateNewDatasetException(AuthorizationException):
     def __init__(
         self,
@@ -262,7 +251,7 @@ class CouldNotCreateNewDatasetException(AuthorizationException):
     ):
         extra = ""
         if path.startswith("hub://"):
-            extra = "Since the path is a `hub://` dataset, if you believe you should have write permissions, try running `activeloop login`."
+            extra = "Since the path is a `hub://` dataset, if you believe you should have write permissions, ensure you are using the 'token' parameter or the ACTIVELOOP_TOKEN environment variable."
 
         message = f"Dataset at '{path}' doesn't exist, and you have no permissions to create one there. Maybe a typo? {extra}"
         super().__init__(message)
@@ -327,6 +316,30 @@ class EmptyTokenException(Exception):
         super().__init__(message)
 
 
+class EmptyDeeplakePathException(Exception):
+    def __init__(self, message="The Deeplake dataset path is empty."):
+        super().__init__(message)
+
+
+class ConflictingDatasetParametersError(Exception):
+    """Raised when both deeplake_query and deeplake_view_id are specified simultaneously."""
+
+    def __init__(self):
+        message = (
+            "A query and view_id were specified simultaneously for a dataset in the config. "
+            "Please specify either the deeplake_query or the deeplake_view_id."
+        )
+        super().__init__(message)
+
+
+class MissingTensorMappingError(Exception):
+    """Raised when the required keys 'img' and 'gt_semantic_seg' are missing in the tensor mapping."""
+
+    def __init__(self):
+        message = "Tensor mapping must include the keys 'img' and 'gt_semantic_seg'."
+        super().__init__(message)
+
+
 # TODO Better S3 Exception handling
 
 
@@ -457,6 +470,19 @@ class TensorMetaInvalidHtypeOverwriteValue(MetaError):
         )
 
 
+class MeshTensorMetaMissingRequiredValue(MetaError):
+    def __init__(self, htype: str, key: Union[str, List[str]], compr_list: List[str]):
+        extra = ""
+        if key == "sample_compression":
+            extra = f"Available compressors: {compr_list}"
+
+        if isinstance(key, list):
+            message = f"Htype '{htype}' requires you to specify either one of {key} inside the `create_tensor` method call. {extra}"
+        else:
+            message = f"Htype '{htype}' requires you to specify '{key}' inside the `create_tensor` method call. {extra}"
+        super().__init__(message)
+
+
 class TensorMetaMissingRequiredValue(MetaError):
     def __init__(self, htype: str, key: Union[str, List[str]]):
         extra = ""
@@ -483,7 +509,7 @@ class TensorDtypeMismatchError(MetaError):
 
         # TODO: we may want to raise this error at the API level to determine if the user explicitly overwrote the `dtype` or not. (to make this error message more precise)
         # TODO: because if the user uses `dtype=np.uint8`, but the `htype` the tensor is created with has it's default dtype set as `uint8` also, then this message is ambiguous
-        htype_dtype = HTYPE_CONFIGURATIONS[htype].get("dtype", None)
+        htype_dtype = deeplake.HTYPE_CONFIGURATIONS[htype].get("dtype", None)
         if htype_dtype is not None and htype_dtype == expected:
             msg += f" Htype '{htype}' expects samples to have dtype='{htype_dtype}'."
             super().__init__("")
@@ -548,9 +574,10 @@ def has_path(sample):
 
 
 class TransformError(Exception):
-    def __init__(self, index=None, sample=None, samples_processed=0):
+    def __init__(self, index=None, sample=None, samples_processed=0, suggest=False):
         self.index = index
         self.sample = sample
+        self.suggest = suggest
         # multiprocessing re raises error with str
         if isinstance(index, str):
             super().__init__(index)
@@ -574,6 +601,13 @@ class TransformError(Exception):
                 msg += f" Last checkpoint: {samples_processed} samples processed. You can slice the input to resume from this point."
 
             msg += " See traceback for more details."
+
+            if suggest:
+                msg += (
+                    " If you wish to skip the samples that cause errors,"
+                    " please specify `ignore_errors=True`."
+                )
+
             super().__init__(msg)
 
 
@@ -588,7 +622,7 @@ class SampleAppendError(Exception):
 
             if print_item:
                 msg += str(sample) + " "
-            elif print_path:
+            else:
                 msg += f"at path '{sample.path}' "
         else:
             msg = f"Failed to append a sample "
@@ -598,6 +632,15 @@ class SampleAppendError(Exception):
         super().__init__(msg)
 
 
+class SampleExtendError(Exception):
+    def __init__(self, message):
+        message += (
+            " If you wish to skip the samples that cause errors,"
+            " please specify `ignore_errors=True`."
+        )
+        super().__init__(message)
+
+
 class FilterError(Exception):
     pass
 
@@ -791,21 +834,12 @@ class AgreementNotAcceptedError(AgreementError):
 class NotLoggedInAgreementError(AgreementError):
     def __init__(self):
         super().__init__(
-            "You are not logged in. Please log in to accept the agreement."
+            "This dataset requires acceptance of a user agreement. "
+            "You must set an API token in order to agree to the license terms. "
+            "Visit https://docs.activeloop.ai/getting-started/using-activeloop-storage for more information."
         )
 
 
-class NotLoggedInError(AgreementError):
-    def __init__(self, msg=None):
-        msg = msg or (
-            "This dataset includes an agreement that needs to be accepted before you can use it.\n"
-            "You need to be signed in to accept this agreement.\n"
-            "You can login using 'activeloop login' on the command line if you have an account or using 'activeloop register' if you don't have one."
-        )
-
-        super().__init__(msg)
-
-
 class RenameError(Exception):
     def __init__(self, msg="Only name of the dataset can be different in new path."):
         super().__init__(msg)
@@ -884,6 +918,17 @@ class SampleAppendingError(Exception):
         super().__init__(message)
 
 
+class SampleExtendingError(Exception):
+    def __init__(self):
+        message = (
+            "Cannot extend because tensor(s) are not specified. Expected input to ds.extend is a dictionary. "
+            "To extend tensors, you need to either specify the tensors and add the samples as a dictionary, like: "
+            "`ds.extend({'image_tensor': samples, 'label_tensor': samples})` or you need to call `extend` method of the required tensor, "
+            "like: `ds.image_tensor.extend(samples)`"
+        )
+        super().__init__(message)
+
+
 class DatasetTooLargeToDelete(Exception):
     def __init__(self, ds_path):
         message = f"Deep Lake Dataset {ds_path} was too large to delete. Try again with large_ok=True."
@@ -978,8 +1023,14 @@ class GetChunkError(Exception):
         chunk_key: Optional[str],
         global_index: Optional[int] = None,
         tensor_name: Optional[str] = None,
+        cause: Optional[Exception] = None,
     ):
+        self.root_cause = cause
         self.chunk_key = chunk_key
+
+        if isinstance(cause, GetChunkError):
+            self.root_cause = cause.root_cause
+
         message = "Unable to get chunk"
         if chunk_key is not None:
             message += f" '{chunk_key}'"
@@ -988,6 +1039,14 @@ class GetChunkError(Exception):
         if tensor_name is not None:
             message += f" in tensor {tensor_name}"
         message += "."
+
+        if cause is not None:
+            cause_message = str(cause)
+            if isinstance(cause, KeyError):
+                cause_message = f"The file {cause} does not exist."
+
+            message += f" Root Cause: {cause_message}"
+
         super().__init__(message)
 
 
@@ -1071,3 +1130,66 @@ class IncorrectEmbeddingShapeError(Exception):
             "Please either use different embedding function or exclude invalid "
             "files that are not supported by the embedding function. "
         )
+
+
+class IncompatibleHtypeError(Exception):
+    def __init__(self, msg):
+        super().__init__(msg)
+
+
+class DeepMemoryAccessError(Exception):
+    def __init__(self):
+        msg = (
+            "Deep Memory is not available for organizations on Community plan."
+            "Please, consider upgrading or start a free trial at https://app.activeloop.ai/pricing."
+        )
+        super().__init__(msg)
+
+
+class EmbeddingTensorPopError(Exception):
+    def __init__(self, name, index):
+        super().__init__(
+            f"Can't pop sample from the middle of embedding tensor '{name}'. "
+            f"This operation is not allowed because that will enforce regenerating vector search index. "
+            f"Hint: Swap the sample {index} with the last one and pop the last."
+        )
+
+
+class IncorrectRelevanceTypeError(Exception):
+    def __init__(self):
+        msg = (
+            "Specified data type for relevance isn't correct. "
+            "Please make sure, that relevence is of type List[List[Tuple[str, int]]]"
+        )
+        super().__init__(msg)
+
+
+class IncorrectQueriesTypeError(Exception):
+    def __init__(self):
+        msg = (
+            "Specified data type for queries isn't correct. "
+            "Please make sure, that queries is of type List[str]"
+        )
+        super().__init__(msg)
+
+
+class ValidationDatasetMissingError(Exception):
+    def __init__(self):
+        msg = (
+            "Validation dataset is not specified even though validate = True. "
+            "Please set validate = False or specify a validation dataset."
+        )
+        super().__init__(msg)
+
+
+class InvalidAuthContextError(Exception):
+    def __init__(
+        self,
+        message: str = "Authentication failed due to invalid or insufficient configuration.",
+    ):
+        super().__init__(message)
+
+
+class InvalidPandasDataframeError(Exception):
+    def __init__(self):
+        super().__init__("Source provided is not a valid pandas dataframe object")
