@@ -1,4 +1,5 @@
 import numpy as np
+import deeplake
 from deeplake.util import shuffle
 
 
@@ -7,7 +8,7 @@ def test_shuffle(memory_ds):
     ds.create_tensor("ints", dtype="int64")
     ds.ints.extend(np.arange(10, dtype="int64").reshape((10, 1)))
 
-    np.random.seed(0)
+    deeplake.random.seed(0)
     ds = shuffle(ds)
     expected = [[2], [8], [4], [9], [1], [6], [7], [3], [0], [5]]
     assert ds.ints.numpy().tolist() == expected
