@@ -19,6 +19,7 @@ from deeplake.constants import (
     TRANSFORM_CHUNK_CACHE_SIZE,
 )
 from deeplake.util.dataset import try_flushing
+from deeplake.util.path import relpath
 from deeplake.util.remove_cache import (
     get_base_storage,
     get_dataset_with_zero_size_cache,
@@ -33,18 +34,15 @@ from deeplake.util.exceptions import (
     TensorMismatchError,
     TensorDoesNotExistError,
     TransformError,
+    SampleAppendError,
 )
 
+import traceback
 import posixpath
 import time
 
 import numpy as np
 
-try:
-    import pandas as pd  # type: ignore
-except ImportError:
-    pd = None
-
 
 def transform_sample(
     sample: Any,
@@ -163,6 +161,11 @@ def _handle_transform_error(
     end_input_idx,
     ignore_errors,
 ):
+    try:
+        import pandas as pd  # type: ignore
+    except ImportError:
+        pd = None
+
     start_input_idx = transform_dataset.start_input_idx
     skipped_samples = 0
     for i in range(start_input_idx, end_input_idx + 1):
@@ -181,7 +184,9 @@ def _handle_transform_error(
             if ignore_errors:
                 skipped_samples += 1
                 continue
-            raise TransformError(offset + i, sample) from e
+            raise TransformError(
+                offset + i, sample, suggest=isinstance(e, SampleAppendError)
+            ) from e
     return skipped_samples
 
 
@@ -196,6 +201,11 @@ def _transform_and_append_data_slice(
     ignore_errors,
 ):
     """Appends a data slice. Returns ``True`` if any samples were appended and ``False`` otherwise."""
+    try:
+        import pandas as pd  # type: ignore
+    except ImportError:
+        pd = None
+
     n = len(data_slice)
     skipped_samples = 0
     skipped_samples_in_current_batch = 0
@@ -227,7 +237,9 @@ def _transform_and_append_data_slice(
                     skipped_samples += 1
                     skipped_samples_in_current_batch += 1
                 else:
-                    raise TransformError(offset + i, sample) from e
+                    raise TransformError(
+                        offset + i, sample, suggest=isinstance(e, SampleAppendError)
+                    ) from e
 
             finally:
                 if i == n - 1:
@@ -382,7 +394,18 @@ def store_data_slice_with_pbar(pg_callback, transform_input: Tuple) -> Dict:
         # retrieve relevant objects from memory
         meta = _retrieve_memory_objects(all_chunk_engines)
         meta.update(ret)
-        meta["error"] = err
+
+        err_dict: Optional[Dict[str, Any]] = None
+        if err:
+            err_dict = {}
+            err_dict["raise"] = err
+            cause = err.__cause__
+            if cause:
+                cause_traceback = "".join(
+                    traceback.format_exception(cause.__class__, cause, cause.__traceback__)  # type: ignore
+                )
+                err_dict["traceback"] = cause_traceback
+        meta["error"] = err_dict
         return meta
 
 
@@ -420,9 +443,11 @@ def create_worker_chunk_engines(
                 tiling_threshold = storage_chunk_engine.tiling_threshold
                 new_tensor_meta = TensorMeta(
                     htype=existing_meta.htype,
-                    dtype=np.dtype(existing_meta.typestr)
-                    if existing_meta.typestr
-                    else existing_meta.dtype,
+                    dtype=(
+                        np.dtype(existing_meta.typestr)
+                        if existing_meta.typestr
+                        else existing_meta.dtype
+                    ),
                     sample_compression=existing_meta.sample_compression,
                     chunk_compression=existing_meta.chunk_compression,
                     max_chunk_size=chunk_size,
@@ -433,6 +458,9 @@ def create_worker_chunk_engines(
                     hidden=existing_meta.hidden,
                     verify=existing_meta.verify,
                 )
+                new_tensor_meta.max_shape = existing_meta.max_shape.copy()
+                new_tensor_meta.min_shape = existing_meta.min_shape.copy()
+                new_tensor_meta.name = existing_meta.name
                 meta_key = get_tensor_meta_key(tensor, version_state["commit_id"])
                 memory_cache[meta_key] = new_tensor_meta  # type: ignore
                 storage_cache.clear_cache()
@@ -521,7 +549,8 @@ def check_transform_ds_out(
         for tensor in tensors:
             if len(ds_out[tensor]) != len(ds_out):
                 raise InvalidOutputDatasetError(
-                    "One or more tensors of the ds_out have different lengths. Transform only supports ds_out having same number of samples for each tensor (This includes empty datasets that have 0 samples per tensor)."
+                    "One or more tensors of the ds_out have different lengths. Transform only supports ds_out having same number of samples for each tensor by default."
+                    " Set `check_lengths=False` in your `.eval(...)` call to disable this check."
                 )
 
     output_base_storage = get_base_storage(ds_out.storage)
