@@ -4,9 +4,12 @@ import hashlib
 import pickle
 from typing import Any, Dict, Optional, List
 import warnings
+
+from deeplake.core.meta.encode.chunk_id import ChunkIdEncoder
+
 from deeplake.client.log import logger
 from deeplake.constants import FIRST_COMMIT_ID
-from deeplake.core import lock
+from deeplake.core import lock, StorageProvider
 from deeplake.core.fast_forwarding import ffw_dataset_meta
 from deeplake.core.meta.dataset_meta import DatasetMeta
 from deeplake.core.storage.deeplake_memory_object import DeepLakeMemoryObject
@@ -40,11 +43,14 @@ from deeplake.util.keys import (
     get_version_control_info_key_old,
     get_version_control_info_lock_key,
     get_commit_info_key,
+    get_pad_encoder_key,
 )
 from deeplake.constants import COMMIT_INFO_FILENAME
+from deeplake.util.path import relpath
 from deeplake.util.remove_cache import get_base_storage
 from deeplake.hooks import dataset_committed
 from datetime import datetime
+import deeplake.core.dataset
 
 import posixpath
 import json
@@ -124,7 +130,7 @@ def integrity_check(dataset):
             num_sequences = getattr(engine.sequence_encoder, "num_samples", None)
             for l, info in t.meta.links.items():
                 l = rev_tensor_names[l]
-                l = posixpath.relpath(l, dataset.group_index)
+                l = relpath(l, dataset.group_index)
                 if num_sequences is not None and not info["flatten_sequence"]:
                     n2 = num_sequences
                 else:
@@ -169,7 +175,7 @@ def commit(
         hash = generate_hash()
     version_state["commit_id"] = hash
     new_node = CommitNode(version_state["branch"], hash)
-    stored_commit_node.add_successor(new_node, message)
+    stored_commit_node.add_successor(new_node, dataset.username, message)
     stored_commit_node.is_checkpoint = is_checkpoint
     stored_commit_node.total_samples_processed = total_samples_processed
     version_state["commit_node"] = new_node
@@ -287,19 +293,125 @@ def checkout(
         ) from e
 
 
+def _squash_main(
+    dataset,
+) -> None:
+    """
+    Combines all commits in the main branch into a single commit.
+    """
+    storage = dataset.storage
+    storage.check_readonly()
+
+    version_state = dataset.version_state
+
+    if len(dataset.branches) > 1:
+        raise VersionControlError(
+            f"Cannot squash commits if there are multiple branches"
+        )
+    if len(dataset.get_views()) > 0:
+        raise VersionControlError(f"Cannot squash commits if there are views present")
+
+    try:
+        base_storage = get_base_storage(storage)
+        versioncontrol_lock = PersistentLock(
+            base_storage, get_version_control_info_lock_key()
+        )
+        versioncontrol_lock.acquire()  # Blocking
+
+        dataset_lock = lock.lock_dataset(dataset, dataset.branches[0])
+
+        for tensor in dataset._tensors(
+            include_hidden=True, include_disabled=True
+        ).values():
+            chunk_engine = tensor.chunk_engine
+            for chunk_id in [row[0] for row in chunk_engine.chunk_id_encoder._encoded]:
+                chunk = chunk_engine.get_chunk_from_chunk_id(chunk_id)
+                if chunk.key.startswith("versions"):
+                    base_storage[
+                        "/".join(
+                            [
+                                tensor.key,
+                                "chunks",
+                                ChunkIdEncoder.name_from_id(chunk_id),
+                            ]
+                        )
+                    ] = chunk.tobytes()
+
+            for key_fn in [
+                get_tensor_info_key,
+                get_tensor_meta_key,
+                get_creds_encoder_key,
+                get_chunk_id_encoder_key,
+                get_pad_encoder_key,
+                get_sequence_encoder_key,
+                get_tensor_tile_encoder_key,
+            ]:
+                try:
+                    data_bytes = storage.get_bytes(
+                        key_fn(chunk_engine.key, dataset.pending_commit_id)
+                    )
+                except KeyError:
+                    continue
+
+                base_storage[key_fn(chunk_engine.key, FIRST_COMMIT_ID)] = data_bytes
+
+        commits_to_delete = [
+            commit_id
+            for commit_id in version_state["commit_node_map"].keys()
+            if commit_id != FIRST_COMMIT_ID
+        ]
+
+        dataset.version_state["commit_node_map"] = {
+            FIRST_COMMIT_ID: dataset.version_state["commit_node_map"][FIRST_COMMIT_ID],
+        }
+        dataset.version_state["commit_node_map"][FIRST_COMMIT_ID].children = []
+        dataset.version_state["commit_node_map"][FIRST_COMMIT_ID].commit_message = None
+        dataset.version_state["commit_node_map"][FIRST_COMMIT_ID].commit_time = None
+        dataset.version_state["commit_node_map"][
+            FIRST_COMMIT_ID
+        ].commit_user_name = None
+
+        dataset.version_state["branch_commit_map"]["main"] = FIRST_COMMIT_ID
+        dataset.version_state["commit_id"] = FIRST_COMMIT_ID
+        dataset.version_state["commit_node"] = dataset.version_state["commit_node_map"][
+            FIRST_COMMIT_ID
+        ]
+
+        base_storage[get_version_control_info_key()] = json.dumps(
+            _version_info_to_json(
+                {
+                    "commit_node_map": version_state["commit_node_map"],
+                    "branch_commit_map": version_state["branch_commit_map"],
+                }
+            )
+        ).encode("utf-8")
+
+        for commit_to_delete in commits_to_delete:
+            delete_version_from_storage(dataset.storage, commit_to_delete)
+
+        dataset._reload_version_state()
+
+        dataset.commit("Squashed commits")
+
+    finally:
+        versioncontrol_lock.release()
+        dataset_lock and dataset_lock.release()
+    #
+    # dataset._send_branch_deletion_event(branch_name)
+
+
 def delete_branch(
     dataset,
     branch_name: str,
 ) -> None:
     """
-    Deletes the branch reference and cleans up any unneeded data.
-    Branches can only be deleted if there are no sub-branches or if the branch has been merged into another branch ever.
+    Deletes the branch and cleans up any unneeded data.
+    Branches can only be deleted if there are no sub-branches and if it has never been merged into another branch.
     """
 
     storage = dataset.storage
     storage.check_readonly()
 
-    # storage = dataset.storage
     version_state = dataset.version_state
     if version_state["branch"] == branch_name:
         raise VersionControlError(
@@ -369,7 +481,7 @@ def delete_branch(
 
 
 def _delete_branch_and_commits(
-    branch_name: str, all_branch_commits: list[str], dataset, storage
+    branch_name: str, all_branch_commits: List[str], dataset, storage
 ) -> None:
     """
     Physically deletes the given branch and list of commits from the version_control_info.json and versions directories.
@@ -401,7 +513,7 @@ def _delete_branch_and_commits(
     ).encode("utf-8")
 
 
-def _find_branch_commits(branch_name: str, version_state: dict) -> list[str]:
+def _find_branch_commits(branch_name: str, version_state: dict) -> List[str]:
     """
     Returns a list of all commits used by the given branch
     """
@@ -591,7 +703,9 @@ def reset_and_checkout(ds, address, err, verbose=True):
 def _merge_commit_node_maps(map1, map2):
     merged_map = {}
 
-    def _merge_node(commit_id):
+    commit_ids = [FIRST_COMMIT_ID]
+    while commit_ids:
+        commit_id = commit_ids.pop()
         if commit_id in map1 and commit_id in map2:
             node1 = map1[commit_id]
             node2 = map2[commit_id]
@@ -605,23 +719,37 @@ def _merge_commit_node_maps(map1, map2):
                 "total_samples_processed",
             ):
                 setattr(merged_node, attr, getattr(node1, attr) or getattr(node2, attr))
-            for child in set(
-                [node.commit_id for node in node1.children]
-                + [node.commit_id for node in node2.children]
-            ):
-                merged_node.add_child(_merge_node(child))
+
+            if node1.parent:
+                assert node1.parent.commit_id == node2.parent.commit_id
+                parent_id = node1.parent.commit_id
+            else:
+                parent_id = None
+
+            commit_ids.extend(
+                set(
+                    [node.commit_id for node in node1.children]
+                    + [node.commit_id for node in node2.children]
+                )
+            )
         else:
             if commit_id in map1:
                 orig_node = map1[commit_id]
             else:
                 orig_node = map2[commit_id]
             merged_node = orig_node.copy()
-            for child in [node.commit_id for node in orig_node.children]:
-                merged_node.add_child(_merge_node(child))
-        merged_map[commit_id] = merged_node
-        return merged_node
 
-    _merge_node(FIRST_COMMIT_ID)
+            if orig_node.parent:
+                parent_id = orig_node.parent.commit_id
+            else:
+                parent_id = None
+
+            commit_ids.extend([node.commit_id for node in orig_node.children])
+
+        if parent_id:
+            parent_node = merged_map[parent_id]
+            parent_node.add_child(merged_node)
+        merged_map[commit_id] = merged_node
     return merged_map
 
 
@@ -709,7 +837,9 @@ def get_parent_and_reset_commit_ids(version_info, address):
     return previous_commit_id, commit_id
 
 
-def _create_new_head(storage, version_state, branch, parent_commit_id, new_commit_id):
+def _create_new_head(
+    storage: LRUCache, version_state, branch, parent_commit_id, new_commit_id
+):
     # populate new commit folder
     copy_metas(parent_commit_id, new_commit_id, storage)
     create_commit_chunk_maps(parent_commit_id, new_commit_id, storage)
@@ -724,7 +854,7 @@ def _create_new_head(storage, version_state, branch, parent_commit_id, new_commi
     return new_node
 
 
-def _replace_head(storage, version_state, commit_id, new_head):
+def _replace_head(storage: LRUCache, version_state, commit_id, new_head):
     parent_node = new_head.parent
     del version_state["commit_node_map"][commit_id]
     for i, child in enumerate(parent_node.children):
@@ -735,13 +865,13 @@ def _replace_head(storage, version_state, commit_id, new_head):
     save_version_info(version_state, storage)
 
 
-def delete_version_from_storage(storage, commit_id):
+def delete_version_from_storage(storage: LRUCache, commit_id: str):
     deletion_folder = "/".join(("versions", commit_id))
     storage.clear(prefix=deletion_folder)
     storage.flush()
 
 
-def replace_head(storage, version_state, reset_commit_id):
+def replace_head(storage: LRUCache, version_state: Dict, reset_commit_id: str):
     """Replace HEAD of current branch with new HEAD"""
     branch = version_state["commit_node_map"][reset_commit_id].branch
     parent_commit_id = version_state["commit_id"]
@@ -758,7 +888,7 @@ def replace_head(storage, version_state, reset_commit_id):
     return new_node.commit_id
 
 
-def _replace_missing_with_head(missing_id, commits, branch_commit_map):
+def _replace_missing_with_head(missing_id: str, commits: Dict, branch_commit_map: Dict):
     new_commit_id = generate_hash()
     branch = None
     parent_commit_id = None
@@ -784,12 +914,12 @@ def _replace_missing_with_head(missing_id, commits, branch_commit_map):
     return branch, parent_commit_id, new_commit_id
 
 
-def rebuild_version_info(storage):
+def rebuild_version_info(storage: LRUCache):
     """Rebuilds version info from commit info."""
-    branch_commit_map = {}
-    commits = {}
+    branch_commit_map: Dict[str, str] = {}
+    commits: Dict[str, Dict] = {}
 
-    # dont do anything if first commit info is missing
+    # don't do anything if first commit info is missing
     try:
         commit_info = load_commit_info(FIRST_COMMIT_ID, storage)
     except Exception:
@@ -962,7 +1092,7 @@ def _get_dataset_meta_at_commit(storage, commit_id):
     return meta
 
 
-def load_meta(dataset):
+def load_meta(dataset: "deeplake.core.dataset.Dataset"):
     """Loads the meta info for the version state."""
     from deeplake.core.tensor import Tensor
 
