@@ -33,5 +33,13 @@ def normalize_index(index, nframes):
 
         if reverse:
             start, stop = stop + 1, start + 1
+    elif isinstance(index, list):
+        raise IndexError(
+            "Cannot specify a list video frames. You must specify a range with an optional step such as [5:10] or [0:100:5]"
+        )
+    else:
+        raise IndexError(
+            f"Invalid video index type: {type(index)}. You must specify either a specific frame index or a range."
+        )
 
     return start, stop, step, reverse
